SystemOrganization addCategory: #'Tweak-DevTools'!

!CWorldPlayer class methodsFor: '*Tweak-DevTools' stamp: 'ar 1/22/2005 14:48'!
compileFixes
	CDebugger class compile: 'primitiveError: aString
	^CWorldPlayer primitiveError: aString'.

	Warning compile: 'defaultAction
	CDebugger
		openContext: thisContext
		label: ''Warning''
		contents: self messageText, ''\\Select Proceed to continue, or close this window to cancel the operation.'' withCRs.
	self resume'.

	UnhandledError compile: 'isDevelopmentEnvironmentPresent
	^ Smalltalk includesKey: #CDebugger'.! !

!CWorldPlayer class methodsFor: '*Tweak-DevTools' stamp: 'ar 2/18/2006 19:30'!
interruptName: labelString
	| preemptedProcess world script |
	preemptedProcess := ScriptScheduler activeScript.
	preemptedProcess ifNil:[^self primitiveError: labelString].
	preemptedProcess suspend offList.
	preemptedProcess hand interrupted.
	world := preemptedProcess hand world.
	script := world startScript:[
		CDebugger openInterrupt: labelString onProcess: preemptedProcess.
	].
	script island: preemptedProcess island.
	script hand: preemptedProcess hand.
	script setScheduler: preemptedProcess scheduler.
	script resume.
! !

!CWorldPlayer class methodsFor: '*Tweak-DevTools' stamp: 'ar 1/29/2005 09:38'!
zapAllButTweak
	| list |
	#(WonderlandConstants References ScriptingSystem ActiveWorld) do:[:sym|
		Smalltalk at: sym ifPresent:[:v| Smalltalk at: sym put: nil].
	].

	self compileFixes.
	PasteUpMorph allInstancesDo:[:each| each removeAllMorphs].

	World := nil.
	Sensor installInterruptWatcher.
	Smalltalk installLowSpaceWatcher.

	"Move some classes to devtools"
	SystemOrganization classify: #MethodReference under: 'Tweak-DevTools'.
	SystemOrganization classify: #FakeClassPool under: 'Tweak-DevTools'.

	"Put some stuff into last removals"
	SystemOrganization classify: #FileList under: 'Last-Removals'.
	SystemOrganization classify: #TheWorldMenu under: 'Last-Removals'.
	SystemOrganization classify: #Flaps under: 'Last-Removals'.

	"Preserve changes"
	#(ChangeRecord ChangeSet ClassChangeRecord MethodChangeRecord) do:[:clsName|
		SystemOrganization classify: clsName under: 'Tweak-Changes'.
	].

	"Preserve morphic events for now"
	SystemOrganization renameCategory: 'Morphic-Events' toBe: 'Tweak-Morphic-Events'.

	"Preserve canvas hierarchy for now"
	SystemOrganization renameCategory: 'Morphic-Support' toBe: 'Tweak-Morphic-Support'.

	SystemOrganization classify: #TextAnchor under: 'Collections-Text'.
	SystemOrganization classify: #TextLine under: 'Tweak-Morphic-Support'.


	"Go zap all morph subclasses - written this way to catch classes from other categories too"
	Morph withAllSubclassesDo: [:aClass| 
		Transcript cr; show: aClass name.
		self waitTick.
		aClass removeFromSystem: true].

	TestCase withAllSubclassesDo: [:aClass| 
		Transcript cr; show: aClass name.
		self waitTick.
		aClass removeFromSystem: true].

	"Zap all..."
	#(
		'Morphic-*'			"... of Morphic ..."
		'ST80-*'				"... of MVC ..."
		'Kernel-ST80 Remnants'	".... its remnants ..."
		'Tools-*'				"... the Morphic/MVC tools..."
		'Genie-*'			"... Genie (not supported right now) ..."
		'StarSqueak-*'		"... for the moment (and for the heck of it!!)..."
		'Protocols-*'			"... etc. ..."

		'Speech-*'
		'Network-*'
		'Graphics-Tools-*'
		'Graphics-External*'
		'Balloon-MMFlash*'
		'Balloon-Fills*'
		'VMMaker-*'
		'Balloon3D-*'
		'Sound-*'
		'XML-*'
		'FFI-*'
		'Movies-*'
		'SUnit*'
		'SM-*'
		"---"
		'System-Benchmarking'
		'EToy-UserObjects'
		'Squeak-EToy-Download'
	) do:[:pattern|
		(SystemOrganization categoriesMatching: pattern) do:[:catName|
			(SystemOrganization listAtCategoryNamed: catName) do:[:each|
				Transcript cr; show: 'Removing ', each.
				self waitTick.
				Smalltalk removeClassNamed: each.
			].
		].
	].
	SystemOrganization removeEmptyCategories.

	list := ExternalSettings registeredClients.
	[list size > 0] whileTrue:[list remove: list anyOne].

	FileList classPool at: #FileReaderRegistry put: OrderedCollection new.
	TheWorldMenu classPool at: #OpenMenuRegistry put: OrderedCollection new.
	CChangeSorter classPool at: #AllChangeSets put: OrderedCollection new.
	CChangeSorter classPool at: #ChangeSetCategories put: nil.

	Smalltalk removeClassNamed: #Project.
	Smalltalk removeClassNamed: #FileList.
	Smalltalk removeClassNamed: #TheWorldMenu.
	Smalltalk removeClassNamed: #Flaps.

	Preferences classPool at: #DictionaryOfPreferences put: IdentityDictionary new.
	Preferences classPool at: #Parameters put: IdentityDictionary new.

	Utilities classPool at: #CommonRequestStrings put: nil.
	Utilities classPool at: #RecentSubmissions put: OrderedCollection new.
	Utilities classPool at: #ScrapsBook put: nil.

	DataStream initialize.
! !

CTreeItem subclass: #CDebuggerTreeItem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CDebuggerTreeItem defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CDebuggerTreeItem methodsFor: 'accessing' stamp: 'ar 11/12/2004 16:42'!
addChild: treeItem
	children := children copyWith: treeItem.! !

!CDebuggerTreeItem methodsFor: 'accessing' stamp: 'ar 11/12/2004 16:42'!
getChildren
	^children! !

!CDebuggerTreeItem methodsFor: 'accessing' stamp: 'ar 11/12/2004 16:42'!
hasChildren
	^false! !

CPanel subclass: #CCodeHolder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CCodeHolder subclass: #CAnnotationsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CAnnotationsBrowser commentStamp: 'tk 2/28/2005 14:18' prior: 0!
An Annotations Browser allows the user to find Tweak code methods that are annotated.  People who use scripts to make things happen, do not need to know about annotations.
	An annotation is a part of a method that allows the method to start up in response to a signal from anywhere else in the system.  The signaling method does not have to know who is listening to the signal.  Annotations apper in angle brackets in methods.  The most common are <on: xxx>  <on:xxx in: anObject> and <bewareOf: xxx>.  There are other uses for annotation that do not involve signals.  Get an Annotations Browser from the Tools menu.[general]


### Allow Users to see the above explanation ###!

CAnnotationsBrowser defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="annotations" toSet="annotations:" changeEvent="annotationsChanged">annotations</field>
	<field toGet="propertyList" toSet="propertyList:" changeEvent="propertyListChanged">propertyList</field>
	<field toGet="selectedProperty" toSet="selectedProperty:" changeEvent="selectedPropertyChanged">selectedProperty</field>
	<field toGet="valueList" toSet="valueList:" changeEvent="valueListChanged">valueList</field>
	<field toGet="selectedValue" toSet="selectedValue:" changeEvent="selectedValueChanged">selectedValue</field>
	<field toGet="methodList" toSet="methodList:" changeEvent="methodListChanged">methodList</field>
	<field toGet="selectedMethod" toSet="selectedMethod:" changeEvent="selectedMethodChanged">selectedMethod</field>
</fields>'!

!CAnnotationsBrowser class methodsFor: 'class initialization' stamp: 'ar 4/14/2005 22:36'!
initialize
	"doIt: [self initialize]"
	self registerOpenCommand.! !

!CAnnotationsBrowser class methodsFor: 'class initialization' stamp: 'ar 4/14/2005 22:36'!
registerOpenCommand
	"self initialize"
	Smalltalk at: #CProjectBuilder ifPresent:[:pb|
		pb registerOpenCommand: {
			'Annotations Browser'.
			{ self. #open. }.
			'A Method Annotations Browser'. }.
	].! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
annotations
	"Answer the annotations of the receiver"
	<bewareOf: #annotationsChanged>
	^self propertyValueAt: #annotations! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
annotations: aValue
	"Modify the receiver's annotations"
	^self propertyValueAt: #annotations put: aValue with: #annotationsChanged! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:12'!
browseMethodFull

	selectedMethod ifNotNil: [
		CClassBrowser new
			selectClass: selectedMethod actualClass selector: selectedMethod methodSymbol;
			open].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:37'!
browseMethodHierarchy

	| aClass selector aClassNonMeta isMeta list theClassOrMeta |
	selectedMethod ifNotNil: [
		aClass := selectedMethod actualClass.
		selector := selectedMethod methodSymbol.
		aClassNonMeta := aClass theNonMetaClass.
		isMeta := aClassNonMeta ~~ aClass.
		list := OrderedCollection new.
		aClass allSuperclasses reverseDo: [:cl |
			(cl includesSelector: selector) ifTrue: [
				list addLast: (MethodReference new
					setStandardClass: cl methodSymbol: selector)]].
		aClassNonMeta
			allSubclassesWithLevelDo: [:cl :level |
				theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
				(theClassOrMeta includesSelector: selector) ifTrue: [
					list addLast: (MethodReference new 
						setStandardClass: theClassOrMeta methodSymbol: selector)]]
	 		startingLevel: 0.
	self browseMessageList: list
		name: 'Inheritance of ' , selector
		autoSelect: nil].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:42'!
browseMethodVersions

	self cNotYet.! !

!CAnnotationsBrowser methodsFor: 'code pane' stamp: 'rhi 1/15/2005 21:36'!
codePaneContents

	^ selectedMethod
		ifNil: [String new]
		ifNotNil: [selectedMethod sourceCode asText
			makeSelectorBoldIn: (selectedMethod actualClass ifNil:[self])]! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/16/2005 00:06'!
confirmRemovalOf: aSelector in: aClass
	"Copied from CClassBrowser>>confirmRemovalOf:in:!!"

	| count answer allCalls aMenu |
	allCalls := self allCallsOn: aSelector.
	count := allCalls size.
	count = 0 ifTrue: [^ true].
	count == 1 ifTrue: [
		(allCalls first actualClass == aClass 
			and: [allCalls first methodSymbol == aSelector])
				ifTrue: [^ true]].
	aMenu := CMenu new.
	aMenu label: 'This message has ', count, ' sender(s)'.
	answer := aMenu chooseFrom: #(
		('Remove it' 1)
		('Remove, then browse senders' 2)
		('Don''t remove, but show me those senders' 3)
		('Forget it -- do nothing -- sorry I asked' 4)).
	answer = 1 ifTrue: [^ true].
	answer = 2 ifTrue: [^ nil].
	answer = 3 ifTrue: [self browseMessageList: allCalls
		name: 'Senders of ' , aSelector autoSelect: aSelector keywords first].
	^ false! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:20'!
copySelector

	selectedMethod ifNotNil: [
		Clipboard clipboardText: selectedMethod methodSymbol asString].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:16'!
fileOutMessage

	selectedMethod ifNotNil: [
		Cursor write showWhile: [
			selectedMethod actualClass fileOutMethod: selectedMethod methodSymbol]].! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 22:13'!
getMethodList

	| list |
	list := Set new.
	selectedProperty
		ifNil: [annotations do: [:each | list addAll: each methods]]
		ifNotNil: [
			selectedValue
				ifNil: [list addAll: selectedProperty methods]
				ifNotNil: [selectedProperty methods do: [:mref |
					((mref actualClass compiledMethodAt: mref methodSymbol)
						valueOfProperty: selectedProperty property) = selectedValue
							ifTrue: [list add: mref]]]].
	^ list asSortedCollection: [:a :b | a stringVersion <= b stringVersion]! !

!CAnnotationsBrowser methodsFor: 'property list' stamp: 'rhi 1/15/2005 22:38'!
getPropertyList

	^ annotations! !

!CAnnotationsBrowser methodsFor: 'value list' stamp: 'rhi 1/15/2005 22:35'!
getValueList

	^ selectedProperty
		ifNil: [Array new]
		ifNotNil: [selectedProperty values
			asSortedCollection: [:a :b | a printString <= b printString]]! !

!CAnnotationsBrowser methodsFor: 'initialize' stamp: 'rhi 1/15/2005 00:31'!
initAnnotations

	| annos |
	annos := Set new.
	self systemNavigation allBehaviorsDo: [:beh |
		beh methodDict keysAndValuesDo: [:sel :cm |
			| props |
			(props := cm properties) ifNotNil: [
				props associationsDo: [:ass |
					| prop |
					prop := (annos
						detect: [:anno | anno property == ass key]
						ifNone: [annos add: (CAnnotationsProperty new
							property: ass key)]).
					prop values add: ass value.
					prop methods add: (MethodReference class: beh selector: sel)]]]].
	annotations := annos asSortedCollection: [:a :b | a property <= b property].! !

!CAnnotationsBrowser methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:56'!
initialize

	super initialize.

	(self define: #propertyList as: CStringList new)
		allowDeselect: true.

	(self define: #valueList as: CStringList new)
		allowDeselect: true.

	(self define: #methodList as: CStringList new)
		menu: self methodListMenu;
		userDataAt: #keyMap put: self methodListKeyMap;
		allowDeselect: true.

	self initAnnotations.
	self flag: #rhi. "Das hier ist nicht so doll. Muesste eigentlich via events funktionieren!!"
	propertyList contents: annotations.! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 18:53'!
methodList
	"Answer the methodList of the receiver"
	<bewareOf: #methodListChanged>
	^self propertyValueAt: #methodList! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 18:53'!
methodList: aValue
	"Modify the receiver's methodList"
	^self propertyValueAt: #methodList put: aValue with: #methodListChanged! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/14/2005 20:46'!
methodListKeyMap
	"Copied from CClassBrowser>>messageListKeyMap!!"

	^ CKeyboardDispatchTable new
		addAll: #(
			((ctrl b)			browseMethodFull)
			((ctrl h)		browseClassHierarchy)
			((ctrl p)			browseFullProtocol)
			((ctrl o)			fileOutMessage)
			((cmd c)		copySelector)
			((cmd n)		browseSenders)
			((cmd m)		browseAllMessages)
			((ctrl i)			browseMethodHierary)
			((ctrl shift n)	browseClassRefs)
			((ctrl x)			removeMessage));
		yourself! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/14/2005 19:58'!
methodListMenu
	"Copied from CClassBrowser>>messageListMenu!!"

	| aMenu |
	aMenu := CMenu new.
	aMenu label: 'Messages'.
	aMenu
		addList: #(
			('browse full (b)' 			browseMethodFull)
			('browse hierarchy (h)'		classHierarchy)
			('browse method (O)'		openSingleMessageBrowser)
			('browse protocol (p)'		browseFullProtocol)
			-
			('fileOut (o)'				fileOutMessage)
			('printOut'					printOutMessage)
			('copy selector (c)'			copySelector)
			-
			('senders of... (n)'			browseSenders)
			('implementors of... (m)'		browseImplementors)
			('inheritance (i)'			browseMethodHierarchy)
			('versions (v)'				browseMethodVersions)
			-
			('inst var refs...'			browseInstVarRefs)
			('inst var defs...'			browseInstVarDefs)
			('class var refs...'			browseClassVarRefs)
			('class refs (N)'				browseClassRefs)
			-
			('remove method (x)'		removeMessage))
		target: self.
	^ aMenu! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 21:59'!
onAnnotationsChanged4MethodList

	<on: annotationsChanged>

	self onSelectedPropertyChanged4MethodList.! !

!CAnnotationsBrowser methodsFor: 'property list' stamp: 'rhi 1/15/2005 22:38'!
onAnnotationsChanged4PropertyList

	<on: annotationsChanged>

	propertyList contents: self getPropertyList.! !

!CAnnotationsBrowser methodsFor: 'value list' stamp: 'rhi 1/15/2005 22:48'!
onChangedInPropertyList4ValueList

	<on: changed in: propertyList>

	self onSelectedPropertyChanged4ValueList.! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 22:31'!
onChangedInValueList4MethodList

	<on: changed in: valueList>

	self onSelectedValueChanged4MethodList.! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 22:30'!
onCursorChangedInMethodList4MethodList

	<on: cursorChanged in: methodList>

	(self okToChangeList: methodList from: self selectedMethod)
		ifTrue: [selectedMethod := methodList selection].! !

!CAnnotationsBrowser methodsFor: 'property list' stamp: 'rhi 1/15/2005 22:23'!
onCursorChangedInPropertyList4PropertyList

	<on: cursorChanged in: propertyList>

	(self okToChangeList: propertyList from: selectedProperty)
		ifTrue: [selectedProperty := propertyList selection].! !

!CAnnotationsBrowser methodsFor: 'value list' stamp: 'rhi 1/15/2005 22:27'!
onCursorChangedInValueList4ValueList

	<on: cursorChanged in: valueList>

	(self okToChangeList: valueList from: selectedValue)
		ifTrue: [selectedValue := valueList selection].! !

!CAnnotationsBrowser methodsFor: 'annotations' stamp: 'rhi 1/16/2005 11:05'!
onPropertiesChangedInCompiledMethod

	<on: propertiesChanged in: CompiledMethod>

	self initAnnotations.! !

!CAnnotationsBrowser methodsFor: 'code pane' stamp: 'rhi 1/15/2005 00:50'!
onSelectedMethodChanged

	<on: selectedMethodChanged>

	self signal: #updateCodePane.! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 21:54'!
onSelectedPropertyChanged4MethodList

	<on: selectedPropertyChanged>

	self onSelectedValueChanged4MethodList.! !

!CAnnotationsBrowser methodsFor: 'value list' stamp: 'rhi 1/15/2005 22:41'!
onSelectedPropertyChanged4ValueList

	<on: selectedPropertyChanged>

	valueList contents: self getValueList.! !

!CAnnotationsBrowser methodsFor: 'method list' stamp: 'rhi 1/15/2005 22:10'!
onSelectedValueChanged4MethodList

	<on: selectedValueChanged>

	methodList contents: self getMethodList.! !

!CAnnotationsBrowser methodsFor: 'code pane' stamp: 'rhi 1/15/2005 00:51'!
onUpdateCodePane

	<on: updateCodePane>

	codePane contents: self codePaneContents.! !

!CAnnotationsBrowser methodsFor: 'annotations' stamp: 'ar 2/9/2005 14:13'!
onWindowCloseRequest

	<on: closeRequest>

	self okToChange ifTrue: [self destroy].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:11'!
openSingleMessageBrowser

	selectedMethod ifNotNil: [
		self browseMessageList: { selectedMethod. }
			name: selectedMethod asStringOrText
			autoSelect: nil].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:40'!
printOutMessage

	selectedMethod ifNotNil: [
		Cursor write showWhile: [
			selectedMethod actualClass
				fileOutMethod: selectedMethod methodSymbol asHtml: true]].! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 20:41'!
propertyList
	"Answer the propertyList of the receiver"
	<bewareOf: #propertyListChanged>
	^self propertyValueAt: #propertyList! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 20:41'!
propertyList: aValue
	"Modify the receiver's propertyList"
	^self propertyValueAt: #propertyList put: aValue with: #propertyListChanged! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/16/2005 00:01'!
removeMessage

	| messageName removal |
	selectedMethod ifNotNil: [
		messageName := selectedMethod methodSymbol.
		self okToChange ifFalse: [^ self].
		removal := self confirmRemovalOf: messageName in: self selectedClassOrMetaClass.
		removal ifFalse: [^ self].
		self selectedClassOrMetaClass removeSelector: messageName.
		removal ifNil: [self browseAllCallsOn: messageName].
		self onSelectedValueChanged4MethodList].! !

!CAnnotationsBrowser methodsFor: 'method list actions' stamp: 'rhi 1/15/2005 23:27'!
selectMessage: label andEvaluate: aBlock

	| selector method messages msgMenu result |
	selectedMethod ifNotNil: [
		selector := selectedMethod methodSymbol.
		method := selectedMethod compiledMethod.
		messages := method messages copyWithout: selector.
		messages isEmpty ifTrue: [^ aBlock value: selector].
		msgMenu := CMenu new.
		msgMenu label: (label ifNil: ['Choose message']).
		result := msgMenu chooseFromValues: messages default: selector.
		result ifNotNil: [aBlock value: result]].! !

!CAnnotationsBrowser methodsFor: 'private' stamp: 'rhi 1/15/2005 23:49'!
selectedClass

	^ selectedMethod ifNotNil: [selectedMethod actualClass]! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedMethod
	"Answer the selectedMethod of the receiver"
	<bewareOf: #selectedMethodChanged>
	^self propertyValueAt: #selectedMethod! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedMethod: aValue
	"Modify the receiver's selectedMethod"
	^self propertyValueAt: #selectedMethod put: aValue with: #selectedMethodChanged! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedProperty
	"Answer the selectedProperty of the receiver"
	<bewareOf: #selectedPropertyChanged>
	^self propertyValueAt: #selectedProperty! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedProperty: aValue
	"Modify the receiver's selectedProperty"
	^self propertyValueAt: #selectedProperty put: aValue with: #selectedPropertyChanged! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedValue
	"Answer the selectedValue of the receiver"
	<bewareOf: #selectedValueChanged>
	^self propertyValueAt: #selectedValue! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 11:36'!
selectedValue: aValue
	"Modify the receiver's selectedValue"
	^self propertyValueAt: #selectedValue put: aValue with: #selectedValueChanged! !

!CAnnotationsBrowser methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:55'!
setupCostume

	super setupCostume.

	self extent: 550@350.

	propertyList
		color: Color transparent;
		layoutFrame: (0@0 corner: 0.33@0.45).
	self add: propertyList.

	valueList
		color: Color transparent;
		layoutFrame: (0.33@0 corner: 0.66@0.45).
	self add: valueList.

	methodList
		color: Color transparent;
		layoutFrame: (0.66@0 corner: 1@0.45).
	self add: methodList.

	codePane layoutFrame: (0@0.45 corner: 1@1).

	elements do: [:each | each
		borderWidth: 2;
		borderStyle: #complexAltInset;
		borderColor: Color lightGray;
		windowColor: Color lightGray].
	self color:  Color lightGray.

	self flag: #rhi. "Muss das wirklich sein?"
	self signal: #annotationsChanged.! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 18:53'!
valueList
	"Answer the valueList of the receiver"
	<bewareOf: #valueListChanged>
	^self propertyValueAt: #valueList! !

!CAnnotationsBrowser methodsFor: 'accessing' stamp: 'rhi 1/14/2005 18:53'!
valueList: aValue
	"Modify the receiver's valueList"
	^self propertyValueAt: #valueList put: aValue with: #valueListChanged! !

CCodeHolder subclass: #CChangeSorter
	instanceVariableNames: ''
	classVariableNames: 'AllChangeSets ChangeSetCategories ChangeSetNamesInRelease PreviousSet RecentUpdateMarker'
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CChangeSorter commentStamp: 'tk 2/28/2005 14:17' prior: 0!
A Change Sorter shows changeSets.  A user who makes object do things by building scripts will not need a Change Sorter.
	A changeSet is a group of method that have been edited.  It holds the methods that are needed make a single coordinated change to Tweak.  Get a Change Sorter from the Tools menu of the menu bar.[general]


### Allow Users to see the above explanation ###!

CChangeSorter defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="parent" toSet="parent:" changeEvent="parentChanged">parent</field>
	<field toGet="myChangeSet" toSet="myChangeSet:" changeEvent="myChangeSetChanged">myChangeSet</field>
	<field toGet="priorChangeSetList" toSet="priorChangeSetList:" changeEvent="priorChangeSetListChanged">priorChangeSetList</field>
	<field toGet="changeSetCategory" toSet="changeSetCategory:" changeEvent="changeSetCategoryChanged">changeSetCategory</field>
	<field toGet="classListPane" toSet="classListPane:" changeEvent="classListPaneChanged">classListPane</field>
	<field toGet="methodListPane" toSet="methodListPane:" changeEvent="methodListPaneChanged">methodListPane</field>
	<field toGet="changeSetListPane" toSet="changeSetListPane:" changeEvent="changeSetListPaneChanged">changeSetListPane</field>
	<field toGet="selectedMessage" toSet="selectedMessage:" changeEvent="selectedMessageChanged">selectedMessage</field>
	<field toGet="selectedChangeSetName" toSet="selectedChangeSetName:" changeEvent="selectedChangeSetNameChanged">selectedChangeSetName</field>
	<field toGet="selectedClassName" toSet="selectedClassName:" changeEvent="selectedClassNameChanged">selectedClassName</field>
</fields>'!

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
allChangeSetNames
	^ self allChangeSets collect: [:c | c name]! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
allChangeSets
	"Return the list of all current ChangeSets"

	^ AllChangeSets! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
allChangeSetsWithClass: class selector: selector
	class ifNil: [^ #()].
	^ self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none]! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
assuredChangeSetNamed: aName
	"Answer a change set of the given name.  If one already exists, answer that, else create a new one and answer it."

	| existing |
	^ (existing := self changeSetNamed: aName)
		ifNotNil:
			[existing]
		ifNil:
			[self basicNewChangeSet: aName]! !

!CChangeSorter class methodsFor: 'adding' stamp: 'ar 1/5/2005 18:03'!
basicNewChangeSet: newName
	| newSet |
	newName ifNil: [^ nil].
	(self changeSetNamed: newName) ifNotNil:
		[self inform: 'Sorry that name is already used'.
		^ nil].
	newSet := ChangeSet basicNewNamed: newName.
	AllChangeSets add: newSet.
	^ newSet! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInAdditions: aChangeSet
	"Answer whether a change set belongs in the Additions category, which is fed by all change sets that are neither numbered nor in the initial release"

	^ (((self belongsInProjectsInRelease: aChangeSet) or:
		[self belongsInNumbered: aChangeSet])) not! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInAll: aChangeSet
	"Answer whether a change set belongs in the All category"

	^ true ! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInMyInitials: aChangeSet
	"Answer whether a change set belongs in the MyInitials category. "

	^ aChangeSet name endsWith: ('-', Utilities authorInitials)! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInNumbered:  aChangeSet
	"Answer whether a change set belongs in the Numbered category. "

	^  aChangeSet name startsWithDigit! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInProjectChangeSets: aChangeSet
	"Answer whether a change set belongs in the MyInitials category. "

	^ aChangeSet belongsToAProject! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInProjectsInRelease:  aChangeSet
	"Answer whether a change set belongs in the ProjectsInRelease category.  You can hand-tweak this to suit your working style.  This just covers the space of project names in the 2.9, 3.0, and 3.1a systems"

	| aString |
	^ ((aString := aChangeSet name) beginsWith: 'Play With Me') or: [self changeSetNamesInReleaseImage includes: aString]! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
belongsInRecentUpdates: aChangeSet
	"Answer whether a change set belongs in the RecentUpdates category."

	^ aChangeSet name startsWithDigit and:
			[aChangeSet name asInteger >= self recentUpdateMarker]! !

!CChangeSorter class methodsFor: 'services' stamp: 'ar 1/5/2005 18:03'!
browseChangeSetsWithClass: class selector: selector
	"Put up a menu comprising a list of change sets that hold changes for the given class and selector.  If the user selects one, open a single change-sorter onto it"

	| hits selected |
	hits := self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ self inform: class name, '.', selector , '
is not in any change set'].
	selected := hits size == 1
		ifTrue:	[hits first]
		ifFalse:	[ (CMenu new) chooseFrom:  
				(hits collect: [:cs | Array with: cs name with: cs ])].
	(CChangeSorter new myChangeSet: selected) open.
! !

!CChangeSorter class methodsFor: 'services' stamp: 'ar 1/5/2005 18:03'!
browseChangeSetsWithSelector: aSelector
	"Put up a list of all change sets that contain an addition, deletion, or change of any method with the given selector"

	| hits selected |
	hits := self allChangeSets select: 
		[:cs | cs hasAnyChangeForSelector: aSelector].
	hits isEmpty ifTrue: [^ self inform: aSelector , '\is not in any change set' withCRs].
	selected := hits size == 1
		ifTrue:	[hits first]
		ifFalse:	[CMenu new 
							chooseFromValues: (hits collect: [:cs | cs name]);
							run].
	self new cNotYet.
	"(ChangeSetBrowser new myChangeSet: selected) open"

"ChangeSorter browseChangeSetsWithSelector: #initialize"
! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
buildAggregateChangeSet
	"Establish a change-set named Aggregate which bears the union of all the changes in all the existing change-sets in the system (other than any pre-existing Aggregate).  This can be useful when wishing to discover potential conflicts between a disk-resident change-set and an image.  Formerly very useful, now some of its unique contributions have been overtaken by new features"

	| aggregateChangeSet |
	aggregateChangeSet := self existingOrNewChangeSetNamed: 'Aggregate'.
	aggregateChangeSet clear.
	self allChangeSets do:
		[:aChangeSet | aChangeSet == aggregateChangeSet ifFalse:
			[aggregateChangeSet assimilateAllChangesFoundIn: aChangeSet]]

"ChangeSorter buildAggregateChangeSet"

	! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
changeSet: aChangeSet containsClass: aClass
	| theClass |
	theClass := Smalltalk classNamed: aClass.
	theClass ifNil: [^ false].
	^ aChangeSet containsClass: theClass! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'ar 3/1/2005 13:39'!
changeSetCategories
	ChangeSetCategories ifNil:[self initializeChangeSetCategories].
	^ChangeSetCategories! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'ar 3/1/2005 13:38'!
changeSetCategoryNamed: aName
	"Answer the changeSetCategory of the given name, or nil if none"

	^ self changeSetCategories elementAt: aName asSymbol ! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'tk 7/23/2004 10:26'!
changeSetNamed: aName
	"Return the change set of the given name, or nil if none found."

	AllChangeSets do: [:aChangeSet | aChangeSet name = aName ifTrue: [^ aChangeSet]].
	CChangeSorter gatherChangeSets.	"mismatch -- some added in normal Squeak"
	^ AllChangeSets
			detect: [:aChangeSet | aChangeSet name = aName]
			ifNone: [nil].
! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
changeSetNamesInReleaseImage
	"Answer a list of names of project change sets that come pre-shipped in the latest sytem release.  On the brink of shipping a new release, call 'ChangeSorter noteChangeSetsInRelease'  "

	^ ChangeSetNamesInRelease ifNil:
		[ChangeSetNamesInRelease := self changeSetNamesInThreeOh]! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
changeSetNamesInThreeOh
	"Hard-coded: answer a list of names of project change sets that came pre-shipped in Squeak 3.0"

	^ #('The Worlds of Squeak' 'Fun with Morphic' 'Games' 'Fun With Music' 'Building with Squeak' 'Squeak and the Internet' 'Squeak in 3D' 'More About Sound' ) ! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
changeSetsNamedSuchThat: nameBlock
	"(ChangeSorter changeSetsNamedSuchThat:
		[:name | name first isDigit and: [name initialInteger >= 373]])
		do: [:cs | AllChangeSets remove: cs wither]"

	^ AllChangeSets select: [:aChangeSet | nameBlock value: aChangeSet name]! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
countOfChangeSetsWithClass: aClass andSelector: aSelector
	"Answer how many change sets record a change for the given class and selector"

	^ (self allChangeSetsWithClass: aClass selector: aSelector) size! !

!CChangeSorter class methodsFor: 'removing' stamp: 'ar 11/23/2004 19:28'!
deleteChangeSetsNumberedLowerThan: anInteger
	"Delete all changes sets whose names start with integers smaller than anInteger"

	self removeChangeSetsNamedSuchThat:
		[:aName | aName first isDigit and: [aName initialIntegerOrNil < anInteger]].

	"CChangeSorter deleteChangeSetsNumberedLowerThan: (CChangeSorter highestNumberedChangeSet name initialIntegerOrNil - 500)"
! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
doesAnyChangeSetHaveClass: aClass andSelector: aSelector
	"Answer whether any known change set bears a change for the given class and selector"

	^ (self countOfChangeSetsWithClass: aClass andSelector: aSelector) > 0! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:49'!
existingOrNewChangeSetNamed: aName

	| newSet |

	^(self changeSetNamed: aName) ifNil: [
		newSet := ChangeSet basicNewNamed: aName.
		AllChangeSets add: newSet.
		newSet
	]! !

!CChangeSorter class methodsFor: 'fileIn/Out' stamp: 'md 4/6/2004 16:50'!
fileIntoNewChangeSet: fullName
	"File in all of the contents of the currently selected file, if any, into a new change set." 

	| fn ff |
	fullName ifNil: [^ self beep].
	ff := FileStream readOnlyFileNamed: (fn := GZipReadStream uncompressedFileName: fullName).
	((FileDirectory extensionFor: fn) sameAs: 'html') ifTrue: [ff := ff asHtml].
	self newChangesFromStream: ff named: (FileDirectory localNameFor: fn)! !

!CChangeSorter class methodsFor: 'fileIn/Out' stamp: 'md 4/6/2004 16:50'!
fileReaderServicesForFile: fullName suffix: suffix

	^(suffix = 'st') | (suffix = 'cs') | (suffix = '*')
		ifTrue: [ self services]
		ifFalse: [#()]! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:50'!
gatherChangeSets		"ChangeSorter gatherChangeSets"
	"Collect any change sets created in other projects"
	| allChangeSets obsolete |
	allChangeSets := AllChangeSets asSet.
	ChangeSet allSubInstances do: [:each |
		(allChangeSets includes: each) == (obsolete := each isMoribund) ifTrue:[
			obsolete
				ifTrue: ["Was included and is obsolete."
						AllChangeSets remove: each]
				ifFalse: ["Was not included and is not obsolete."
						AllChangeSets add: each]]].
	^ AllChangeSets
! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:50'!
highestNumberedChangeSet
	"ChangeSorter highestNumberedChangeSet"
	| aList |
	aList := (self allChangeSetNames select: [:aString | aString startsWithDigit] thenCollect:
		[:aString | aString initialIntegerOrNil]).
	^ (aList size > 0)
		ifTrue:
			[aList max]
		ifFalse:
			[nil]
! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'ar 3/1/2005 13:37'!
initialize
	"Initialize the class variables"

	"CChangeSorter initialize"	
	self registerOpenCommand.
	AllChangeSets == nil ifTrue:
		[AllChangeSets := OrderedCollection new].
	self gatherChangeSets.
	"ChangeSetCategories ifNil:
		[self initializeChangeSetCategories]."
	RecentUpdateMarker := 0.

! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
initializeChangeSetCategories
	"Initialize the set of change-set categories"
	"ChangeSorter initializeChangeSetCategories"

	| aCategory |
	ChangeSetCategories := ElementCategory new categoryName: #ChangeSetCategories.

	aCategory := ChangeSetCategory new categoryName: #All.
	aCategory membershipSelector: #belongsInAll:.
	aCategory documentation: 'All change sets known to the system'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #Additions.
	aCategory membershipSelector: #belongsInAdditions:.
	aCategory documentation: 'All unnumbered change sets except those representing projects in the system as initially released.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #MyInitials.
	aCategory membershipSelector: #belongsInMyInitials:.
	aCategory documentation: 'All change sets whose names end with the current author''s initials.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #Numbered.
	aCategory membershipSelector: #belongsInNumbered:.
	aCategory documentation: 'All change sets whose names start with a digit -- normally these will be the official updates to the system.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #ProjectChangeSets.
	aCategory membershipSelector: #belongsInProjectChangeSets:.
	aCategory documentation: 'All change sets that are currently associated with projects present in the system right now.'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #ProjectsInRelease.
	aCategory membershipSelector: #belongsInProjectsInRelease:.
	aCategory documentation: 'All change sets belonging to projects that were shipped in the initial release of this version of Squeak'.
	ChangeSetCategories addCategoryItem: aCategory.

	aCategory := ChangeSetCategory new categoryName: #RecentUpdates.
	aCategory membershipSelector: #belongsInRecentUpdates:.
	aCategory documentation: 'Updates whose numbers are at or beyond the number I have designated as the earliest one to qualify as Recent'.
	ChangeSetCategories addCategoryItem: aCategory.

	ChangeSetCategories elementsInOrder do: [:anElem | anElem reconstituteList] ! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/6/2004 16:50'!
mostRecentChangeSetWithChangeForClass: class selector: selector
	| hits |
	hits := self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ 'not in any change set'].
	^ 'recent cs: ', hits last name! !

!CChangeSorter class methodsFor: 'adding' stamp: 'ar 1/5/2005 18:03'!
newChangeSet
	"Prompt the user for a name, and establish a new change set of
	that name (if ok), making it the current changeset.  Return nil
	of not ok, else return the actual changeset."

	| newName newSet |
	newName := self 
		request: 'Please name the new change set:'
		initialAnswer: ChangeSet defaultName.
	newName isEmptyOrNil ifTrue:
		[^ nil].
	newSet := self basicNewChangeSet: newName.
	newSet ifNotNil:
		[ChangeSet  newChanges: newSet].
	^ newSet! !

!CChangeSorter class methodsFor: 'adding' stamp: 'md 4/6/2004 16:50'!
newChangeSet: aName
	"Makes a new change set called aName, add author initials to try to
	ensure a unique change set name."

	| newName |
	newName := aName , FileDirectory dot , Utilities authorInitials.
	^ self basicNewChangeSet: newName! !

!CChangeSorter class methodsFor: 'adding' stamp: 'md 4/6/2004 16:50'!
newChangesFromStream: aStream named: aName
	"File in the code from the stream into a new change set whose
	name is derived from aName. Leave the 'current change set'
	unchanged. Return the new change set or nil on failure."

	| oldChanges newName newSet |
	oldChanges := ChangeSet current.
	PreviousSet := oldChanges name. 		"so a Bumper update can find it"
	newName := aName sansPeriodSuffix.
	newSet := self basicNewChangeSet: newName.
	[newSet ifNotNil:
		[ChangeSet  newChanges: newSet.
		aStream fileInAnnouncing: 'Loading ', newName, '...'.
		Transcript cr; show: 'File ', aName, ' successfully filed in to change set ', newName].
	aStream close] ensure: [
			ChangeSet  newChanges: oldChanges].
	^ newSet! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/6/2004 16:50'!
noteChangeSetsInRelease
	"Freshly compute what the change sets in the release are; to be called manually just before a release"

	ChangeSetNamesInRelease := (Project allProjects collect: [:p | p name]) asSet asOrderedCollection.

"ChangeSorter noteChangeSetsInRelease"! !

!CChangeSorter class methodsFor: 'opening' stamp: 'ar 3/1/2005 13:35'!
open
	^ToolBuilder open: DualChangeSorter new! !

!CChangeSorter class methodsFor: 'enumerating' stamp: 'md 4/25/2004 16:02'!
promoteToTop: aChangeSet
	"make aChangeSet the first in the list from now on"
	AllChangeSets remove: aChangeSet ifAbsent: [^ self].
	AllChangeSets add: aChangeSet.! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
recentUpdateMarker
	"Answer the number representing the threshold of what counts as 'recent' for an update number.  This allow you to use the RecentUpdates category in a ChangeSorter to advantage"

	^ RecentUpdateMarker ifNil: [RecentUpdateMarker := 0]! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
recentUpdateMarker: aNumber
	"Set the recent update marker as indicated"

	^ RecentUpdateMarker := aNumber! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'ar 1/22/2005 22:44'!
registerOpenCommand
	Smalltalk at: #CProjectBuilder ifPresent:[:pb|
		pb registerOpenCommand: { 'Change Sorter' . { self . #open }. 'A Change Sorter' }.
	].
! !

!CChangeSorter class methodsFor: 'removing' stamp: 'md 4/6/2004 16:50'!
removeChangeSet: aChangeSet
	"Remove the given changeSet.  Caller must assure that it's cool to do this"

	AllChangeSets remove: aChangeSet ifAbsent: [].
	aChangeSet wither
! !

!CChangeSorter class methodsFor: 'removing' stamp: 'ar 11/23/2004 19:28'!
removeChangeSetsNamedSuchThat: nameBlock
	(self changeSetsNamedSuchThat: nameBlock)
		do: [:cs | self removeChangeSet: cs]! !

!CChangeSorter class methodsFor: 'removing' stamp: 'md 4/6/2004 16:50'!
removeEmptyUnnamedChangeSets
	"Remove all change sets that are empty, whose names start with Unnamed,
		and which are not nailed down by belonging to a Project."
	"ChangeSorter removeEmptyUnnamedChangeSets"
	| toGo |
	(toGo := (self changeSetsNamedSuchThat: [:csName | csName beginsWith: 'Unnamed'])
		select: [:cs | cs isEmpty and: [cs okayToRemoveInforming: false]])
		do: [:cs | self removeChangeSet: cs].
	self inform: toGo size printString, ' change set(s) removed.'! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/25/2004 16:04'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come the project changesets that come with the release.  These are mostly empty.
		Next come all numbered updates.
		Next come all remaining changesets
	In a ChangeSorter, they will appear in the reversed order."

	"ChangeSorter reorderChangeSets"

	| newHead newMid newTail |
	newHead := OrderedCollection new.
	newMid := OrderedCollection new.
	newTail := OrderedCollection new.
	AllChangeSets do:
		[:aChangeSet |
			(self belongsInProjectsInRelease: aChangeSet)
				ifTrue:
					[newHead add: aChangeSet]
				ifFalse:
					[(self belongsInNumbered: aChangeSet)
						ifTrue:
							[newMid add: aChangeSet]
						ifFalse:
							[newTail add: aChangeSet]]].
	AllChangeSets := newHead, newMid, newTail.! !

!CChangeSorter class methodsFor: 'services' stamp: 'md 4/6/2004 16:50'!
secondaryChangeSet
	"Answer a likely change set to use as the second initial one in a Dual Change Sorter.  "

	AllChangeSets size = 1 ifTrue: [^ AllChangeSets first].
	AllChangeSets last == ChangeSet current
		ifTrue: 	[^ AllChangeSets at: (AllChangeSets size - 1)]
		ifFalse:	[^ AllChangeSets last]! !

!CChangeSorter class methodsFor: 'fileIn/Out' stamp: 'md 4/6/2004 16:50'!
serviceFileIntoNewChangeSet
	"Answer a service for installing a file into a new change set"

	^ SimpleServiceEntry 
		provider: self 
		label: 'install into new change set'
		selector: #fileIntoNewChangeSet:
		description: 'install the file as a body of code in the image: create a new change set and file-in the selected file into it'
		buttonLabel: 'install'! !

!CChangeSorter class methodsFor: 'fileIn/Out' stamp: 'md 4/6/2004 16:50'!
services

	^ Array with: self serviceFileIntoNewChangeSet

	! !

!CChangeSorter class methodsFor: 'class initialization' stamp: 'md 4/25/2004 16:17'!
unload
	"Unload the receiver from global registries"

	self environment at: #FileList ifPresent: [:cl |
	cl unregisterFileReader: self].! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 13:17'!
addPreamble
	self selectedChangeSet assurePreambleExists.
	self okToChange ifTrue:
		[self selectedChangeSetName: myChangeSet name]! !

!CChangeSorter methodsFor: 'annotation' stamp: 'md 4/6/2004 16:49'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"Add an annotation detailing the prior versions count.  Specially handled here for the case of a selector no longer in the system, whose prior version is pointed to by the lost-method pointer in the change held on to by the changeset"

	(aClass includesSelector: aSelector) ifTrue:
		[^ super addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream].
	aStream nextPutAll:
		((myChangeSet methodInfoFromRemoval: {aClass name. aSelector})
			ifNil:
				['no prior versions']
			ifNotNil:
				['version(s) retrievable here']), self annotationSeparator! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:01'!
addToCategoryOpposite
	"Add the current change set to the category viewed on the opposite side, if it's of the sort to accept things like that"

	| categoryOpposite |
	categoryOpposite := (parent other: self) changeSetCategory.
	categoryOpposite acceptsManualAdditions
		ifTrue:
			[categoryOpposite addChangeSet: myChangeSet.
			categoryOpposite reconstituteList.
			self update]
		ifFalse:
			[self inform: 
'sorry, this command only makes sense
if the category showing on the opposite
side is a static category whose
members are manually maintained']! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/23/2004 17:20'!
browseChangeSet
	"Open a message list browser on the new and changed methods in the current change set"


	CChangedMessageSet openFor: myChangeSet

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:01'!
browseMethodConflicts
	"Check to see if any other change set also holds changes to any methods in the selected change set; if so, open a browser on all such."

	| aList |

	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			(CChangeSorter allChangeSetsWithClass: aClass selector: aSelector) size > 1
		]
		ifNone: [^ self inform: 'No other change set has changes
for any method in this change set.'].
	
	CMessageSet 
		openMessageList: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in other change sets (', aList size printString, ')'
	! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/23/2004 16:40'!
browseVersions
	"Create and schedule a changelist browser on the versions of the 
	selected message."
	| class selector method category pair sourcePointer |

	(selector := self selectedMessageName) ifNil: [^ self].
	class := self selectedClassOrMetaClass.
	(class includesSelector: selector)
		ifTrue: [method := class compiledMethodAt: selector.
				category := class whichCategoryIncludesSelector: selector.
				sourcePointer := nil]
		ifFalse: [pair := myChangeSet methodInfoFromRemoval: {class name. selector}.
				pair ifNil: [^ nil].
				sourcePointer := pair first.
				method := CompiledMethod toReturnSelf setSourcePointer: sourcePointer.
				category := pair last].

	self cNotYet.
	"VersionsBrowser
		browseVersionsOf: method
		class: self selectedClass meta: class isMeta
		category: category selector: selector
		lostMethodPointer: sourcePointer."
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/20/2004 16:21'!
categorySubmenu: aMenu
	"Fill aMenu with less-frequently-needed category items"
	
	aMenu label: 'Change set category'.

	aMenu addList: #(
		('make a new category...' makeNewCategory)
		('make a new category with class...' makeNewCategoryShowingClassChanges)
		('rename this category' renameCategory)
		('remove this category' removeCategory)
		('show categories of this changeset' showCategoriesOfChangeSet)
	-) target: self..

	parent ifNotNil:
		[aMenu addList: #(
			('add change set to category opposite' addToCategoryOpposite)) target: self. ].

	aMenu addList: #(
		('remove change set from this category' removeFromCategory)
		-
		('file out category''s change sets' fileOutAllChangeSets)
		('set recent-updates marker' setRecentUpdatesMarker)
		('fill aggregate change set' fillAggregateChangeSet) ) target: self.

	^ aMenu! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/6/2004 16:49'!
changeSet
	^ myChangeSet! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:39'!
changeSetCategories
	^self class changeSetCategories! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:35'!
changeSetCategory
	"Answer the changeSetCategory of the receiver"
	<bewareOf: #changeSetCategoryChanged>
	^self propertyValueAt: #changeSetCategory! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:35'!
changeSetCategory: aValue
	"Modify the receiver's changeSetCategory"
	^self propertyValueAt: #changeSetCategory put: aValue with: #changeSetCategoryChanged! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/6/2004 16:49'!
changeSetCurrentlyDisplayed
	^ myChangeSet! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
changeSetList
	"Answer a list of ChangeSet names to be shown in the change sorter."

	^ self changeSetCategory changeSetList! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
changeSetListKey: aChar from: view
	"Respond to a Command key.  I am a model with a listView that has a list of changeSets."

	aChar == $b ifTrue: [^ self browseChangeSet].
	aChar == $B ifTrue: [^ self openChangeSetBrowser].
	aChar == $c ifTrue: [^ self copyAllToOther].
	aChar == $D ifTrue: [^ self toggleDiffing]. 
	aChar == $f ifTrue: [^ self findCngSet].
	aChar == $m ifTrue: [^ self newCurrent].
	aChar == $n ifTrue: [^ self newSet].
	aChar == $o ifTrue: [^ self fileOut].
	aChar == $p ifTrue: [^ self addPreamble].
	aChar == $r ifTrue: [^ self rename].
	aChar == $s ifTrue: [^ self chooseChangeSetCategory].
	aChar == $x ifTrue: [^ self remove].
	aChar == $- ifTrue: [^ self subtractOtherSide].

	^ self messageListKey: aChar from: view! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/20/2004 16:20'!
changeSetListMenu
	"Set up aMenu to hold commands for the change-set-list pane.  This could be for a single or double changeSorter"

	| aMenu subMenu |

	aMenu := CMenu new.

	aMenu label: 'Change Set'.
	aMenu add: 'make changes go to me (m)' target: self action: #newCurrent.
	aMenu addSeparator.
	aMenu add: 'new change set... (n)'  target: self action: #newSet.
	aMenu add: 'find...(f)'  target: self action: #findCngSet.
	aMenu add: 'show category... (s)'  target: self action:  #chooseChangeSetCategory.
	aMenu add: 'select change set...'  target: self action: #chooseCngSet.
	aMenu addSeparator.
	aMenu add: 'rename change set (r)'  target: self action: #rename.
	aMenu add: 'file out (o)'  target: self action: #fileOut.
	aMenu add: 'mail to list'  target: self action: #mailOut.
	aMenu add: 'browse methods (b)'  target: self action: #browseChangeSet.
	aMenu add: 'browse change set (B)'  target: self action: #openChangeSetBrowser.
	aMenu addSeparator.
	parent
		ifNotNil: 
			[aMenu add: 'copy all to other side (c)'  target: self action: #copyAllToOther.
			aMenu add: 'submerge into other side'  target: self action: #submergeIntoOtherSide.
			aMenu add: 'subtract other side (-)'  target: self action: #subtractOtherSide.
			aMenu addSeparator].
	myChangeSet hasPreamble
		ifTrue: 
			[aMenu add: 'edit preamble (p)'  target: self action: #addPreamble.
			aMenu add: 'remove preamble'  target: self action: #removePreamble]
		ifFalse: [aMenu add: 'add preamble (p)'  target: self action: #addPreamble].
	myChangeSet hasPostscript
		ifTrue: 
			[aMenu add: 'edit postscript...'  target: self action: #editPostscript.
			aMenu add: 'remove postscript'  target: self action: #removePostscript]
		ifFalse: [aMenu add: 'add postscript...'  target: self action: #editPostscript].
	aMenu addSeparator.

	aMenu add: 'category functions...' subMenu: (subMenu := CMenu new).
	self categorySubmenu: subMenu.
	aMenu addSeparator.

	aMenu add: 'destroy change set (x)'  target: self action: #remove.
	aMenu addSeparator.
	aMenu add: 'more...' subMenu: (subMenu := CMenu new).
	self shiftedChangeSetMenu: subMenu.
	^ aMenu.! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:37'!
changeSetListPane
	"Answer the changeSetListPane of the receiver"
	<bewareOf: #changeSetListPaneChanged>
	^self propertyValueAt: #changeSetListPane! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:37'!
changeSetListPane: aValue
	"Modify the receiver's changeSetListPane"
	^self propertyValueAt: #changeSetListPane put: aValue with: #changeSetListPaneChanged! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship"

	myChangeSet checkForAlienAuthorship

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForAnyAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship, even historically"

	myChangeSet checkForAnyAlienAuthorship

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForUnclassifiedMethods
	"Open a message list browser on all methods in the current change set that have not been categorized"

	myChangeSet checkForUnclassifiedMethods

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForUncommentedClasses
	"Open a class list browser on classes in the change set that lack class comments"

	myChangeSet checkForUncommentedClasses! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForUncommentedMethods
	"Open a message list browser on all uncommented methods in the current change set"

	myChangeSet checkForUncommentedMethods

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
checkForUnsentMessages
	"Open a message list browser on all unsent messages in the current change set"

	myChangeSet checkForUnsentMessages

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
checkThatSidesDiffer: escapeBlock
	"If the change sets on both sides of the dual sorter are the same, put up an error message and escape via escapeBlock, else proceed happily"

	(myChangeSet == (parent other: self) changeSet)
		ifTrue:
			[self inform: 
'This command requires that the
change sets selected on the two
sides of the change sorter *not*
be the same.'.
			^ escapeBlock value]
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:37'!
chooseChangeSetCategory
	"Present the user with a list of change-set-categories and let her choose one.  In this morphic variant, we include balloon help"

	|  aMenu item |
	
	self okToChange ifFalse: [^ self].

	aMenu := CMenu new.
	aMenu label: 'Choose the category of\change sets to show in\this Change Sorter' withCRs.
	self changeSetCategories elementsInOrder do:
		[:aCategory |
			item := aMenu 
						add: aCategory categoryName 
						target: self 
						action: #showChangeSetCategory: 
						argument: aCategory.
			item checked: aCategory == changeSetCategory].
	aMenu run.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 11/23/2004 19:28'!
chooseCngSet
	"Present the user with an alphabetical list of change set names, and let her choose one"

	| changeSetsSortedAlphabetically chosen |
	self okToChange ifFalse: [^ self].

	changeSetsSortedAlphabetically := self changeSetList asSortedCollection:
		[:a :b | a asLowercase withoutLeadingDigits < b asLowercase withoutLeadingDigits].
	
	chosen := (CMenu new chooseFromValues: changeSetsSortedAlphabetically).
	chosen ifNil: [^ self].
	self showChangeSet: (self changeSetNamed: chosen)! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/6/2004 16:49'!
classList
	"Computed.  View should try to preserve selections, even though index changes"

	^ myChangeSet ifNotNil: [myChangeSet changedClassNames] ifNil: [OrderedCollection new]
! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/6/2004 16:49'!
classListKey: aChar from: view
	"Respond to a Command key in the class-list pane."

	aChar == $x ifTrue: [^ self removeClass].
	aChar == $d ifTrue: [^ self forgetClass]. 

	^ self messageListKey: aChar from: view "picks up b,h,p"! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/20/2004 16:28'!
classListMenu
	"Fill aMenu with items appropriate for the class list"

	| aMenu subMenu |

	aMenu := CMenu new.

	aMenu label: 'class list'.
	
	(parent notNil)
		ifTrue: [aMenu addList: #( "These two only apply to dual change sorters"
			('copy class chgs to other side'			copyClassToOther)	
			('move class chgs to other side'			moveClassToOther)) target: self].

	aMenu addList: 
		#(
			-
			('delete class from change set (d)'		forgetClass)
			('remove class from system (x)'			removeClass)
			-
			('browse full (b)'						browseMethodFull)
			('browse hierarchy (h)'					spawnHierarchy)
			('browse protocol (p)'					browseFullProtocol)
			-
			('printOut'								printOutClass)
			('fileOut'								fileOutClass)
			-
			('inst var refs...'						browseInstVarRefs)
			('inst var defs...'						browseInstVarDefs)
			('class var refs...'						browseClassVarRefs)
			('class vars'								browseClassVariables)
			('class refs (N)'							browseClassRefs)
			-)
			target: self.

			aMenu add: 'more...' subMenu: (subMenu := CMenu new).	

			subMenu addList: #(
			('unsent methods'						browseUnusedMethods)
			('unreferenced inst vars'				showUnreferencedInstVars)
			('unreferenced class vars'				showUnreferencedClassVars)
			-
			('sample instance'						makeSampleInstance)
			('inspect instances'						inspectInstances)
			('inspect subinstances'					inspectSubInstances)) target: self.
	^ aMenu! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:35'!
classListPane
	"Answer the classListPane of the receiver"
	<bewareOf: #classListPaneChanged>
	^self propertyValueAt: #classListPane! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:35'!
classListPane: aValue
	"Modify the receiver's classListPane"
	^self propertyValueAt: #classListPane put: aValue with: #classListPaneChanged! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
clearChangeSet
	"Clear out the current change set, after getting a confirmation."
	| message |

	self okToChange ifFalse: [^ self].
	myChangeSet isEmpty ifFalse:
		[message := 'Are you certain that you want to\forget all the changes in this set?' withCRs.
		(self confirm: message) ifFalse: [^ self]].
	myChangeSet clear.
	self changed.
! !

!CChangeSorter methodsFor: 'code pane' stamp: 'md 4/25/2004 16:19'!
contents: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Create an error if the category of the selected message is unknown. 
	Answer false if the compilation fails. Otherwise, if the compilation 
	created a new method, deselect the current selection. Then answer true."
	| category selector class oldSelector |

	(class := self selectedClassOrMetaClass) ifNil:
		[(myChangeSet preambleString == nil or: [aString size == 0]) ifTrue: [ ^ false].
		(aString count: [:char | char == $"]) odd 
			ifTrue: [self inform: 'unmatched double quotes in preamble']
			ifFalse: [(Scanner new scanTokens: aString) size > 0 ifTrue: [
				self inform: 'Part of the preamble is not within double-quotes.
To put a double-quote inside a comment, type two double-quotes in a row.
(Ignore this warning if you are including a doIt in the preamble.)']].
		myChangeSet preambleString: aString.
		^ true].
	oldSelector := self selectedMessageName.
	category := class organization categoryOfElement: oldSelector.
	selector := class compile: aString
				classified: category
				notifying: aController. 
	selector ifNil: [^ false].
	self update.
	^ true! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:23'!
copyAllToOther
	"Copy this entire change set into the one on the other side"
	| companionSorter |
	self checkThatSidesDiffer: [^ self].
	(companionSorter := parent other: self) changeSetCurrentlyDisplayed assimilateAllChangesFoundIn: myChangeSet.
	companionSorter changed.! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/25/2004 13:20'!
copyClassToOther
	"Place these changes in the other changeSet also"

	| otherSorter otherChangeSet |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ self beep].
	selectedClassName ifNil: [^ self beep].
	otherSorter := parent other: self.
	otherChangeSet := otherSorter changeSet.

	otherChangeSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	otherSorter showChangeSet: otherChangeSet.! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:23'!
copyMethodToOther
	"Place this change in the other changeSet also"
	| other cls sel |
	self checkThatSidesDiffer: [^ self].
	selectedMessage ifNotNil:
		[other := (parent other: self) changeSet.
		cls := self selectedClassOrMetaClass.
		sel := selectedMessage asSymbol.

		other absorbMethod: sel class: cls from: myChangeSet.
		(parent other: self) showChangeSet: other]
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
currentCngSet
	^ myChangeSet name! !

!CChangeSorter methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:24'!
defaulWindowColor
	^Color r: 0.664 g: 0.8 b: 0.8! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
editPostscript
	"Allow the user to edit the receiver's change-set's postscript -- in a separate window"

	myChangeSet editPostscript! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
editPreamble
	"Allow the user to edit the receiver's change-set's preamble -- in a separate window."

	myChangeSet editPreamble! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:23'!
expungeUniclasses
	"remove all memory of uniclasses in the receiver"

	self okToChange ifFalse: [^ self].
	myChangeSet expungeUniclasses.
	self changed.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
fileIntoNewChangeSet
	"Obtain a file designation from the user, and file its contents into a new change set whose name is a function of the filename.  Show the new set and leave the current changeSet unaltered."
	| aFileName  aNewChangeSet |

	self okToChange ifFalse: [^ self].
	aFileName := self request: 'Name of file to be imported: '.
	aFileName size == 0 ifTrue: [^ self].
	(FileDirectory default fileExists: aFileName) ifFalse:
		[^ self inform: 'Sorry -- cannot find that file'].

	aNewChangeSet := self class 
			newChangesFromStream: (FileStream readOnlyFileNamed: aFileName) 
			named: aFileName.
	aNewChangeSet ifNotNil: [self showChangeSet: aNewChangeSet]! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:24'!
fileOut
	"File out the current change set."

	myChangeSet fileOut.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
fileOutAllChangeSets
	"File out all nonempty change sets in the current category, probably"

	self changeSetCategory fileOutAllChangeSets! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/25/2004 16:19'!
fileOutClass
	"this is a hack!!!! makes a new change set, called the class name, adds author initials to try to make a unique change set name, files it out and removes it. kfr 16 june 2000" 
	| aSet |
	"File out the selected class set."
     aSet := self class newChangeSet: selectedClassName.
	aSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	aSet fileOut.
	self class removeChangeSet: aSet.
	! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
fillAggregateChangeSet
	"Create a change-set named Aggregate and pour into it all the changes in all the change-sets of the currently-selected category"

	self changeSetCategory fillAggregateChangeSet! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
findCngSet 
	"Search for a changeSet by name.  Pop up a menu of all changeSets whose name contains the string entered by the user.  If only one matches, then the pop-up menu is bypassed"
	| pattern candidates nameList varMenu result |
	self okToChange ifFalse: [^ self].
	pattern := self request: 'ChangeSet name or fragment?'.
	pattern isEmpty ifTrue: [^ self].
	nameList := self changeSetList asSet.
	candidates := AllChangeSets select:
			[:c | (nameList includes: c name) and: 
				[c name includesSubstring: pattern caseSensitive: false]].
	candidates size = 0 ifTrue: [^ self beep].
	candidates size = 1 ifTrue:
		[^ self showChangeSet: candidates first].

	candidates := candidates collect: [:each | Array with: each name with: each].
	varMenu := CMenu new.
	result := varMenu chooseFrom: candidates.
	self showChangeSet: result.
! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:23'!
forget
	"Drop this method from the changeSet"

	self okToChange ifFalse: [^ self].
	selectedMessage ifNotNil: [
		myChangeSet removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
		selectedMessage := nil.
		self showChangeSet: myChangeSet]! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 5/3/2004 14:21'!
forgetClass
	"Remove all mention of this class from the changeSet"

	self okToChange ifFalse: [^ self].
	selectedClassName ifNotNil: [
		self selectedChangeSet removeClassChanges: selectedClassName].

	classListPane selection: classListPane prev.
	self onSelectedChangeSetNameChanged.
	
	! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
goToChangeSetsProject
	"Transport the user to a project which bears the selected changeSet as its current changeSet"

	| aProject |
	(aProject := myChangeSet correspondingProject) 
		ifNotNil:
			[aProject enter: false revert: false saveForRevert: false]
		ifNil:
			[self inform: 'Has no project']! !

!CChangeSorter methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:57'!
initialize
	super initialize.
	"Define our components"
	
	self define: #changeSetListPane as: CStringList new.
	changeSetListPane allowDeselect: false.

	self define: #classListPane as: CStringList new.
	classListPane allowDeselect: true.

	self define: #methodListPane as: CStringList new.
	methodListPane allowDeselect: false.

	self setDefaultChangeSetCategory.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
inspectChangeSet
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet inspectWithLabel: 'Change set: ', myChangeSet name

! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/6/2004 16:49'!
label
	^ self labelString! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/25/2004 13:56'!
labelString
	"The label for my entire window.  The large button that displays my name is gotten via mainButtonName"

	^ String streamContents:
		[:aStream |
			aStream nextPutAll: (ChangeSet current == self selectedChangeSet
				ifTrue: ['Changes go to "', selectedChangeSetName, '"']
				ifFalse: ['ChangeSet: ', selectedChangeSetName]).
		(self changeSetCategory categoryName ~~ #All)
			ifTrue:
				[aStream nextPutAll:  ' - ', self parenthesizedCategoryName]]! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
lookForSlips
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet lookForSlips

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/23/2004 17:11'!
mailOut
	"Create a mail with a gzipped attachment holding out the current change 
	set. "
	
	self cNotYet.

	"
	myChangeSet mailOut.
	parent modelWakeUp
	"! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
mainButtonName

	^ myChangeSet name! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:38'!
makeNewCategory
	"Create a new, static change-set category, which will be populated entirely by change sets that have been manually placed in it"

	| catName aCategory |
	catName := self request: 'Please give the new category a name' initialAnswer: ''.
	catName isEmptyOrNil ifTrue: [^ self].
	catName := catName asSymbol.
	(self changeSetCategories includesKey: catName) ifTrue:
		[^ self inform: 'Sorry, there is already a category of that name'].

	aCategory := StaticChangeSetCategory new categoryName: catName.
	self changeSetCategories elementAt: catName put: aCategory.
	aCategory addChangeSet: myChangeSet.
	self showChangeSetCategory: aCategory! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:38'!
makeNewCategoryShowingClassChanges
	"Create a new, static change-set category, which will be populated entirely by change sets that have been manually placed in it"

	| catName aCategory clsName |
	clsName := self selectedClass ifNotNil: [self selectedClass name ] ifNil: [''].
	clsName := self request: 'Which class?' initialAnswer: clsName.
	clsName isEmptyOrNil ifTrue: [^ self].
	catName := ('Changes to ', clsName) asSymbol.
	(self changeSetCategories includesKey: catName) ifTrue:
		[^ self inform: 'Sorry, there is already a category of that name'].

	aCategory := ChangeSetCategoryWithParameters new categoryName: catName.
	aCategory membershipSelector: #changeSet:containsClass: ; parameters: { clsName }.
	self changeSetCategories elementAt: catName put: aCategory.
	aCategory reconstituteList.
	self showChangeSetCategory: aCategory! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:26'!
messageList 

	| probe newSelectors |
	probe := (selectedClassName endsWith: ' class')
		ifTrue: [selectedClassName]
		ifFalse: [selectedClassName asSymbol].
	newSelectors := myChangeSet selectorsInClass: probe.
	(newSelectors includes: selectedMessage) ifFalse: [selectedMessage := nil].
	^ newSelectors asSortedCollection
! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/6/2004 16:49'!
messageListKey: aChar from: view
	"Respond to a Command key in the message-list pane."

	aChar == $d ifTrue: [^ self forget].
	super messageListKey: aChar from: view! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/23/2004 16:34'!
messageMenu
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	| aMenu subMenu |

	aMenu := CMenu new.

	aMenu label: 'message list'.

	parent ifNotNil:
		[aMenu addList: #(
			('copy method to other side'			copyMethodToOther)
			('move method to other side'			moveMethodToOther)) target: self].

	aMenu addList: #(
			('delete method from changeSet (d)'	forget)
			-
			('remove method from system (x)'	removeMessage)
				-
			('browse full (b)'					browseMethodFull)
			('browse hierarchy (h)'				spawnHierarchy)
			('browse method (O)'				openSingleMessageBrowser)
			('browse protocol (p)'				browseFullProtocol)
			-
			('fileOut'							fileOutMessage)
			('printOut'							printOutMessage)
			-
			('senders of... (n)'					browseSenders)
			('implementors of... (m)'				browseImplementors)
			('inheritance (i)'					methodHierarchy)
			('versions (v)'						browseVersions)
			-
			) target: self.
			aMenu add: 'more...' subMenu: (subMenu := CMenu new).	
			self shiftedMessageMenu: subMenu.
	^ aMenu
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
methodConflictsWithOppositeCategory
	"Check to see if ANY change set on the other side shares any methods with the selected change set; if so, open a browser on all such."

	| aList otherCategory |

	otherCategory := (parent other: self) changeSetCategory.
	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			aClass notNil and: 
				[otherCategory 
					hasChangeForClassName: aClass name 
					selector: aSelector 
					otherThanIn: myChangeSet]
		]
		ifNone: [^ self inform: 
'There are no methods that appear both in
this change set and in any change set
(other than this one) on the other side.'].
	
	CMessageSet 
		openMessageList: aList 
		name: 'Methods in "', myChangeSet name, '" also in some other change set in category ', otherCategory categoryName,' (', aList size printString, ')'
	! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
methodConflictsWithOtherSide
	"Check to see if the change set on the other side shares any methods with the selected change set; if so, open a browser on all such."

	| aList other |

	self checkThatSidesDiffer: [^ self].
	other := (parent other: self) changeSet.
	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			aClass notNil and: [(other methodChangesAtClass: aClass name) includesKey: aSelector]
		]
		ifNone:  [^ self inform: 'There are no methods that appear
both in this change set and
in the one on the other side.'].
	
	CMessageSet 
		openMessageList: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in ', other name,' (', aList size printString, ')'
	! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:35'!
methodListPane
	"Answer the methodListPane of the receiver"
	<bewareOf: #methodListPaneChanged>
	^self propertyValueAt: #methodListPane! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/20/2004 12:35'!
methodListPane: aValue
	"Modify the receiver's methodListPane"
	^self propertyValueAt: #methodListPane put: aValue with: #methodListPaneChanged! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/25/2004 13:19'!
moveClassToOther
	"Place class changes in the other changeSet and remove them from this one"

	self checkThatSidesDiffer: [^ self].
	(self okToChange and: [selectedClassName notNil]) ifFalse: [^ self beep].

	self copyClassToOther.
	self forgetClass! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:26'!
moveMethodToOther
	"Place this change in the other changeSet and remove it from this side"
	| other cls sel |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ self beep].
	selectedMessage ifNotNil:
		[other := (parent other: self) changeSet.
		other == myChangeSet ifTrue: [^ self beep].
		cls := self selectedClassOrMetaClass.
		sel := selectedMessage asSymbol.
		other absorbMethod: sel class: cls from: myChangeSet.
	
		(parent other: self) showChangeSet: other.
		self forget "removes the method from this side"]
! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
myChangeSet
	"Answer the myChangeSet of the receiver"
	<bewareOf: #myChangeSetChanged>
	^self propertyValueAt: #myChangeSet! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
myChangeSet: aValue
	"Modify the receiver's myChangeSet"
	^self propertyValueAt: #myChangeSet put: aValue with: #myChangeSetChanged! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:26'!
newCurrent
	"make my change set be the current one that changes go into"

	ChangeSet  newChanges: myChangeSet.
	self update.  "Because list of changes in a category may thus have changed"
	! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 14:02'!
newSet
	"Create a new changeSet and show it., making it the current one.  Reject name if already in use."

	| aSet |
	self okToChange ifFalse: [^ self].
	aSet := self class newChangeSet.
	aSet ifNotNil:
		[
		self changeSetCategory acceptsManualAdditions ifTrue:
			[changeSetCategory addChangeSet: aSet].
		self update.
		(changeSetCategory includesChangeSet: aSet) ifTrue:
			[self showChangeSet: aSet].
		]! !

!CChangeSorter methodsFor: 'events' stamp: 'ar 1/22/2005 14:32'!
onAcceptCodePane
	"Update the class list when the system category list's cursor changes"
	<on: save in: codePane> 

	self contents: codePane contents notifying: nil! !

!CChangeSorter methodsFor: 'events - lists' stamp: 'md 4/25/2004 13:09'!
onChangeSetListCursorChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: changeSetListPane> 
	self selectedChangeSetName: changeSetListPane selection! !

!CChangeSorter methodsFor: 'events - lists' stamp: 'md 4/25/2004 13:09'!
onClassListCursorChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: classListPane> 
	self selectedClassName: classListPane selection.
	! !

!CChangeSorter methodsFor: 'events - lists' stamp: 'md 4/25/2004 13:38'!
onMethodListCursorChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: methodListPane> 
	self selectedMessage: methodListPane selection.
	! !

!CChangeSorter methodsFor: 'events' stamp: 'md 4/25/2004 13:55'!
onSelectedChangeSetNameChanged
	"Get the current selection from the list"
	| selected |
	<on: selectedChangeSetNameChanged>
	selected := selectedChangeSetName.	
	selected ifNil:[^classListPane contents: #()].
	myChangeSet := self selectedChangeSet.
	changeSetListPane menu: self changeSetListMenu.
	classListPane menu: self classListMenu.

	classListPane contents: self classList.
	self signal: #updateWindowLabel.
	self signal: #updateCodePane.! !

!CChangeSorter methodsFor: 'events' stamp: 'md 4/25/2004 13:13'!
onSelectedClassNameChanged
	"Get the current selection from the list"
	| selected |
	<on: selectedClassNameChanged>
	selected := self selectedClassName.
	"Update the class list accordingly"
	selected ifNil:[^methodListPane contents: #()].
	methodListPane contents: self messageList.
	methodListPane menu: self messageMenu.! !

!CChangeSorter methodsFor: 'events' stamp: 'md 4/25/2004 13:13'!
onSelectedMessageChanged
	<on: selectedMessageChanged>
	self signal: #updateCodePane.! !

!CChangeSorter methodsFor: 'events' stamp: 'ar 1/22/2005 14:32'!
onUpdateCodePane
	"Update the contents of the code pane"
	codePane contents: self setContents.! !

!CChangeSorter methodsFor: 'events' stamp: 'ar 2/7/2005 01:04'!
onUpdateWindowLabel
	<on: updateWindowLabel>
	self label: self labelString.! !

!CChangeSorter methodsFor: 'creation' stamp: 'md 4/25/2004 13:44'!
open
	"ChangeSorter new open"
	super open.
	self showChangeSet: ChangeSet current 
	
	
! !

!CChangeSorter methodsFor: 'TODO' stamp: 'md 4/25/2004 12:31'!
openAsMorphIn: awindow rect: rect
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 0.5@0."

"	| csListHeight msgListHeight csMsgListHeight |
	contents := ''.
	csListHeight := 0.25.
	msgListHeight := 0.25.
	csMsgListHeight := csListHeight + msgListHeight.
	self addDependent: window.		

	window addMorph: ((PluggableListMorphByItem on: self
				list: #changeSetList
				selected: #currentCngSet
				changeSelected: #showChangeSetNamed:
				menu: #changeSetMenu:shifted:
				keystroke: #changeSetListKey:from:)
			autoDeselect: false)
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
				list: #classList
				selected: #currentClassName
				changeSelected: #currentClassName:
				menu: #classListMenu:shifted:
				keystroke: #classListKey:from:)
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
				list: #messageList
				selected: #currentSelector
				changeSelected: #currentSelector:
				menu: #messageMenu:shifted:
				keystroke: #messageListKey:from:)
		frame: (((0@csListHeight extent: 1@msgListHeight)
			scaleBy: rect extent) translateBy: rect origin).

	 self addLowerPanesTo: window
		at: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin)
		with: nil.
"! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/23/2004 17:17'!
openChangeSetBrowser
	"Open a ChangeSet browser on the current change set"

	self browseChangeSet ! !

!CChangeSorter methodsFor: 'TODO' stamp: 'md 4/25/2004 12:32'!
openView: topView offsetBy: offset
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 360@0."

"	| classView messageView codeView cngSetListView basePane annoPane annoHeight |
	contents := ''.
	annoHeight := 20.
	self addDependent: topView. 

	cngSetListView := PluggableListViewByItem on: self
		list: #changeSetList
		selected: #currentCngSet
		changeSelected: #showChangeSetNamed:
		menu: #changeSetMenu:shifted:
		keystroke: #changeSetListKey:from:.
	cngSetListView window: ((0@0 extent: 180@100) translateBy: offset).
	topView addSubView: cngSetListView.

	classView := PluggableListViewByItem on: self
		list: #classList
		selected: #currentClassName
		changeSelected: #currentClassName:
		menu: #classListMenu:shifted:
		keystroke: #classListKey:from:.
	classView window: ((0@0 extent: 180@100) translateBy: offset).
	topView addSubView: classView toRightOf: cngSetListView.

	messageView := PluggableListViewByItem on: self
		list: #messageList
		selected: #currentSelector
		changeSelected: #currentSelector:
		menu: #messageMenu:shifted:
		keystroke: #messageListKey:from:.
	messageView menuTitleSelector: #messageListSelectorTitle.
	messageView window: ((0@0 extent: 360@100) translateBy: offset).
	topView addSubView: messageView below: cngSetListView.

	 self wantsAnnotationPane
		ifFalse:
			[basePane := messageView]
		ifTrue:
			[annoPane := PluggableTextView on: self
				text: #annotation
				accept: nil
				readSelection: nil
				menu: nil.
			annoPane window: ((0@0 extent: 360@annoHeight) translateBy: offset).
			topView addSubView: annoPane below: messageView.
			basePane := annoPane].

	codeView := PluggableTextView on: self 
		text: #contents
		accept: #contents:notifying:
		readSelection: #contentsSelection
		menu: #codePaneMenu:shifted:.
	codeView window: ((0 @ 0 extent: 360 @ 180) translateBy: offset).
	topView addSubView: codeView below: basePane.
"! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
parent
	"Answer the parent of the receiver"
	<bewareOf: #parentChanged>
	^self propertyValueAt: #parent! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
parent: aValue
	"Modify the receiver's parent"
	^self propertyValueAt: #parent put: aValue with: #parentChanged! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/6/2004 16:49'!
parenthesizedCategoryName
	"Answer my category name in parentheses"

	^ ' (', self changeSetCategory categoryName, ')'! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
priorChangeSetList
	"Answer the priorChangeSetList of the receiver"
	<bewareOf: #priorChangeSetListChanged>
	^self propertyValueAt: #priorChangeSetList! !

!CChangeSorter methodsFor: 'accessing' stamp: 'ar 6/2/2004 16:34'!
priorChangeSetList: aValue
	"Modify the receiver's priorChangeSetList"
	^self propertyValueAt: #priorChangeSetList put: aValue with: #priorChangeSetListChanged! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:02'!
promoteToTopChangeSet
	self class promoteToTop: myChangeSet.
	self showChangeSet: ChangeSet current.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
remove
	"Completely destroy my change set.  Check if it's OK first"

	self okToChange ifFalse: [^ self].
	self removePrompting: true.
	self update! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:38'!
removeCategory
	"Remove the current category"

	| itsName |
	self changeSetCategory acceptsManualAdditions ifFalse:
		[^ self inform: 'sorry, you can only remove manually-added categories.'].

	(self confirm: 'Really remove the change-set-category
named ', (itsName := changeSetCategory categoryName), '?') ifFalse: [^ self].

	self changeSetCategories removeElementAt: itsName.
	self setDefaultChangeSetCategory.

	self update! !

!CChangeSorter methodsFor: 'class list' stamp: 'ar 1/5/2005 18:02'!
removeClass
	"Remove the selected class from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."
	| message  className classToRemove |
	self okToChange ifFalse: [^ false].
	classToRemove := self selectedClassOrMetaClass ifNil: [self flash. ^ false].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you want to REMOVE

	', className, '

from the system?'.
	(self confirm: message) ifFalse:[^false].
	classToRemove subclasses size > 0 ifTrue: [
		self inform:  className, ' has subclasses!!
It cannot be removed'.
		^false].
	classToRemove removeFromSystem.
	self onUpdateClassListPane.
	^true! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
removeFromCategory
	"Add the current change set to the category viewed on the opposite side, if it's of the sort to accept things like that"

	| aCategory |
	(aCategory := self changeSetCategory) acceptsManualAdditions
		ifTrue:
			[aCategory removeElementAt: myChangeSet name.
			aCategory reconstituteList.
			self update]
		ifFalse:
			[self inform: 
'sorry, this command only makes
sense for static categories whose
members are manually maintained']! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:25'!
removeFromCurrentChanges
	"Redisplay after removal in case we are viewing the current changeSet"

	super removeFromCurrentChanges.
	self signal: #selectedChangeSetNameChanged.! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:23'!
removeMessage
	"Remove the selected msg from the system. Real work done by the 
	parent, a ChangeSorter"
	| confirmation sel |
	self okToChange
		ifFalse: [^ self].
	selectedMessage
		ifNotNil: [confirmation := self systemNavigation   confirmRemovalOf: (sel := self selectedMessageName) on: self selectedClassOrMetaClass.
			confirmation == 3
				ifTrue: [^ self].
			self selectedClassOrMetaClass removeSelector: sel.
			self update.
			confirmation == 2
				ifTrue: [self systemNavigation browseAllCallsOn: sel]]! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 15:43'!
removePostscript
	myChangeSet removePostscript.
	self showChangeSet: myChangeSet! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
removePreamble
	myChangeSet removePreamble.
	self showChangeSet: myChangeSet! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
removePrompting: doPrompt
	"Completely destroy my change set.  Check if it's OK first, and if doPrompt is true, get the user to confirm his intentions first."

	| message aName changeSetNumber msg |
	aName := myChangeSet name.
	myChangeSet okayToRemove ifFalse: [^ self]. "forms current changes for some project"
	(myChangeSet isEmpty or: [doPrompt not]) ifFalse:
		[message := 'Are you certain that you want to 
remove (destroy) the change set
named  "', aName, '" ?'.
		(self confirm: message) ifFalse: [^ self]].

	doPrompt ifTrue:
		[msg := myChangeSet hasPreamble
			ifTrue:
				[myChangeSet hasPostscript
					ifTrue:
						['a preamble and a postscript']
					ifFalse:
						['a preamble']]
			ifFalse:
				[myChangeSet hasPostscript
					ifTrue:
						['a postscript']
					ifFalse:
						['']].
		msg isEmpty ifFalse:
			[(self confirm: 
'Caution!!  This change set has
', msg, ' which will be
lost if you destroy the change set.
Do you really want to go ahead with this?') ifFalse: [^ self]]].

	"Go ahead and remove the change set"
	changeSetNumber := myChangeSet name initialIntegerOrNil.
	changeSetNumber ifNotNil: [SystemVersion current unregisterUpdate: changeSetNumber].
	CChangeSorter removeChangeSet: myChangeSet.
	self showChangeSet: ChangeSet current.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
rename
	"Store a new name string into the selected ChangeSet.  reject duplicate name; allow user to back out"

	| newName |
	newName := self request: 'New name for this change set'
						initialAnswer: myChangeSet name.
	(newName = myChangeSet name or: [newName size == 0]) ifTrue:
			[^ self beep].

	(self class changeSetNamed: newName) ifNotNil:
			[^ Utilities inform: 'Sorry that name is already used'].

	myChangeSet name: newName.
	changeSetListPane contents: self changeSetList.
	self update.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:38'!
renameCategory
	"Obtain a new name for the category and, if acceptable, apply it"

	| catName oldName |
	self changeSetCategory acceptsManualAdditions ifFalse:
		[^ self inform: 'sorry, you can only rename manually-added categories.'].

	catName := self request: 'Please give the new category a name' initialAnswer:  (oldName := changeSetCategory categoryName).
	catName isEmptyOrNil ifTrue: [^ self].
	(catName := catName asSymbol) = oldName ifTrue: [^ self inform: 'no change.'].
	(self changeSetCategories includesKey: catName) ifTrue:
		[^ self inform: 'Sorry, there is already a category of that name'].

	changeSetCategory categoryName: catName.
	self changeSetCategories removeElementAt: oldName.
	self changeSetCategories elementAt: catName put: changeSetCategory.

	self update! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
reorderChangeSets
	"apply a standard reordering -- let the class handle this"

	^ self class reorderChangeSets! !

!CChangeSorter methodsFor: 'accessing' stamp: 'md 4/25/2004 12:55'!
selectedChangeSet
	^ CChangeSorter changeSetNamed: selectedChangeSetName! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:51'!
selectedChangeSetName
	"Answer the selectedChangeSetName of the receiver"
	<bewareOf: #selectedChangeSetNameChanged>
	^self propertyValueAt: #selectedChangeSetName! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:51'!
selectedChangeSetName: aValue
	"Modify the receiver's selectedChangeSetName"
	^self propertyValueAt: #selectedChangeSetName put: aValue with: #selectedChangeSetNameChanged! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/25/2004 15:53'!
selectedClass
	"Answer the currently-selected class.  If there is no selection, or if the selection refers to a class no longer extant, return nil"
	| c |
	^ selectedClassName ifNotNil: [(c := self selectedClassOrMetaClass)
		ifNotNil: [c theNonMetaClass]]! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:52'!
selectedClassName
	"Answer the selectedClassName of the receiver"
	<bewareOf: #selectedClassNameChanged>
	^self propertyValueAt: #selectedClassName! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:52'!
selectedClassName: aValue
	"Modify the receiver's selectedClassName"
	^self propertyValueAt: #selectedClassName put: aValue with: #selectedClassNameChanged! !

!CChangeSorter methodsFor: 'class list' stamp: 'md 4/25/2004 13:22'!
selectedClassOrMetaClass
	"Careful, the class may have been removed!!"

	| cName |
	selectedClassName ifNil: [^ nil].
	(selectedClassName endsWith: ' class')
		ifTrue: [cName := (selectedClassName copyFrom: 1 to: selectedClassName size-6) asSymbol.
				^ (Smalltalk at: cName ifAbsent: [^nil]) class]
		ifFalse: [cName := selectedClassName asSymbol.
				^ Smalltalk at: cName ifAbsent: [nil]]! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:39'!
selectedMessage
	"Answer the selectedMessage of the receiver"
	<bewareOf: #selectedMessageChanged>
	^self propertyValueAt: #selectedMessage! !

!CChangeSorter methodsFor: 'field access' stamp: 'md 4/25/2004 12:39'!
selectedMessage: aValue
	"Modify the receiver's selectedMessage"
	^self propertyValueAt: #selectedMessage put: aValue with: #selectedMessageChanged! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/25/2004 13:23'!
selectedMessageName

	selectedMessage ifNil: [^ nil].
	^ selectedMessage asSymbol! !

!CChangeSorter methodsFor: 'code pane' stamp: 'md 4/25/2004 13:25'!
setContents
	"return the source code that shows in the bottom pane"

	| sel class strm changeType contents contentsSymbol |
	selectedClassName ifNil: [	myChangeSet ifNil: [^''].
								^ myChangeSet preambleString ifNil: ['']].
	class := self selectedClassOrMetaClass.
	(sel := selectedMessage) == nil
		ifFalse: [changeType := (myChangeSet atSelector: (sel := sel asSymbol) class: class).
			changeType == #remove
				ifTrue: [^   'Method has been removed (see versions)'].
			changeType == #addedThenRemoved
				ifTrue: [^  'Added then removed (see versions)'].
			class ifNil: [^   'Method was added, but cannot be found!!'].
			(class includesSelector: sel)
				ifFalse: [^  'Method was added, but cannot be found!!'].
			contents := class sourceCodeAt: sel.
			(#(prettyPrint colorPrint prettyDiffs altSyntax) includes: contents) ifTrue:
				[contents := class compilerClass new
					format: contents in: class notifying: nil contentsSymbol: contentsSymbol].
			"self showingAnyKindOfDiffs" false
				ifTrue: [contents := self diffFromPriorSourceFor: contents].
			^ contents asText makeSelectorBoldIn: class]
		ifTrue: [strm := WriteStream on: (String new: 100).
			(myChangeSet classChangeAt: selectedClassName) do:
				[:each |
				each = #remove ifTrue: [strm nextPutAll: 'Entire class was removed.'; cr].
				each = #addedThenRemoved ifTrue: [strm nextPutAll: 'Class was added then removed.'].
				each = #rename ifTrue: [strm nextPutAll: 'Class name was changed.'; cr].
				each = #add ifTrue: [strm nextPutAll: 'Class definition was added.'; cr].
				each = #change ifTrue: [strm nextPutAll: 'Class definition was changed.'; cr].
				each = #reorganize ifTrue: [strm nextPutAll: 'Class organization was changed.'; cr].
				each = #comment ifTrue: [strm nextPutAll: 'New class comment.'; cr.
				]].
			^ strm contents].! !

!CChangeSorter methodsFor: 'creation' stamp: 'md 4/6/2004 16:49'!
setDefaultChangeSetCategory
	"Set a default ChangeSetCategory for the receiver, and answer it"

	^ changeSetCategory := self class changeSetCategoryNamed: #All! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:02'!
setRecentUpdatesMarker
	"Allow the user to change the recent-updates marker"

	| result |
	result := self request: 
('Enter the lowest change-set number
that you wish to consider "recent"?
(note: highest change-set number
in this image at this time is ', self class highestNumberedChangeSet asString, ')') initialAnswer: self class recentUpdateMarker asString.
	(result notNil and: [result startsWithDigit]) ifTrue:
		[self class recentUpdateMarker: result asInteger].! !

!CChangeSorter methodsFor: 'initialize' stamp: 'ar 5/26/2005 20:23'!
setupCostume
	"Create the default costume"
	super setupCostume.
	self extent: 360@360.
	self layout: CProportionalLayout new.

	changeSetListPane color: Color transparent.
	changeSetListPane layoutFrame: (0@0 extent: 0.5@0.25).
	self add: changeSetListPane.

	classListPane color: Color transparent.
	classListPane layoutFrame: (0.5@0 extent: 0.5@0.25).
	self add: classListPane.

	methodListPane color: Color transparent.
	methodListPane layoutFrame: (0@0.25 extent: 1@0.25).
	self add: methodListPane.


	codePane color: Color transparent.
	codePane layoutFrame: (0@0.5 corner: 1@1).
	codePane hResizing: #spaceFill; vResizing: #spaceFill.
	codePane textWrap: true.
	codePane hScrollable: false; vScrollable: true.
	self add: codePane.



	"Give all of my contents the default look"
	self decorate: elements color: self defaultWindowColor.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 16:04'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' target: self action: #browseMethodConflicts.
	"aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'."
	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' target: self action: #methodConflictsWithOtherSide.
			"aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'."

			aMenu add: 'conflicts with category opposite' target: self action: #methodConflictsWithOppositeCategory.
			"aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in ANY change set in the category list on the opposite side of this change sorter, other of course than this change set itself.  (Caution -- this could be VERY slow)'"].
	aMenu addSeparator.

	"CHECKS SECTION"
	aMenu add: 'check for slips' target: self action: #lookForSlips.
	"aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'."

	aMenu add: 'check for unsent messages' target: self action: #checkForUnsentMessages.
	"aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'."

	aMenu add: 'check for uncommented methods' target: self action: #checkForUncommentedMethods.
	"aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'."

	aMenu add: 'check for uncommented classes' target: self action: #checkForUncommentedClasses.
	"aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'."

	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' target: self action: #checkForAlienAuthorship.
		"aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'".

	aMenu add: 'check for any other authors' target: self action: #checkForAnyAlienAuthorship.
	"aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Utilities authorInitials, '"'"].

	aMenu add: 'check for uncategorized methods' target: self action: #checkForUnclassifiedMethods.
	"aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'".
	aMenu addSeparator.

	aMenu add: 'inspect change set' target: self action: #inspectChangeSet.
	"aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'."


	aMenu add: 'go to change set''s project' target: self action: #goToChangeSetsProject.
	"aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'."

	aMenu add: 'promote to top of list' target: self action: #promoteToTopChangeSet.
	"aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'."

	aMenu add: 'trim history' target: self action: #trimHistory.
	"aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'."

	aMenu add: 'clear this change set' target: self action: #clearChangeSet.
	"aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'."
	aMenu add: 'expunge uniclasses' target: self action: #expungeUniclasses.
	"aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'."

	aMenu add: 'uninstall this change set' target: self action: #uninstallChangeSet.
	"aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'."

	aMenu addSeparator.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	"aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'."

	^ aMenu! !

!CChangeSorter methodsFor: 'message list' stamp: 'md 4/23/2004 17:37'!
shiftedMessageMenu: aMenu
	"Arm the menu so that it holds items appropriate to the message-list while the shift key is down.  Answer the menu."

	^ aMenu addList: #(
		-
		('toggle diffing (D)'					toggleDiffing)
		('implementors of sent messages'		browseAllMessages)
			-
		('sample instance'					makeSampleInstance)
		('inspect instances'					inspectInstances)
		('inspect subinstances'				inspectSubInstances)
		-
		('change sets with this method'		findMethodInChangeSets)
		('revert to previous version'			revertToPreviousVersion)
		('revert & remove from changes'	revertAndForget)) target: self.! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 3/1/2005 13:38'!
showCategoriesOfChangeSet
	"Show a list of all the categories in which the selected change-set occurs at the moment.  Install the one the user chooses, if any."

	| aMenu item |

	aMenu := CMenu new.
	aMenu label: 
'Categories which
contain change set
"', myChangeSet name, '"'.

		self changeSetCategories elementsInOrder do: [:aCategory |
					(aCategory includesChangeSet: myChangeSet) ifTrue: [
							item := aMenu 
										add: aCategory categoryName
										target: self 
										action: #showChangeSetCategory: 
										argument: aCategory.
							item checked: aCategory == changeSetCategory].
						].
			aMenu run.! !

!CChangeSorter methodsFor: 'creation' stamp: 'md 4/25/2004 14:54'!
showChangeSet: aChangeSet

	self selectedChangeSetName: aChangeSet name.
	changeSetListPane contents: self changeSetList.
	changeSetListPane selection: aChangeSet name asSymbol.
	self onSelectedChangeSetNameChanged.
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 11/23/2004 19:28'!
showChangeSetCategory: aChangeSetCategory
	"Show the given change-set category"
	
	changeSetCategory := aChangeSetCategory.
	self changed: #changeSetList.
	(self changeSetList includes: myChangeSet name) ifFalse:
			[self showChangeSet: (self changeSetNamed: self changeSetList first)].! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:03'!
submergeIntoOtherSide
	"Copy the contents of the receiver to the other side, then remove the receiver -- all after checking that all is well."
	| other message nextToView i |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ self].
	other := (parent other: self) changeSet.
	other == myChangeSet ifTrue: [^ self inform: 'Both sides are the same!!'].
	myChangeSet isEmpty ifTrue: [^ self inform: 'Nothing to copy.  To remove,
simply choose "remove".'].

	myChangeSet okayToRemove ifFalse: [^ self].
	message := 'Please confirm:  copy all changes
in "', myChangeSet name, '" into "', other name, '"
and then destroy the change set
named "', myChangeSet name, '"?'.
 
	(self confirm: message) ifFalse: [^ self].

	(myChangeSet hasPreamble or: [myChangeSet hasPostscript]) ifTrue:
		[(self confirm: 
'Caution!!  This change set has a preamble or
a postscript or both.  If you submerge it into
the other side, these will be lost.
Do you really want to go ahead with this?') ifFalse: [^ self]].

	other assimilateAllChangesFoundIn: myChangeSet.

	nextToView := ((AllChangeSets includes: myChangeSet)
		and: [(i := AllChangeSets indexOf: myChangeSet) < AllChangeSets size])
		ifTrue: [AllChangeSets at: i+1]
		ifFalse: [other].

	self removePrompting: false.
	self showChangeSet: nextToView.
	! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
subtractOtherSide
	"Subtract the changes found on the other side from the requesting side."
	self checkThatSidesDiffer: [^ self].
	myChangeSet forgetAllChangesFoundIn: ((parent other: self) changeSet).
	self showChangeSet: myChangeSet! !

!CChangeSorter methodsFor: 'code pane' stamp: 'md 4/23/2004 17:25'!
toggleDiffing
	"Toggle whether diffs should be shown in the code pane"

	self cNotYet.

	"self okToChange ifTrue:
		[super toggleDiffing.
		self changed: #contents.
		self update]

	"! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/6/2004 16:49'!
trimHistory
	"Drop non-essential history (rename, reorg, method removals) from newly-added classes."

	myChangeSet trimHistory

! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/5/2005 18:03'!
uninstallChangeSet
	"Attempt to uninstall the current change set, after confirmation."

	self okToChange ifFalse: [^ self].
	(self confirm: 'Uninstalling a changeSet is unreliable at best.
It will only work if the changeSet consists only of single
changes, additions and removals of methods, and if
no subsequent changes have been to any of them.
No changes to classes will be undone.
The changeSet will be cleared after uninstallation.
Do you still wish to attempt to uninstall this changeSet?')
	ifFalse: [^ self].

	myChangeSet uninstall.
	self update.
! !

!CChangeSorter methodsFor: 'changeSet menu' stamp: 'md 4/25/2004 14:23'!
update
	"recompute all of my panes"
 
	self showChangeSet: myChangeSet.
	parent ifNotNil: [(parent other: self) update]! !

!CChangeSorter methodsFor: 'code pane' stamp: 'md 4/6/2004 16:49'!
wantsOptionalButtons
	"No optional buttons for ChangeSorter"
	^false! !

CCodeHolder subclass: #CClassBrowser
	instanceVariableNames: ''
	classVariableNames: 'EventMap'
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CClassBrowser commentStamp: 'tk 2/28/2005 14:17' prior: 0!
A Class Browser shows the classes and methods in Tweak.  Users who make objects do things with scripts do not need to use a Class Browser.  A class describes a single kind of object, and it can have many instances.  An instance is an object that is a member of a class.  The class browser shows both classes in Tweak and the underlying classes in Squeak.  Methods with code can be edited and created in the bottom pane.  Get a Browser from the Tools menu in the menu bar.[general]


### Allow Users to see the above explanation ###!

CClassBrowser defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="selectedClassName" toSet="selectedClassName:" changeEvent="selectedClassNameChanged">selectedClassName</field>
	<field toGet="selectedMessage" toSet="selectedMessage:" changeEvent="selectedMessageChanged">selectedMessage</field>
	<field toGet="selectedMessageCategory" toSet="selectedMessageCategory:" changeEvent="selectedMessageCategoryChanged">selectedMessageCategory</field>
	<field toGet="selectedSystemCategory" toSet="selectedSystemCategory:" changeEvent="selectedSystemCategoryChanged">selectedSystemCategory</field>
	<field toGet="systemCategoryList" toSet="systemCategoryList:" changeEvent="systemCategoryListChanged">systemCategoryList</field>
	<field toGet="classList" toSet="classList:" changeEvent="classListChanged">classList</field>
	<field toGet="messageCategoryList" toSet="messageCategoryList:" changeEvent="messageCategoryListChanged">messageCategoryList</field>
	<field toGet="messageList" toSet="messageList:" changeEvent="messageListChanged">messageList</field>
	<field toGet="classButton" toSet="classButton:" changeEvent="classButtonChanged">classButton</field>
	<field toGet="instanceButton" toSet="instanceButton:" changeEvent="instanceButtonChanged">instanceButton</field>
	<field toGet="commentButton" toSet="commentButton:" changeEvent="commentButtonChanged">commentButton</field>
	<field toGet="dialect" toSet="dialect:" changeEvent="dialectChanged">dialect</field>
</fields>'!

!CClassBrowser class methodsFor: 'class initialization' stamp: 'ar 4/24/2005 18:05'!
initialize
	"CClassBrowser initialize"
	self registerOpenCommand.
	self initializeEventMap.! !

!CClassBrowser class methodsFor: 'class initialization' stamp: 'ar 4/24/2005 18:05'!
initializeEventMap
	"Initialize the event map"
	EventMap := Dictionary new.
	"Mouse events"
	#(
		mouseDown mouseEnter mouseMove mouseStillDown mouseLeave mouseUp
		mouseClick mouseDoubleClick mouseDrag
		dragEnter dragMove dragLeave dragDrop
		dropAccept dropReject
		redButtonDown redButtonMove redButtonUp
		yellowButtonDown yellowButtonMove yellowButtonUp
		blueButtonDown blueButtonMove blueButtonUp
	) do:[:event| EventMap at: event put: TweakBrowser mouseEventIcon].

	"Keyboard events"
	#(
		keyboardFocusEnter keyboardFocusLeave
		keyDown keyUp keyStroke
	) do:[:event| EventMap at: event put: TweakBrowser keyboardEventIcon].

	"Structure events"
	#(
		structureChanged frameChanged 
		layoutChanged containerLayoutChanged
		open close
		opened closed "remove those"
	) do:[:event| EventMap at: event put: TweakBrowser structureEventIcon].

	"Window events"
	#(
		closeRequest expandRequest collapseRequest activate passivate
	) do:[:event| EventMap at: event put: TweakBrowser windowEventIcon].

	"Text events"
	#(
		textModified textSelectionChanged save accept revert
	) do:[:event| EventMap at: event put: TweakBrowser textEventIcon].

	"Tick events"
	#(
		tick
	) do:[:event| EventMap at: event put: TweakBrowser tickEventIcon].

	"Button events"
	#(
		fire
	) do:[:event| EventMap at: event put: TweakBrowser buttonEventIcon].

	"Collection and list events"
	#(
		changed cursorChanged wrapCursorChanged
	) do:[:event| EventMap at: event put: TweakBrowser listEventIcon].
! !

!CClassBrowser class methodsFor: 'class initialization' stamp: 'ar 1/22/2005 22:45'!
registerOpenCommand
	"CClassBrowser initialize"
	Smalltalk at: #CProjectBuilder ifPresent:[:pb|
		pb registerOpenCommand: { 'Class Browser' . { self . #open }. 'A Class Browser' }.
	].! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'tk 1/5/2005 20:33'!
addField
	"Add a new instance field"
	| fieldName aClass toGet toSet |
	aClass := self selectedClass ifNil:[^self].
	fieldName := self request:'Field name?'.
	fieldName isEmptyOrNil ifTrue:[^self].
	fieldName := fieldName asSymbol.
	(aClass allVariableNames anySatisfy:[:varName| varName = fieldName]) ifTrue:[
		(self confirm: 'A field named "', fieldName, '" already exists. Proceed?') ifFalse:[^nil].
	].
	toGet := fieldName.
	toSet := (fieldName,':') asSymbol.
	"Check for conflicting methods"
	(aClass includesSelector: toGet) ifTrue:[
		(self confirm: 'A method with the name "', toGet,'" already exists.
Proceed?') ifFalse:[^self].
	].
	(aClass includesSelector: toSet) ifTrue:[
		(self confirm: 'A method with the name "', toSet,'" already exists.
Proceed?') ifFalse:[^self].
	].
	((aClass canUnderstand: toGet) or:[aClass canUnderstand: toSet]) ifTrue:[
		| proceed |
		proceed := self confirm: 'The definition of "', fieldName, '" will override \methods which have been defined \(but may be invisible) in this object.\Do you wish to proceed?' withCRs.
		proceed ifFalse:[^nil].
	].
	aClass addField: fieldName.
! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'ar 1/5/2005 18:03'!
addMessageCategory
	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| reject cats newName aClass list newToken msgMenu |
	aClass := self selectedClassOrMetaClass ifNil:[^self].
	self okToChange ifFalse: [^ self].
	list := Array new writeStream.
	newToken := Object new. "any token that is unique"
	list nextPut: {'new...'. newToken}.
	list nextPut: #-.
	reject := Set new.
	reject
		addAll: aClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	self selectedClassOrMetaClass allSuperclasses do: [:cls |
		cls = Object ifFalse: [
			cats := cls organization categories reject:[:cat | reject includes: cat].
			cats isEmpty ifFalse: [
				cats asSortedCollection do:[:cc| list nextPut: {cc. cc}].
				list nextPut: #-.
				reject addAll: cats.
			].
		].
	].
	msgMenu := CMenu new.
	msgMenu label: 'Add Category'.
	newName := msgMenu chooseFrom: list contents.
	newName == newToken ifTrue:[
		newName := self request: 'Please type new category name' initialAnswer: 'category name'.
	].
	newName isEmpty ifTrue: [^ self].
	newName := newName asSymbol.
	self selectedClassOrMetaClass organization
		addCategory: newName
		before: self selectedMessageCategory.
	self onSelectedClassNameChanged.
	messageCategoryList selection: newName.
! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 1/5/2005 18:03'!
addSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| newName |
	self okToChange ifFalse: [^ self].
	newName := self
		request: 'Please type new category name'
		initialAnswer: 'Category-Name'.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	self systemOrganizer
		addCategory: newName
		before: systemCategoryList selection.
	self updateSystemCategories.
	systemCategoryList selection: newName.! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'md 4/23/2004 14:17'!
alphabetizeMessageCategories
	self okToChange ifFalse: [^ false].
	ChangeSet current reorganizeClass: self selectedClassOrMetaClass.
	self selectedClassOrMetaClass organization sortCategories.
	self onSelectedClassNameChanged.
	^ true! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:09'!
browseAllClasses
	self flash.! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'md 5/3/2004 14:06'!
browseAllMessages
	"Create and schedule a message set browser on all implementors of all the messages sent by the current method."

	| aClass aName method filteredList |
	(aName := self selectedSelector) ifNotNil: [
		method := (aClass := self selectedClassOrMetaClass) compiledMethodAt: aName.
		filteredList := method messages reject: 
			[:each | #(new initialize = ) includes: each].
		self browseAllImplementorsOfList: filteredList asSortedCollection
			 title: 'All messages sent in ', aClass name, '.', aName]
! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 3/7/2004 01:42'!
browseMethodVersions
	"Create and schedule a Versions Browser, showing all versions of the currently selected message."
	^self flash.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:09'!
browseSystemCategory
	self flash! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 4/29/2004 13:10'!
classButton
	"Answer the classButton of the receiver"
	<bewareOf: #classButtonChanged>
	^self propertyValueAt: #classButton! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 4/29/2004 13:10'!
classButton: aValue
	"Modify the receiver's classButton"
	^self propertyValueAt: #classButton put: aValue with: #classButtonChanged! !

!CClassBrowser methodsFor: 'comments' stamp: 'md 5/1/2004 15:51'!
classComment

	| theClass comment |
	(theClass := self selectedClass) ifNil: [^ ''].	
	comment := theClass comment.
	comment isEmpty ifTrue: [comment := 'This class has not yet been commented.'].
	^comment

! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
classList
	"Answer the classList of the receiver"
	<bewareOf: #classListChanged>
	^self propertyValueAt: #classList! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
classList: aValue
	"Modify the receiver's classList"
	^self propertyValueAt: #classList put: aValue with: #classListChanged! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'ar 3/7/2004 03:14'!
classListKeyMap
	"Set up the keymap to apply to the receiver's class list"
	| map |
	map := CKeyboardDispatchTable new.
	map add: #(ctrl B) action: #browseMethodFull.
	map add: #(ctrl H) action: #spawnHierarchy.
	map add: #(ctrl P) action: #browseFullProtocol.
	map add: #(ctrl shift N) action: #browseClassRefs.
	map add: #(ctrl X) action: #removeClass.
	^map! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'ar 3/27/2004 20:48'!
classListMenu
	"Set up the menu to apply to the receiver's class list, honoring the #shifted boolean"
	| classMenu |
	classMenu := CMenu new.
	classMenu label: 'Class menu'.
	classMenu addList: #(
		('add field'					addField)
		-
		('browse full (b)'			browseMethodFull)
		('browse hierarchy (h)'		spawnHierarchy)
		('browse protocol (p)'		browseFullProtocol)
		-
		('printOut'					printOutClass)
		('fileOut'					fileOutClass)
		-
		('show hierarchy'			hierarchy)
		('show definition'			editClass)
		('show comment'			editComment)
		-
		('inst var refs...'			browseInstVarRefs)
		('inst var defs...'			browseInstVarDefs)
		-
		('class var refs...'			browseClassVarRefs)
		('class refs (N)'				browseClassRefs)
		-
		('rename class ...'			renameClass)
		('copy class'				copyClass)
		('remove class (x)'			removeClass)
		-
		('find method...'				findMethod)
		-
		('more...'					offerShiftedClassListMenu)
	) target: self.
	^classMenu! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 1/22/2005 14:32'!
classNotFound
	codePane flash.! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 6/2/2004 16:51'!
codePaneContents
	"Answer the current contents of the code pane"
	self commentIndicated ifTrue: [^self classComment].

	self selectedSystemCategory 
		ifNil:[^''].
	self selectedClassName 
		ifNil:[^Class template: self selectedSystemCategory].
	self selectedMessageCategory
		ifNil:[^self selectedClass browserDefinition: #default].
	self selectedMessage
		ifNil:[^self selectedClass sourceCodeTemplate].
	^(self selectedClassOrMetaClass sourceCodeAt: self selectedMessage ifAbsent:['']) asText
				makeSelectorBoldIn: self selectedClass.
! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 5/1/2004 15:27'!
commentButton
	"Answer the commentButton of the receiver"
	<bewareOf: #commentButtonChanged>
	^self propertyValueAt: #commentButton! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 5/1/2004 15:27'!
commentButton: aValue
	"Modify the receiver's commentButton"
	^self propertyValueAt: #commentButton put: aValue with: #commentButtonChanged! !

!CClassBrowser methodsFor: 'comments' stamp: 'md 5/1/2004 15:52'!
commentIndicated
	^commentButton selected.! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 3/27/2004 20:48'!
confirmRemovalOf: aSelector in: aClass 
	"Determine if it is okay to remove the given selector. Answer true if it  should be removed, nil if it should be removed followed by a senders  browse, and false if it should not be removed."
	| count answer allCalls aMenu |
	allCalls := self allCallsOn: aSelector.
	count := allCalls size.
	count = 0 ifTrue: [^true]. "no senders"
	count == 1 ifTrue: [
		"If only one sender see if it it's sent by itself"
		(allCalls first actualClass == aClass 
			and: [allCalls first methodSymbol == aSelector])
				ifTrue: [^true]].
	aMenu := CMenu new.
	aMenu label: 'This message has ' , count , ' sender(s)'.
	answer := aMenu chooseFrom:#(
		('Remove it' 1)
		('Remove, then browse senders' 2)
		('Don''t remove, but show me those senders' 3)
		('Forget it -- do nothing -- sorry I asked' 4)
	).
	answer = 1 ifTrue:[^true].
	answer = 2 ifTrue:[^nil].
	answer = 3 ifTrue: [
		self
			browseMessageList: allCalls
			name: 'Senders of ' , aSelector
			autoSelect: aSelector keywords first].
	^false! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'ar 1/5/2005 18:03'!
copyClass
	| oldDefinition newDefinition oldClass newName newClass |
	oldClass := self selectedClass ifNil:[^self].
	self okToChange ifFalse: [^ self].
	newName := self request: 'Please type new class name' initialAnswer: oldClass name.
	newName isEmptyOrNil ifTrue: [^ self].
	newName := newName asSymbol.
	newName = oldClass name ifTrue: [^ self].
	(Smalltalk includesKey: newName)
		ifTrue: [^ self inform: newName , ' already exists'].
	oldDefinition := oldClass definition.
	newDefinition := oldDefinition 
		copyReplaceAll: '#' , oldClass name asString 
		with: '#' , newName asString.
	Cursor wait showWhile: [
		newClass := CCompiler evaluate: newDefinition logged: true.
		newClass copyAllCategoriesFrom: oldClass.
		newClass class copyAllCategoriesFrom: oldClass class.
	].
	self selectClass: newClass.! !

!CClassBrowser methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:25'!
defaultWindowColor
	^Color r: 0.652 g: 0.8 b: 0.504! !

!CClassBrowser methodsFor: 'actions-codepane' stamp: 'ar 1/5/2005 18:03'!
defineClass: defString
	"The receiver's textual content is a request to define a new class. The
	source code is defString. If any errors occur in compilation, notify
	aController."
	| oldClass class newClassName defTokens keywdIx envt |
	oldClass := self selectedClassOrMetaClass.
	defTokens := defString findTokens: Character separators.
	keywdIx := defTokens findFirst: [:t | t beginsWith: 'category'].
	envt := Smalltalk environmentForCategory: ((defTokens at: keywdIx+1) copyWithout: $').
	keywdIx := defTokens findFirst: [:t | '*subclass*' match: t].
	newClassName := (defTokens at: keywdIx+1) copyWithoutAll: '#()'.
	((oldClass isNil or: [oldClass name asString ~= newClassName])
		and: [envt includesKeyOrAbove: newClassName asSymbol]) ifTrue:
			["Attempting to define new class over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newClassName , ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newClassName size))
				ifFalse: [^ false]].
	"ar 8/29/1999: Use oldClass superclass for defining oldClass
	since oldClass superclass knows the definerClass of oldClass."
	oldClass ifNotNil:[oldClass := oldClass superclass].
	class := oldClass subclassDefinerClass
				evaluate: defString
				notifying: self
				logged: true.
	(class isKindOf: Behavior)
		ifTrue: [
				self onSelectedSystemCategoryChanged.
				classList selection:
						((class isKindOf: Metaclass)
							ifTrue: [class soleInstance name asSymbol]
							ifFalse: [class name asSymbol]).
				self selectedClassName: classList selection.
				^ true]
		ifFalse: [^ false]! !

!CClassBrowser methodsFor: 'comments' stamp: 'ar 1/22/2005 14:32'!
defineComment: aText
	| theClass |

	theClass := self selectedClass.
	theClass ifNil: [self 
						inform: 'You must select a class\before giving it a comment.' withCRs.
					^ false].
	theClass comment: aText stamp: Utilities changeStamp.
	codePane contents: aText copy.
	^ true! !

!CClassBrowser methodsFor: 'actions-codepane' stamp: 'ar 1/22/2005 14:32'!
defineMessageFrom: aString
	"Compile the expressions in aString. Install the compiled method in the selected class 
	classified under  the currently selected message category name. Answer the selector 
	obtained if compilation succeeds, nil otherwise."

	| selectedMessageName selector category oldMessageList |
	

	selectedMessageName := self selectedMessage.
	oldMessageList := self messageList.

	selector := (Parser new parseSelector: aString).

"	(self metaClassIndicated
		and: [(self selectedClassOrMetaClass includesSelector: selector) not
		and: [Metaclass isScarySelector: selector]])
		ifTrue: [""A frist-time definition overlaps the protocol of Metaclasses""
				(self confirm: ((selector , ' is used in the existing class system.
Overriding it could cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: selector size))
				ifFalse: [^nil]]."

	selector := self selectedClassOrMetaClass
				compile: aString
				classified: (category := self selectedMessageCategory)
				notifying: self.
	selector == nil ifTrue: [^ nil].
	codePane contents: aString copy.
	selector ~~ selectedMessageName 
		ifTrue: 
			[category = ClassOrganizer nullCategory
				ifTrue: [self onSelectedClassNameChanged.
						messageCategoryList selectionIndex: 1.
						self onSelectedMessageCategoryChanged].
			"self setClassOrganizer."  "In case organization not cached"
			(oldMessageList includes: selector)
				ifFalse: [self onSelectedMessageCategoryChanged].
			messageList selection: selector].
	^ selector! !

!CClassBrowser methodsFor: 'accessing' stamp: 'tk 3/24/2005 15:43'!
dialect
	"Answer the dialect of the receiver"
	<bewareOf: #dialectChanged>
	^self propertyValueAt: #dialect! !

!CClassBrowser methodsFor: 'accessing' stamp: 'tk 3/24/2005 15:43'!
dialect: aValue
	"Modify the receiver's dialect"
	^self propertyValueAt: #dialect put: aValue with: #dialectChanged! !

!CClassBrowser methodsFor: 'evaluating' stamp: 'ar 3/31/2004 14:44'!
doItReceiver
	"This class's classPool has been jimmied to be the classPool of the class 
	being browsed. A doIt in the code pane will let the user see the value of 
	the class variables."
	^ self selectedClass ifNil: [FakeClassPool new]! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 6/19/2005 13:51'!
drawMessage: index in: itemBox on: aCanvas
	"Drawing support. Draw an item in the message list."
	| selector class trigger icon up down itemColor |
	class := self selectedClassOrMetaClass.
	selector := self messageList at: index.
	trigger := self methodTriggerFor: selector in: class.
	trigger ifNil:[
		icon := nil.
		up := self isThisAnOverride: selector in: class.
		down := self isThereAnOverride: selector in: class.
		(up and:[down]) 
			ifTrue:[icon := TweakBrowser bothArrowIcon]
			ifFalse:[
				up ifTrue:[icon := TweakBrowser upArrowIcon].
				down ifTrue:[icon := TweakBrowser downArrowIcon].
			].
	] ifNotNil:[
		icon := EventMap at: trigger ifAbsent:[nil].
		icon ifNil:[
			(trigger endsWith: 'changed' caseSensitive: false) 
				ifTrue:[icon := TweakBrowser changeEventIcon]
				ifFalse:[icon := TweakBrowser defaultEventIcon].
		].
	].

	icon ifNotNil:[
		aCanvas paintImage: icon at: itemBox leftCenter - (-2@icon height // 2)
	].

	itemColor := index = messageList selectionIndex 
			ifTrue:[Color white] 
			ifFalse:[Color black].
	aCanvas drawString: selector asString
			in: (itemBox insetBy: (15@1 corner: 0@1)) 
			font: messageList font 
			color: itemColor.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:10'!
editSystemCategories
	self flash.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:10'!
fileOutSystemCategory
	self flash.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'jrd 6/4/2009 17:45'!
findClass
	"Search for a class by name."
	| pattern foundClass classNames toMatch exactMatch potentialClassNames choice |
	self okToChange ifFalse: [^ self classNotFound].
	pattern := self uiManager request: 'Class name or fragment?'.
	pattern isEmptyOrNil ifTrue: [^ self classNotFound].
	toMatch := (pattern copyWithout: $.) asLowercase.
	potentialClassNames := self potentialClassNames asOrderedCollection.
	classNames := pattern last = $. 
		ifTrue: [potentialClassNames select:
					[:nm |  nm asLowercase = toMatch]]
		ifFalse: [potentialClassNames select: 
					[:n | n includesSubstring: toMatch caseSensitive: false]].
	classNames isEmpty ifTrue: [^ self classNotFound].
	exactMatch := classNames detect: [:each | each asLowercase = toMatch] ifNone: [nil].

	choice := classNames size = 1
		ifTrue:	[classNames first]
		ifFalse:	[(CMenu new) label: 'Select class';
					chooseFromValues: classNames default: exactMatch].
	choice ifNil:[^ self classNotFound].
	foundClass := Smalltalk at: choice asSymbol.
	self selectClass: foundClass! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'ar 1/5/2005 18:03'!
findMethod
	"Pop up a list of the current class's methods, and select the one chosen by the user"

	| aClass selectors reply msgMenu |
	aClass := self selectedClassOrMetaClass ifNil:[^self].
	self okToChange ifFalse: [^ self].
	selectors := aClass selectors asSortedArray.
	selectors isEmpty ifTrue: [^self inform: aClass name, ' has no methods.'].
	msgMenu := CMenu new.
	msgMenu label: 'Choose Selector'.
	reply := msgMenu chooseFromValues: selectors.
	reply ifNil:[^ self].
	self selectClass: aClass selector: reply.! !

!CClassBrowser methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:26'!
initialize
	"Initialize the receiver"
	super initialize.

	self define: #systemCategoryList as: CStringList new.
	systemCategoryList menu: self systemCategoryListMenu.
	systemCategoryList userDataAt: #keyMap put: self systemCategoryListKeyMap.
	systemCategoryList allowDeselect: true.

	self define: #classList as: CStringList new.
	classList menu: self classListMenu.
	classList userDataAt: #keyMap put: self classListKeyMap.
	classList allowDeselect: true.


	self define: #instanceButton as: CToolBarButton new.
	self define: #classButton as: CToolBarButton new.
	self define: #commentButton as: CToolBarButton new.

	self define: #messageCategoryList as: CStringList new.
	messageCategoryList menu: self messageCategoryListMenu.
	messageCategoryList userDataAt: #keyMap put: self messageCategoryListKeyMap.
	messageCategoryList allowDeselect: true.

	self define: #messageList as: CStringList new.
	messageList drawAction: (MessageSend receiver: self selector: #drawMessage:in:on: arguments: #(nil nil nil)).
	messageList menu: self messageListMenu.
	messageList userDataAt: #keyMap put: self messageListKeyMap.
	messageList allowDeselect: true.

	"Finally, add the default system categories"
	systemCategoryList contents: SystemOrganization categories.
	self signal: #updateWindowLabel.! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 4/29/2004 13:10'!
instanceButton
	"Answer the instanceButton of the receiver"
	<bewareOf: #instanceButtonChanged>
	^self propertyValueAt: #instanceButton! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 4/29/2004 13:10'!
instanceButton: aValue
	"Modify the receiver's instanceButton"
	^self propertyValueAt: #instanceButton put: aValue with: #instanceButtonChanged! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 4/24/2005 18:10'!
isThereAnOverride: selector in: aClass
	"Answer whether any subclass of my selected class implements my 
	selected selector"
	aClass allSubclassesDo: [ :cls | (cls includesSelector: selector) ifTrue: [ ^true ]].
	^ false! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 4/24/2005 18:10'!
isThisAnOverride: selector in: aClass
	"Answer whether any superclass of my selected class implements my selected selector"
	aClass allSuperclassesDo: [ :cls | (cls includesSelector: selector) ifTrue: [ ^true ]].
	^ false! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
messageCategoryList
	"Answer the messageCategoryList of the receiver"
	<bewareOf: #messageCategoryListChanged>
	^self propertyValueAt: #messageCategoryList! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
messageCategoryList: aValue
	"Modify the receiver's messageCategoryList"
	^self propertyValueAt: #messageCategoryList put: aValue with: #messageCategoryListChanged! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'ar 3/7/2004 03:05'!
messageCategoryListKeyMap
	"Set up the keymap to apply to the receiver's message category list"
	^nil! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'ar 3/27/2004 20:49'!
messageCategoryListMenu
	| aMenu |
	aMenu := CMenu new.
	aMenu label: 'Message Categories'.
	aMenu addList: #(
		('browse' browseMessageCategories)
		('printOut' printOutMessageCategories)
		('fileOut' fileOutMessageCategories)
		-
		('reorganize' editMessageCategories)
		('alphabetize' alphabetizeMessageCategories)
		('remove empty categories' removeEmptyCategories)
		('categorize all uncategorized' categorizeAllUncategorizedMethods)
		-
		('new category...' addMessageCategory)
		('rename category...' renameMessageCategory)
		('remove category' removeMessageCategory)
	) target: self.
	^aMenu! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
messageList
	"Answer the messageList of the receiver"
	<bewareOf: #messageListChanged>
	^self propertyValueAt: #messageList! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
messageList: aValue
	"Modify the receiver's messageList"
	^self propertyValueAt: #messageList put: aValue with: #messageListChanged! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'md 5/1/2004 14:24'!
messageListKeyMap
	"Answer the message-list menu"
	| map |
	map := CKeyboardDispatchTable new.
	map add: #(ctrl b) action: #browseMethodFull.
	map add: #(ctrl h) action: #browseClassHierarchy.
	map add: #(ctrl p) action: #browseFullProtocol.
	map add: #(ctrl o) action: #fileOutMessage.
	map add: #(cmd c) action: #copySelector.
	map add: #(cmd n) action: #browseSenders.
	map add: #(cmd m) action: #browseAllMessages.
	map add: #(ctrl i) action: #browseMethodHierary.
	map add: #(ctrl shift n) action: #browseClassRefs.
	map add: #(ctrl x) action: #removeMessage.
	^map! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 3/27/2004 20:49'!
messageListMenu
	"Answer the message-list menu"
	| aMenu |
	aMenu := CMenu new.
	aMenu label: 'Messages'.
	aMenu addList:#(
			('browse full (b)' 						browseMethodFull)
			('browse hierarchy (h)'					classHierarchy)
			('browse method (O)'					openSingleMessageBrowser)
			('browse protocol (p)'					browseFullProtocol)
			-
			('fileOut (o)'							fileOutMessage)
			('printOut'								printOutMessage)
			('copy selector (c)'						copySelector)
			-
			('senders of... (n)'						browseSenders)
			('implementors of... (m)'					browseImplementors)
			('inheritance (i)'						browseMethodHierarchy)
			('versions (v)'							browseMethodVersions)
			-
			('inst var refs...'						browseInstVarRefs)
			('inst var defs...'						browseInstVarDefs)
			('class var refs...'						browseClassVarRefs)
			('class refs (N)'							browseClassRefs)
			-
			('remove method (x)'					removeMessage)
	) target: self.
	^aMenu! !

!CClassBrowser methodsFor: 'metaclass' stamp: 'md 4/29/2004 13:46'!
metaClassIndicated
	^classButton selected.! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 4/24/2005 18:10'!
methodTriggerFor: sel in: myClass
	"This ought to be cached..."
	| lkupClass trigger |
	(myClass includesBehavior: CProtoObject) ifFalse:[^nil].
	lkupClass := myClass.
	[lkupClass == CProtoObject] whileFalse:[
		trigger := lkupClass methodTriggerAt: sel.
		trigger ifNotNil:[^trigger eventName].
		lkupClass := lkupClass superclass.
	].
	^nil! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 1/22/2005 14:32'!
onAcceptCodePane
	"Update the class list when the system category list's cursor changes"
	| str |
	<on: save in: codePane> 

	self commentIndicated ifTrue: [^self defineComment: codePane contents].

	str := codePane contents asString.
	self selectedClassName ifNil: [^self defineClass: str].
	self selectedMessageCategory ifNil: [^self defineClass: str].

	self defineMessageFrom: str.
! !

!CClassBrowser methodsFor: 'events' stamp: 'md 5/1/2004 15:54'!
onClassButton
	<on: fire in: classButton>
	classButton selected: classButton selected not.
	instanceButton selected: classButton selected not.
	commentButton selected: classButton selected not.
	self signal: #selectedClassNameChanged.! !

!CClassBrowser methodsFor: 'list events' stamp: 'tak 5/16/2005 15:39'!
onClassListCursorChanged
	"Update the message category list when the class list's cursor changes"
	<on: cursorChanged in: classList>
	(self okToChangeList: classList from: self selectedClassName)
		ifTrue:[self selectedClassName: classList selection.
				Browser addRecentClasses: classList selection]
! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 4/24/2005 18:19'!
onClassListKeyDown: evt
	"Update the message category list when the class list's cursor changes"
	<on: keyDown in: classList>
	^self onKeyDown: evt inList: classList! !

!CClassBrowser methodsFor: 'events' stamp: 'md 5/1/2004 15:58'!
onCommentButton
	<on: fire in: commentButton>
	commentButton selected: commentButton selected not.
	self signal: #selectedClassNameChanged.
	! !

!CClassBrowser methodsFor: 'events' stamp: 'tk 1/26/2005 12:09'!
onInstanceButton
	<on: fire in: instanceButton>
	instanceButton selected: instanceButton selected not.
	classButton selected: instanceButton selected not.
	commentButton selected: instanceButton selected not.
	self signal: #selectedClassNameChanged.! !

!CClassBrowser methodsFor: 'list events' stamp: 'md 5/1/2004 14:05'!
onKeyDown: event in: aWidget
	"Perform the action bound via the given key event in aWidget"
	| aMap action |
	aMap := (aWidget userDataAt: #keyMap) ifNil:[^self].
	action := aMap lookupEvent: event.
	action ifNotNil:[self perform: action].! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 4/24/2005 18:21'!
onKeyDown: event inList: aList
	"Perform the action bound via the given key event in aWidget"
	| aMap action keyChar index |
	aMap := (aList userDataAt: #keyMap).
	aMap ifNotNil:[
		action := aMap lookupEvent: event.
		action ifNotNil:[self perform: action].
	].
	event anyModifierKeyPressed ifTrue:[^self].

	keyChar := event keyCharacter asLowercase.
	(keyChar isLetter or:[keyChar isDigit]) ifFalse:[^self].

	"Search from cursor to end"
	index := aList cursor.
	[(index := index + 1) <= aList size] whileTrue:[
		(aList at: index) displayText first asLowercase = keyChar ifTrue:[^aList cursor: index].
	].

	"Search from start to cursor"
	index := 0.
	[(index := index + 1) <= aList cursor] whileTrue:[
		(aList at: index) displayText first asLowercase = keyChar ifTrue:[^aList cursor: index].
	].

	Smalltalk beep.! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 3/6/2004 21:23'!
onMessageCategoryCursorChanged
	"Update the message list when the message category list's cursor changes"
	<on: cursorChanged in: messageCategoryList>
	(self okToChangeList: messageCategoryList from: self selectedMessageCategory)
		ifTrue:[self selectedMessageCategory: messageCategoryList selection]
! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 4/24/2005 18:19'!
onMessageCategoryListKeyDown: evt
	<on: keyDown in: messageCategoryList>
	^self onKeyDown: evt inList: messageCategoryList! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 3/6/2004 21:22'!
onMessageListCursorChanged
	"Update the code pane when the message list's cursor changes"
	<on: cursorChanged in: messageList>
	(self okToChangeList: messageList from: self selectedMessage)
		ifTrue:[self selectedMessage: messageList selection]! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 4/24/2005 18:19'!
onMessageListKeyDown: evt
	"Update the class list when the system category list's cursor changes"
	<on: keyDown in: messageList>
	^self onKeyDown: evt inList: messageList! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 1/4/2005 21:52'!
onSelectedClassNameChanged
	"Get the current selection from the list"
	| selected |
	<on: selectedClassNameChanged>
	self selectedMessageCategory: nil.
	messageCategoryList selection: nil.

	self signal: #updateWindowLabel.
	self signal: #updateCodePane.
	self signal: #selectedClassNameUpdated.
	selected := self selectedClassOrMetaClass.
	"Update the class list accordingly"
	selected ifNil:[^messageCategoryList contents: #()].
	messageCategoryList contents: selected organization categories.
! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 1/4/2005 21:52'!
onSelectedMessageCategoryChanged
	| selected |
	<on: selectedMessageCategoryChanged>
	self signal: #updateCodePane.
	self signal: #selectedMessageCategoryUpdated.
	"Get the current selection from the list"
	selected := self selectedMessageCategory.
	"Update the class list accordingly"
	selected ifNil:[^messageList contents: #()].
	messageList contents: (self selectedClassOrMetaClass organization listAtCategoryNamed: selected)! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 3/6/2004 21:34'!
onSelectedMessageChanged
	<on: selectedMessageChanged>
	self signal: #updateCodePane.! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 1/4/2005 21:51'!
onSelectedSystemCategoryChanged
	| selected |
	<on: selectedSystemCategoryChanged>
	self signal: #updateWindowLabel.
	self signal: #updateCodePane.
	self signal: #selectedSystemCategoryUpdated.
	"Get the current selection from the list"
	selected := self selectedSystemCategory.
	"Update the class list accordingly"
	selected ifNil:[^classList contents: #()].
	classList contents: (SystemOrganization listAtCategoryNamed: selected).
! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 3/6/2004 21:23'!
onSystemCategoryListCursorChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: systemCategoryList>
	(self okToChangeList: systemCategoryList from: self selectedSystemCategory)
		ifTrue:[self selectedSystemCategory: systemCategoryList selection].! !

!CClassBrowser methodsFor: 'list events' stamp: 'ar 4/24/2005 18:19'!
onSystemCategoryListKeyDown: evt
	"Update the class list when the system category list's cursor changes"
	<on: keyDown in: systemCategoryList>
	^self onKeyDown: evt inList: systemCategoryList! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 1/22/2005 14:33'!
onUpdateCodePane
	"Update the contents of the code pane"
	codePane contents: self codePaneContents.! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 2/7/2005 01:03'!
onUpdateWindowLabel
	| selected |
	<on: updateWindowLabel>
	selected := self selectedClassName ifNil:[self selectedSystemCategory].
	selected ifNil:[^self label: 'System Browser'].
	self label: 'System Browser: ', selected.! !

!CClassBrowser methodsFor: 'events' stamp: 'ar 2/7/2005 01:04'!
onWindowCloseRequest
	<on: closeRequest>
	self okToChange ifFalse:[^self].
	self destroy. "bye, bye"! !

!CClassBrowser methodsFor: 'initialize' stamp: 'ar 12/18/2004 23:39'!
postOpen
	"Do the user some good and waste a few cycles in the background to create the menu costumes"
	self preLoad: systemCategoryList menu.
	self preLoad: classList menu.
	self preLoad: messageCategoryList menu.
	self preLoad: messageList menu.
	super postOpen.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 22:50'!
potentialClassNames
	"Answer the names of all the classes that could be viewed in this browser.  This hook is provided so that HierarchyBrowsers can indicate their restricted subset.  For generic Browsers, the entire list of classes known to Smalltalk is provided, though of course that really only is accurate in the case of full system browsers."

	^ Smalltalk classNames! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:09'!
printOutSystemCategory
	self flash.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'tak 5/16/2005 15:49'!
recentClass
	| menu choice foundClass |
	menu := CMenu new.
	menu label: 'Recent Classes'.
	choice := menu chooseFromValues: Browser recentClasses.
	foundClass := Smalltalk at: choice asSymbol.
	self selectClass: foundClass
! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'md 5/3/2004 14:13'!
removeEmptyCategories
	self okToChange ifFalse: [^ self].
	self selectedClassOrMetaClass organization removeEmptyCategories.
	self onSelectedClassNameChanged.
! !

!CClassBrowser methodsFor: 'actions-message list' stamp: 'ar 3/7/2004 01:53'!
removeMessage
	"If a message is selected, create a Confirmer so the user can verify that  
	the currently selected message should be removed from the system. If 
	so,  remove it."
	| messageName removal |
	messageName := self selectedMessage ifNil:[^self].
	self okToChange ifFalse: [^ self].
	removal := self confirmRemovalOf: messageName in: self selectedClassOrMetaClass.
	removal == false ifTrue:[^self].
	self selectedClassOrMetaClass removeSelector: messageName.
	removal ifNil:[self browseAllCallsOn: messageName].
	self onSelectedMessageCategoryChanged.! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'ar 1/5/2005 18:03'!
removeMessageCategory
	"If a message category is selected, create a Confirmer so the user can 
	verify that the currently selected message category should be removed
 	from the system. If so, remove it."
	| messageCategoryName |
	messageCategoryName := self selectedMessageCategory ifNil:[^self].
	self okToChange ifFalse: [^ self].
	(messageList size = 0 or: [self confirm: 'Are you sure you want to
remove this method category 
and all its methods?']) ifFalse:[^self].

	self selectedClassOrMetaClass removeCategory: messageCategoryName.
	self onSelectedClassNameChanged.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:17'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."
	self selectedSystemCategory ifNil:[^self].
	self okToChange ifFalse: [^ self].
	(classList size = 0 or: [self confirm: 'Are you sure you want to
remove this system category 
and all its classes?']) ifFalse:[^self].
	self systemOrganizer removeSystemCategory: self selectedSystemCategory.
	self updateSystemCategories.
	systemCategoryList selection: nil.
! !

!CClassBrowser methodsFor: 'actions-class list' stamp: 'ar 1/5/2005 18:03'!
renameClass
	| oldName newName obs aClass |
	self selectedClassName ifNil:[^self flash].
	self okToChange ifFalse: [^ self].
	oldName := self selectedClass name.
	newName := self request: 'Please type new class name' initialAnswer: oldName.
	newName isEmptyOrNil ifTrue: [^ self].
	newName := newName asSymbol.
	newName = oldName ifTrue: [^ self].
	(Smalltalk includesKey: newName)
		ifTrue: [^self inform: newName , ' already exists'].
	aClass := self selectedClass.
	aClass rename: newName.
	self selectClass: aClass.
	obs := self allCallsOn: (Smalltalk associationAt: newName).
	obs isEmpty ifFalse:[
		self
			browseMessageList: obs
			name: 'Obsolete References to ' , oldName
			autoSelect: oldName].! !

!CClassBrowser methodsFor: 'actions-message categories' stamp: 'ar 1/5/2005 18:03'!
renameMessageCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldName newName oldClass |
	oldClass := self selectedClassOrMetaClass ifNil:[^self].
	oldName := self selectedMessageCategory ifNil:[^self].
	self okToChange ifFalse: [^ self].
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmpty ifTrue: [^ self].
	newName := newName asSymbol.
	newName = oldName ifTrue: [^ self].
	ChangeSet current reorganizeClass: oldClass.
	oldClass organization
		renameCategory: oldName
		toBe: newName.
	self selectClass: oldClass selector: self selectedMessage.! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 1/5/2005 18:03'!
renameSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldName newName |
	oldName := self selectedSystemCategory ifNil:[^self].
	self okToChange ifFalse: [^ self].
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	oldName = newName ifTrue: [^ self].
	self systemOrganizer
		renameCategory: oldName
		toBe: newName.
	self updateSystemCategories.
	systemCategoryList selection: newName.
! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/6/2004 23:29'!
selectClass: theClass
	self selectClass: theClass selector: nil.! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 1/4/2005 21:53'!
selectClass: theClass selector: theSelector
	"Set everything so that theClass is selected"
	systemCategoryList selection: theClass category.
	self waitUntil: #selectedSystemCategoryUpdated.
	classList selection: theClass theNonMetaClass name.
	self waitUntil: #selectedClassNameUpdated.
	messageCategoryList selection: (theClass organization categoryOfElement: theSelector).
	self waitUntil: #selectedMessageCategoryUpdated.
	messageList selection: theSelector.! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/6/2004 19:59'!
selectedClass
	"Answer the currently selected class (not just its name)"
	^Smalltalk at: self selectedClassName ifAbsent:[nil]! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:02'!
selectedClassName
	"This method was automatically generated"
	<bewareOf: #selectedClassNameChanged>
	^self propertyValueAt: #selectedClassName! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:18'!
selectedClassName: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selectedClassName put: aValue with: #selectedClassNameChanged! !

!CClassBrowser methodsFor: 'metaclass' stamp: 'md 4/29/2004 12:27'!
selectedClassOrMetaClass
	"Answer the selected class or metaclass."

	| cls |
	self metaClassIndicated
		ifTrue: [^ (cls := self selectedClass) ifNil: [nil] ifNotNil: [cls class]]
		ifFalse: [^ self selectedClass]! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:02'!
selectedMessage
	"This method was automatically generated"
	<bewareOf: #selectedMessageChanged>
	^self propertyValueAt: #selectedMessage! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:18'!
selectedMessage: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selectedMessage put: aValue with: #selectedMessageChanged! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:03'!
selectedMessageCategory
	"This method was automatically generated"
	<bewareOf: #selectedMessageCategoryChanged>
	^self propertyValueAt: #selectedMessageCategory! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:18'!
selectedMessageCategory: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selectedMessageCategory put: aValue with: #selectedMessageCategoryChanged! !

!CClassBrowser methodsFor: 'accessing' stamp: 'md 5/1/2004 15:07'!
selectedMessageName
	^selectedMessage! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:03'!
selectedSystemCategory
	"This method was automatically generated"
	<bewareOf: #selectedSystemCategoryChanged>
	^self propertyValueAt: #selectedSystemCategory! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:18'!
selectedSystemCategory: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selectedSystemCategory put: aValue with: #selectedSystemCategoryChanged! !

!CClassBrowser methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:27'!
setupCostume
	"Create the default costume"
	super setupCostume.

	self extent: 550@350.

	systemCategoryList color: Color transparent.
	systemCategoryList layoutFrame: (0@0 extent: 0.25@0.5).
	self add: systemCategoryList.

	classList color: Color transparent.
	classList layoutFrame: (0.25@0 extent: 0.25@0.4).
	self add: classList.

	instanceButton label: 'instance'.
	instanceButton layoutFrame: (0.25@0.4 extent: 0.10@0.1).
	instanceButton hResizing: #spaceFill; vResizing: #spaceFill.
	self add: instanceButton.

	commentButton label: '?'.
	commentButton layoutFrame: (0.35@0.4 extent: 0.05@0.1).
	commentButton hResizing: #spaceFill; vResizing: #spaceFill.
	self add: commentButton.

	classButton label: 'class'.
	classButton layoutFrame: (0.40@0.4 extent: 0.10@0.1).
	classButton hResizing: #spaceFill; vResizing: #spaceFill.
	self add: classButton.

	messageCategoryList color: Color transparent.
	messageCategoryList layoutFrame: (0.5@0 extent: 0.25@0.5).
	self add: messageCategoryList.

	messageList color: Color transparent.
	messageList layoutFrame: (0.75@0 extent: 0.25@0.5).
	self add: messageList.

	codePane layoutFrame: (0@0.5 corner: 1@1).

	"Give all of my contents the default look"
	self decorate: elements color: self defaultWindowColor.! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
systemCategoryList
	"Answer the systemCategoryList of the receiver"
	<bewareOf: #systemCategoryListChanged>
	^self propertyValueAt: #systemCategoryList! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
systemCategoryList: aValue
	"Modify the receiver's systemCategoryList"
	^self propertyValueAt: #systemCategoryList put: aValue with: #systemCategoryListChanged! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'md 5/1/2004 14:24'!
systemCategoryListKeyMap
	"Set up the keymap to apply to the receiver's system category list"
	| map |
	map := CKeyboardDispatchTable new.
	map add: #(cmd f) action: #findClass.
	map add: #(cmd r) action: #recentClass.
	^map! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/17/2004 11:45'!
systemCategoryListMenu
	| menu |
	menu := CMenu new.
	menu label: 'System Categories'.
	menu addList: #(
		('find class... (f)' findClass)
		('recent classes... (r)' recentClass)
		-
		('browse all' browseAllClasses)
		('browse' browseSystemCategory)
		-
		('printOut' printOutSystemCategory)
		('fileOut' fileOutSystemCategory)
		-
		('reorganize' editSystemCategories)
		('update' updateSystemCategories)
		-
		('add category...' addSystemCategory)
		('rename category...' renameSystemCategory)
		('remove category' removeSystemCategory)
	) target: self.
	^menu! !

!CClassBrowser methodsFor: 'accessing' stamp: 'ar 3/6/2004 23:12'!
systemOrganizer
	^SystemOrganization! !

!CClassBrowser methodsFor: 'actions-system categories' stamp: 'ar 3/6/2004 23:10'!
updateSystemCategories
	"The class categories were changed in another browser. The receiver must 
	reorganize its lists based on these changes."

	self okToChange ifFalse: [^ self].
	systemCategoryList contents: SystemOrganization categories.! !

CCodeHolder defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="codePane" toSet="codePane:" changeEvent="codePaneChanged">codePane</field>
</fields>'!

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:54'!
allCallsOn: aLiteral 
	"Answer a Collection of all the methods that call on aLiteral even deeply embedded in 
	literal array."
	^SystemNavigation new allCallsOn: aLiteral! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:42'!
allImplementorsOf: aSelector
	^SystemNavigation default allImplementorsOf: aSelector! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:54'!
browseAllAccesses: aClass to: instVarName
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."
	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsAccess: instVarName) do: [:sel |
				sel == #DoIt ifFalse: [
					coll add: (
						MethodReference new
							setStandardClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self 
		browseMessageList: coll 
		name: 'Accesses to ' , instVarName 
		autoSelect: instVarName! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:52'!
browseAllCallsOn: aLiteral 
	"Create and schedule a message browser on each method that refers to aLiteral"
	(aLiteral isKindOf: LookupKey)
		ifTrue: [^ self
				browseMessageList: (self allCallsOn: aLiteral) asSortedCollection
				name: 'Users of ' , aLiteral key
				autoSelect: aLiteral key].
	self
		browseMessageList: (self allCallsOn: aLiteral) asSortedCollection
		name: 'Senders of ' , aLiteral
		autoSelect: aLiteral keywords first! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:57'!
browseAllImplementorsOf: selector 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is the argument, selector."
	^ self
		browseMessageList: (self allImplementorsOf: selector)
		name: 'Implementors of ' , selector
		autoSelect: nil.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 17:34'!
browseAllImplementorsOfList: selectorList title: aTitle 
	"Create and schedule a message browser on each method that implements 
	the message whose selector is in the argument selectorList."

	| implementorLists flattenedList |
	implementorLists := selectorList
				collect: [:each | self allImplementorsOf: each].
	flattenedList := SortedCollection new.
	implementorLists
		do: [:each | flattenedList addAll: each].
	Cursor normal show.
	self
		browseMessageList: flattenedList
		name: 'aTitle'
		autoSelect: nil.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 6/19/2005 13:42'!
browseAllSends: aClass of: messageA or: messageB labeled: aString
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."
	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			class selectorsAndMethodsDo:[:sel :meth|
				"very ugly ... and VERY fast"
				((meth pointsTo: messageA) or:[meth pointsTo: messageB]) ifTrue:[
					sel == #DoIt ifFalse: [
						coll add: (MethodReference new setStandardClass: class  methodSymbol: sel)
					]
				]
			].
		].
	].
	^ self 
		browseMessageList: coll 
		name: aString
		autoSelect: messageA! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseAllStores: aClass into: instVarName
	"Create and schedule a Message Set browser for all the receiver's methods 
	or any methods of a subclass/superclass that refer to the instance variable name."
	| coll |
	coll := OrderedCollection new.
	Cursor wait showWhile: [
		aClass withAllSubAndSuperclassesDo: [:class | 
			(class whichSelectorsStoreInto: instVarName) do: [:sel |
				sel == #DoIt ifFalse: [
					coll add: (
						MethodReference new
							setStandardClass: class 
							methodSymbol: sel
					)
				]
			]
		].
	].
	^ self
		browseMessageList: coll 
		name: 'Stores into ' , instVarName 
		autoSelect: instVarName! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseClassRefs
	| aClass |
	aClass := self selectedClass ifNil:[^self flash].
	self browseAllCallsOn: (Smalltalk associationAt: aClass theNonMetaClass name).! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseClassVarRefs
	self browseClassVarRefs: self selectedClass.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseClassVarRefs: aClass
	| binding |
	aClass ifNil:[^self].
	self chooseClassVar: aClass thenDo:[:varName| 
		aClass withAllSuperclasses do:[:cls|
			binding := cls classPool associationAt: varName ifAbsent:[nil].
			binding ifNotNil:[^self browseAllCallsOn: binding]]].! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 14:02'!
browseClassVariables
	"Browse the class variables of the selected class. 2/5/96 sw"
	| cls |
	cls := self selectedClass.
	cls
		ifNotNil: [self systemNavigation  browseClassVariables: cls]
! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 14:32'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.  If in mvc, an old-style protocol browser is opened instead."

	self cNotYet.

	"| objectToRepresent |
	Smalltalk isMorphic ifFalse: [^ self spawnProtocol].

	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	InstanceBrowser new openOnObject: objectToRepresent inWorld: ActiveWorld showingSelector: nil"! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/22/2004 16:32'!
browseImplementors
	self selectMessage: 'Implementors of ...' 
		andEvaluate:[:msg| self browseAllImplementorsOf: msg].! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseInstVarDefs
	self browseInstVarDefs: self selectedClassOrMetaClass.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 2/7/2005 00:51'!
browseInstVarDefs: aClass
	aClass ifNil:[^self flash].
	aClass chooseInstVarThenDo:[:varName| self browseAllStores: aClass into: varName]! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 3/31/2004 14:53'!
browseInstVarRefs
	self browseInstVarRefs: self selectedClassOrMetaClass.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 2/7/2005 00:52'!
browseInstVarRefs: aClass
	aClass ifNil:[^self flash].
	aClass chooseInstVarThenDo:[:varName| self browseAllAccesses: aClass to: varName]! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 1/5/2005 18:03'!
browseMessageList: aMessageList name: labelString autoSelect: autoSelectString
	"Create and schedule a MessageSet browser on the message list."
	| title sz |
	sz := aMessageList size.
	sz = 0 ifTrue:[^self inform: ('There are no\' , labelString) withCRs].
	sz > 1
		ifTrue:	[title := labelString, ' [', sz, ']']
		ifFalse:	[title := labelString].
	CMessageSet 
		openMessageList: aMessageList 
		name: title 
		autoSelect: autoSelectString! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/21/2004 10:41'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."
	| myClass browser |
	(myClass := self selectedClassOrMetaClass) ifNil:[^self].
	browser := CClassBrowser new.
	browser selectClass: myClass selector: self selectedMessage.
	browser open.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/22/2004 16:45'!
browseMethodHierarchy
	"Create and schedule a message set browser on all implementors of the 
	currently selected message selector. Do nothing if no message is selected."
	| list aClassNonMeta isMeta theClassOrMeta aClass selector |
	aClass := self selectedClassOrMetaClass ifNil: [^ self].
	selector := self selectedMessage ifNil: [^ self].
	aClassNonMeta := aClass theNonMetaClass.
	isMeta := aClassNonMeta ~~ aClass.
	list := OrderedCollection new.
	aClass allSuperclasses reverseDo:[:cl |
		(cl includesSelector: selector) ifTrue:[
			list addLast: (MethodReference new setStandardClass: cl methodSymbol: selector)]].
	aClassNonMeta allSubclassesWithLevelDo:[:cl :level |
		theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
		(theClassOrMeta includesSelector: selector) ifTrue:[
			list addLast: (MethodReference new 
				setStandardClass: theClassOrMeta methodSymbol: selector)]]
	 	startingLevel: 0.
	self browseMessageList: list
		name: 'Inheritance of ' , selector
		autoSelect: nil.
! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:34'!
browseSenders
	self selectMessage:'Senders of ...'
		andEvaluate:[:msg| self browseAllCallsOn: msg].! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 14:43'!
browseUnusedMethods
	| classes unsent messageList cls |
	(cls := self selectedClass)
		ifNil: [^ self].
	classes := Array with: cls with: cls class.
	unsent := Set new.
	classes
		do: [:c | unsent addAll: c selectors].
	unsent := self systemNavigation allUnSentMessagesIn: unsent.
	messageList := OrderedCollection new.
	classes
		do: [:c | (c selectors
				select: [:s | unsent includes: s]) asSortedCollection
				do: [:sel | messageList add: c name , ' ' , sel]].

	CMessageSet openMessageList: messageList name: 'Unsent Methods in ' , cls name.

	"self systemNavigation browseMessageList: messageList name: 'Unsent Methods in ' , cls name"! !

!CCodeHolder methodsFor: 'as yet unclassified' stamp: 'ar 1/5/2005 18:04'!
cNotYet
	(self confirm: 'not yet implemented. Edit?') ifTrue: [
		| class selector browser |
		class := thisContext sender  mclass.
		selector := thisContext sender  selector.
		browser := CClassBrowser new.
		browser selectClass: class selector: selector.
		browser open.
	].

	! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 1/5/2005 18:04'!
chooseClassVar: aClass thenDo: aBlock 
	"Put up a menu of all the instance variables in the selected class, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter.  If the list is 6 or larger, then offer an alphabetical formulation as an alternative. triggered by a 'show alphabetically' item at the top of the list."
	| vars count list varMenu result |
	(count := aClass allClassVarNames size) = 0 ifTrue: 
		[^ self inform: 'There are no
instance variables.'].

	list := Array new writeStream.
	(count > 5) ifTrue:[
		list nextPut: {'show alphabetically'. 'show alphabetically'}.
		list nextPut: #-.
	].
	aClass withAllSuperclasses reverseDo:[:class |
		vars := class classVarNames.
		vars do:[:var | list nextPut: {var. var} ].
		vars isEmpty ifFalse:[list nextPut: #-].
	].
	varMenu := CMenu new.
	varMenu label: 'Class variables in
', aClass name.
	result := varMenu chooseFrom: list contents.
	result = 'show alphabetically' ifFalse:[^aBlock value: result].
	self chooseClassVarAlphabetically: aClass thenDo: aBlock! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 12/14/2004 23:20'!
chooseFieldAlphabetically: aClass thenDo: aBlock
	"Put up a menu of all the instance variables in the receiver, presented in alphabetical order, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter."
	| allVars list varMenu result |
	allVars := aClass allFields asArray sort:[:f1 :f2| f1 name <= f2 name].
	allVars isEmpty ifTrue: [^ self inform: 'There are no fields'].
	list := allVars collect:[:each| {each name. each}].
	varMenu := CMenu new.
	varMenu label: 'Fields in ', aClass name.
	result := varMenu chooseFrom: list.
	^aBlock value: result! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 1/5/2005 18:04'!
chooseFieldName: aClass thenDo: aBlock 
	"Put up a menu of all the fields in the selected class, and when the user chooses one, evaluate aBlock with the chosen variable as its parameter.  If the list is 6 or larger, then offer an alphabetical formulation as an alternative. triggered by a 'show alphabetically' item at the top of the list."
	| vars count list varMenu result |
	(count := aClass allFields size) = 0 ifTrue: 
		[^ self inform: 'There are no fields.'].

	list := Array new writeStream.
	(count > 5) ifTrue:[
		list nextPut: {'show alphabetically'. 'show alphabetically'}.
		list nextPut: #-.
	].
	aClass withAllSuperclasses reverseDo:[:class |
		vars := (class inheritsFrom: CProtoObject) ifTrue:[
			class classFields reject:[:each| each isVirtualField] 
		] ifFalse:[#()].
		vars do:[:var | list nextPut: {var name. var} ].
		vars isEmpty ifFalse:[list nextPut: #-].
	].
	varMenu := CMenu new.
	varMenu label: 'Fields in ', aClass name.
	result := varMenu chooseFrom: list contents.
	result = 'show alphabetically' ifFalse:[^aBlock value: result].
	result ifNil:[^self flash].
	self chooseFieldAlphabetically: aClass thenDo: aBlock! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/22/2004 17:12'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self cNotYet.

	"Utilities spawnHierarchyForClass: self selectedClassOrMetaClass 
		selector: self selectedMessageName"
! !

!CCodeHolder methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
codePane
	"Answer the codePane of the receiver"
	<bewareOf: #codePaneChanged>
	^self propertyValueAt: #codePane! !

!CCodeHolder methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:27'!
codePane: aValue
	"Modify the receiver's codePane"
	^self propertyValueAt: #codePane put: aValue with: #codePaneChanged! !

!CCodeHolder methodsFor: 'accessing' stamp: 'ar 3/28/2004 01:33'!
contents
	^codePane contents! !

!CCodeHolder methodsFor: 'accessing' stamp: 'ar 3/28/2004 01:33'!
contents: newContents
	codePane contents: newContents! !

!CCodeHolder methodsFor: 'hacks' stamp: 'bf 11/29/2004 16:04'!
correctFrom: start to: stop with: aString
	"ParagraphEditor's selection is off by one"
	codePane
		selectFrom: start to: stop+1;
		paste: aString! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:24'!
defaultWindowColor
	^Color r: 0.67 g: 0.67 b: 0.67! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 3/31/2004 14:45'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 3/31/2004 14:44'!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a text selection."
	^nil! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'tak 12/21/2005 20:50'!
evaluateSelection: aPane
	| rcvr ctxt result code |
	aPane editor selectLineIfEmpty.
	FakeClassPool adopt: self selectedClass.  "Include pool vars if any"
	rcvr := self doItReceiver.
	ctxt := self doItContext.
	code := aPane selectionAsStream.
	result := [
		CCompiler new 
			evaluate: code
			in: ctxt
			to: rcvr
			notifying: self
			ifFail: [FakeClassPool adopt: nil. ^ #failedDoit]
	] 
		on: OutOfScopeNotification 
		do: [ :ex | ex resume: true].
	FakeClassPool adopt: nil.
	^ result! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:16'!
fileOutMessage
	"Put a description of the selected message on a file"

	self selectedMessageName ifNotNil:
		[Cursor write showWhile:
			[self selectedClassOrMetaClass fileOutMethod: self selectedMessageName]]! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 15:13'!
findMethodInChangeSets
	"Find and open a changeSet containing the current method."

	| aName |
	(aName := self selectedMessageName) ifNotNil: [
		CChangeSorter browseChangeSetsWithClass: self selectedClassOrMetaClass
					selector: aName]! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 2/7/2005 01:03'!
initialize
	super initialize.
	"Define our components"
	self define: #codePane as: CTextEditor new.! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:04'!
inspectInstances
	"Inspect all instances of the selected class."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[myClass theNonMetaClass inspectAllInstances]. 
! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:04'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil: [
		aClass theNonMetaClass inspectSubInstances].
! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:02'!
makeSampleInstance
	| aClass nonMetaClass anInstance |
	(aClass := self selectedClassOrMetaClass) ifNil: [^ self].
	nonMetaClass := aClass theNonMetaClass.
	anInstance := self sampleInstanceOfSelectedClass.
	(anInstance isNil and: [nonMetaClass ~~ UndefinedObject]) ifTrue: 
		[^ self inform: 'Sorry, cannot make an instance of ', nonMetaClass name].

	anInstance inspectWithLabel: 'An instance of ', nonMetaClass name.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:41'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self cNotYet.
	"Utilities methodHierarchyBrowserForClass: self selectedClassOrMetaClass 
			selector: self selectedMessageName"
! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 12/11/2009 21:32'!
newCodePaneMenu: aPane
	| menu item |
	menu := CMenu new.
	menu label: nil.
		item := menu add: 'Find' target: aPane action: #findText.
		item icon: MenuIcons smallFindIcon.
		item := menu add: 'Find Next' target: aPane action: #findNext.
		item icon: MenuIcons smallFindIcon.
		item := menu add: 'Find and Replace' target: aPane action: #replace.
		item icon: MenuIcons smallFindIcon.
	menu addSeparator.
		item := menu add: 'Undo' target: aPane action: #undo.
		item icon: MenuIcons smallUndoIcon.
		item := menu add: 'Redo' target: aPane action: #undo.
		item icon: MenuIcons smallRedoIcon.
	menu addSeparator.
		item := menu add: 'Copy' target: aPane action: #copySelection.
		item icon: MenuIcons smallCopyIcon.
		item := menu add: 'Cut' target: aPane action: #cutSelection.
		item icon: MenuIcons smallCutIcon.
		item := menu add: 'Paste' target: aPane action: #pasteSelection.
		item icon: MenuIcons smallPasteIcon.
	menu addSeparator.
		item := menu add: 'Evaluate Selection' target: self 
				action: #signal:with: arguments: {#doIt. aPane}.
		item icon: MenuIcons smallDoItIcon.
		item := menu add: 'Print Selection' target: self 
				action: #signal:with: arguments: {#printIt. aPane}.
		item icon: MenuIcons smallPrintIcon.
		item := menu add: 'Inspect Selection' target: self 
				action: #signal:with: arguments: {#inspectIt. aPane}.
		item icon: MenuIcons smallInspectItIcon.
		item := menu add: 'Explore Selection' target: self 
				action: #signal:with: arguments: {#exploreIt. aPane}.
		item icon: MenuIcons smallInspectItIcon.
		item := menu add: 'Debug Selection' target: self
				action: #signal:with: arguments: {#debugIt. aPane}.
		item icon: MenuIcons smallDoItIcon.
	menu addSeparator.
		item := menu add: 'Accept' target: aPane action: #save.
		item icon: MenuIcons smallOkIcon.
		item := menu add: 'Cancel' target: aPane action: #revert.
		item icon: MenuIcons smallCancelIcon.
	^menu! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'bf 1/11/2005 18:11'!
notify: aString at: location in: source

	codePane selectionStart: location.
	codePane selectionStop: location.
	codePane paste: aString select: true.
! !

!CCodeHolder methodsFor: 'validation' stamp: 'ar 1/5/2005 18:04'!
okToChange
	"See if there is unaccepted input"
	codePane modified ifFalse:[^true].
	(self confirm: 'Changes have not been saved.
Is it OK to cancel those changes?') ifFalse:[^false].
	codePane modified: false.
	^true! !

!CCodeHolder methodsFor: 'validation' stamp: 'ar 3/7/2004 00:07'!
okToChangeList: aList from: prevValue
	"This is a slightly modified version of okToChange so that the user is not presented with weird changes in the selection. It silently sets back the list selection to its previous value until the user has decided what to do."
	| nextValue |
	nextValue := aList selection.
	nextValue = prevValue ifTrue:[^false]. "No need to change it"
	aList selection: prevValue.
	self okToChange ifFalse:[^false].
	aList selection: nextValue.
	^true! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 6/19/2005 13:34'!
onBrowseIt: aPane
	| aSymbol |
	<on: browseIt>
	aPane editor selectLineIfEmpty.
	aPane selection size = 0 ifTrue:[^self].
	(aSymbol := aPane selection string findSelector) ifNil:[^ self flash].
	aSymbol first isUppercase ifTrue: [
		Smalltalk at: aSymbol ifPresent: [:aGlobal |
			aGlobal ifNil: [^ self flash].
			^CClassBrowser open
				selectClass: ((aGlobal isKindOf: Class)
						ifTrue: [aGlobal]
						ifFalse:	[aGlobal class])  
				selector: nil]].
	self browseAllImplementorsOf: aSymbol! !

!CCodeHolder methodsFor: 'events' stamp: 'ar 6/19/2005 13:30'!
onCodePaneKeyDown: evt
	"Update the message category list when the class list's cursor changes"
	<on: keyDown in: codePane>
	self onCodePaneKeyDown: evt in: codePane.! !

!CCodeHolder methodsFor: 'events' stamp: 'ar 6/19/2005 13:30'!
onCodePaneKeyDown: evt in: aPane
	"Resignal code pane events"
	| action |
	action := CPlatform current standardCodingKeys lookupEvent: evt.
	action ifNotNil:[self signal: action with: aPane].! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 6/19/2005 13:27'!
onDebugIt: aPane
	"Treat the current text selection as an expression; debug it."
	<on: debugIt>
	Smalltalk beep.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 6/19/2005 13:36'!
onDefinitionsOfIt: aPane
	"Open an implementors browser on the selected selector"
	| aSelector |
	<on: definitionsOfIt>
	aPane editor selectLineIfEmpty.
	aPane selection size = 0 ifTrue:[^self].
	(aSelector := aPane selection string findSelector) ifNil:[^ self flash].
	self browseAllImplementorsOf: aSelector.! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 6/19/2005 13:25'!
onDoIt: aPane
	"Set the context to include pool vars of the model.  Then evaluate."
	<on: doIt>
	^ self evaluateSelection: aPane! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 3/7/2006 19:00'!
onExploreIt: aPane
	"Treat the current text selection as an expression; evaluate it.
	Open an explorer on the result of the expression."
	| result |
	<on: exploreIt>
	result := self evaluateSelection: aPane.
	((result isKindOf: FakeClassPool) or: [result == #failedDoit])
		ifTrue: [^aPane flash].
	ToolSet explore: result! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 6/19/2005 13:25'!
onInspectIt: aPane
	"Treat the current text selection as an expression; evaluate it.
	Open an inspector on the result of the expression."
	| result |
	<on: inspectIt>
	result := self evaluateSelection: aPane.
	((result isKindOf: FakeClassPool) or: [result == #failedDoit])
		ifTrue: [^aPane flash].
	result inspect! !

!CCodeHolder methodsFor: 'actions - code pane' stamp: 'ar 6/19/2005 13:31'!
onPasteIfFalse: aPane
	<on: pasteIfFalse>
	aPane paste: 'ifFalse:'! !

!CCodeHolder methodsFor: 'actions - code pane' stamp: 'ar 6/19/2005 13:31'!
onPasteIfTrue: aPane
	<on: pasteIfTrue>
	aPane paste: 'ifTrue:'! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 6/19/2005 13:47'!
onPrintIt: aPane
	"Treat the current text selection as an expression; evaluate it. Insert the 
	description of the result of evaluation after the selection and then make 
	this description the new text selection."
	| result |
	<on: printIt>
	result := self evaluateSelection: aPane.
	((result isKindOf: FakeClassPool) or: [result == #failedDoit])
		ifTrue: [^aPane flash].
	aPane selectionStart: aPane selectionStop.
	aPane paste: ' ', result printString select: true.
! !

!CCodeHolder methodsFor: 'events' stamp: 'ar 12/15/2004 00:02'!
onRevert
	<on: revert in: codePane>
	self onUpdateCodePane.! !

!CCodeHolder methodsFor: 'browsing' stamp: 'ar 6/19/2005 13:36'!
onSendersOfIt: aPane
	| aSelector |
	<on: sendersOfIt>
	aPane editor selectLineIfEmpty.
	aPane selection size = 0 ifTrue:[^self].
	(aSelector := aPane selection string findSelector) ifNil:[^ self flash].
	self browseAllCallsOn: aSelector
! !

!CCodeHolder methodsFor: 'events' stamp: 'ar 12/15/2004 00:02'!
onUpdateCodePane
	<on: updateCodePane>
	Smalltalk beep.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:16'!
openSingleMessageBrowser
	| msgName mr |
	"Create and schedule a message list browser populated only by the currently selected message"

	(msgName := self selectedMessageName) ifNil: [^ self].

	mr := MethodReference new
		setStandardClass: self selectedClassOrMetaClass
		methodSymbol: msgName.

	self  
		browseMessageList: (Array with: mr)
		name: mr asStringOrText
		autoSelect: nil! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 12/18/2004 23:38'!
postOpen
	"Do the user some good and waste a few cycles in the background to create the menu costumes"
	self preLoad: codePane menu.! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 12/18/2004 23:40'!
preLoad: aPlayer
	"To achieve better UI feel this method allows us to preload a costume such as for menus"
	| cc |
	aPlayer ifNil:[^self].
	self waitTick; waitTick.
	cc := aPlayer costume "force creation" asPrimCostume. "but grab prim costume"
	[cc layoutNeeded] whileTrue:[
		cc fullBounds.
		self waitTick.
	].! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 14:36'!
printOutClass
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .html."

	Cursor write showWhile:
		[self selectedClass fileOutAsHtml: true]! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 16:17'!
printOutMessage
	"Write a file with the text of the selected message, for printing by a web browser"

	self selectedMessageName ifNotNil: [
		self selectedClassOrMetaClass fileOutMethod: self selectedMessageName
							asHtml: true]! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'ar 1/5/2005 18:04'!
removeClass
	"Remove the selected class from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."
	| message  className classToRemove |
	self okToChange ifFalse: [^ false].
	classToRemove := self selectedClassOrMetaClass ifNil: [self flash. ^ false].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you want to REMOVE

	', className, '

from the system?'.
	(self confirm: message) ifFalse:[^false].
	classToRemove subclasses size > 0 ifTrue: [
		self inform:  className, ' has subclasses!!
It cannot be removed'.
		^false].
	classToRemove removeFromSystem.
	self selectedClassName: nil.
	self onSelectedSystemCategoryChanged.
	^true! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 17:44'!
removeFromCurrentChanges
	"Tell the changes mgr to forget that the current msg was changed."

	ChangeSet current removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 17:41'!
revertAndForget
	"Revert to the previous version of the current method, and tell the changes mgr to forget that it was ever changed.  Danger!!  Use only if you really know what you're doing!!"

	self okToChange ifFalse: [^ self].
	self revertToPreviousVersion.
	self removeFromCurrentChanges.
	self signal: #updateCodePane.! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'md 4/23/2004 17:43'!
revertToPreviousVersion
	"Revert to the previous version of the current method"
	| aClass aSelector  changeRecords |
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	aClass ifNil: [^ self changed: #flash].
	aSelector := self selectedMessageName.
	changeRecords := aClass changeRecordsAt: aSelector.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [self changed: #flash.  ^ self beep].
	changeRecords second fileIn.
	self signal: #updateCodePane.
! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:03'!
sampleInstanceOfSelectedClass
	| aClass |
	"Return a sample instance of the class currently being pointed at"
	(aClass := self selectedClassOrMetaClass) ifNil: [^ nil].
	^ aClass theNonMetaClass initializedInstance! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 3/31/2004 15:11'!
selectFrom: start to: stop
	codePane selectFrom: start to: stop! !

!CCodeHolder methodsFor: 'hacks' stamp: 'bf 11/29/2004 16:04'!
selectInvisiblyFrom: start to: stop
	"ParagraphEditor's selection is off by one"
	codePane selectFrom: start to: stop+1! !

!CCodeHolder methodsFor: 'actions - message list' stamp: 'ar 6/19/2005 13:42'!
selectMessage: aString andEvaluate: aBlock
	"Allow the user to choose one selector, chosen from the currently selected message's selector, as well as those of all messages sent by it, and evaluate aBlock on behalf of chosen selector.  If there is only one possible choice, simply make it; if there are multiple choices, put up a menu, and evaluate aBlock on behalf of the the chosen selector, doing nothing if the user declines to choose any"

	| selector method messages msgMenu result |
	selector := self selectedMessage ifNil: [^ self].
	method := (self selectedClassOrMetaClass ifNil: [^ self])
		compiledMethodAt: selector ifAbsent:[nil].
	method ifNil:[^aBlock value: selector].
	messages := method messages copyWithout: selector.
	messages isEmpty ifTrue: [^ aBlock value: selector].
	msgMenu := CMenu new.
	msgMenu label: (aString ifNil:['Choose message']).
	result := msgMenu chooseFromValues: messages default: selector.
	result ifNotNil:[aBlock value: result].! !

!CCodeHolder methodsFor: 'selection access' stamp: 'ar 3/31/2004 14:42'!
selectedClass
	"Answer the currently selected class (not just its name)"
	^nil! !

!CCodeHolder methodsFor: 'selection access' stamp: 'ar 3/31/2004 14:43'!
selectedClassName
	"Answer the name of the currently selected"
	^(self selectedClass ifNil:[^nil]) name! !

!CCodeHolder methodsFor: 'accessing' stamp: 'md 4/21/2004 10:44'!
selectedClassOrMetaClass

	^ self selectedClass	"I don't know any better"! !

!CCodeHolder methodsFor: 'selection access' stamp: 'ar 3/31/2004 15:00'!
selectedSelector
	"Try to make a selector out of the current text selection"
	^codePane selection string findSelector! !

!CCodeHolder methodsFor: 'evaluating' stamp: 'ar 3/31/2004 14:46'!
selectionInterval
	^codePane selectionStart to: codePane selectionStop! !

!CCodeHolder methodsFor: 'initialize' stamp: 'ar 6/19/2005 13:44'!
setupCostume
	"Create the default costume"
	super setupCostume.
	self extent: 400@265.
	self layout: CProportionalLayout new.

	codePane color: Color transparent.
	codePane layoutFrame: (0@0 corner: 1@1).
	codePane hResizing: #spaceFill; vResizing: #spaceFill.
	codePane textWrap: true.
	codePane hScrollable: false; vScrollable: true.
	codePane menu: (self newCodePaneMenu: codePane).
	self add: codePane.

	"Give all of my contents the default look"
	self decorate: elements color: self defaultWindowColor.

	self startScript: #postOpen.! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:40'!
showUnreferencedClassVars
	"Search for all class variables known to the selected class, and put up a 
	list of those that have no references anywhere in the system. The 
	search includes superclasses, so that you don't need to navigate your 
	way to the class that defines each class variable in order to determine 
	whether it is unreferenced"
	| cls aList aReport varMenu |
	(cls := self selectedClass)
		ifNil: [^ self].
	aList := self systemNavigation allUnreferencedClassVariablesOf: cls.
	aList size == 0
		ifTrue: [^ self inform: 'There are no unreferenced
class variables in
' , cls name].
	aReport := String
				streamContents: [:aStream | 
					aStream nextPutAll: 'Unreferenced class variable(s) in ' , cls name;
						 cr.
					aList
						do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	"(SelectionMenu labels: aList selections: aList)
		startUpWithCaption: 'Unreferenced
class variables in 
' , cls name"
	varMenu := CMenu new.
	varMenu label:  'Unreferenced 
class variables in 
' , cls name.
	varMenu chooseFrom: (aList collect: [:each | Array with: each with: each]).! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/23/2004 15:41'!
showUnreferencedInstVars
	"Search for all instance variables known to the selected class, and put up a list of those that have no references anywhere in the system.  The search includes superclasses, so that you don't need to navigate your way to the class that defines each inst variable in order to determine whether it is unreferenced"

	| cls aList aReport varMenu |
	(cls := self selectedClassOrMetaClass) ifNil: [^ self].
	aList := cls allUnreferencedInstanceVariables.
	aList size == 0 ifTrue: [^ self inform: 'There are no unreferenced
instance variables in
', cls name].
	aReport := String streamContents:
		[:aStream |
			aStream nextPutAll: 'Unreferenced instance variable(s) in ', cls name; cr.
			aList do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	"(SelectionMenu labels: aList selections: aList) startUpWithCaption: 'Unreferenced
instance variables in 
', cls name"

		varMenu := CMenu new.
	varMenu label:  'Unreferenced 
instance variables in 
' , cls name.
	varMenu chooseFrom: (aList collect: [:each | Array with: each with: each]).! !

!CCodeHolder methodsFor: 'actions - class list' stamp: 'md 4/22/2004 17:12'!
spawnHierarchy
	"Create and schedule a new hierarchy browser on the currently selected class or meta."
	self cNotYet.


! !

!CCodeHolder methodsFor: 'text' stamp: 'ar 3/31/2004 14:49'!
textHolder
	"Ugly... but what can we do..."
	^codePane textHolder! !

CCodeHolder subclass: #CDebugger
	instanceVariableNames: 'interruptedProcess receiverInspector contextVariablesInspector externalInterrupt proceedValue selectingPC sourceMap tempNames savedCursor labelString theMethodNode'
	classVariableNames: 'ContextStackKeystrokes'
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CDebugger defineFields: '<?xml  version="1.0" ?>
<fields>
	<field instanceVariable="true" toGet="interruptedProcess" toSet="interruptedProcess:">interruptedProcess</field>
	<field instanceVariable="true" toGet="receiverInspector" toSet="receiverInspector:">receiverInspector</field>
	<field instanceVariable="true" toGet="contextVariablesInspector" toSet="contextVariablesInspector:">contextVariablesInspector</field>
	<field instanceVariable="true" toGet="externalInterrupt" toSet="externalInterrupt:">externalInterrupt</field>
	<field instanceVariable="true" toGet="proceedValue" toSet="proceedValue:">proceedValue</field>
	<field instanceVariable="true" toGet="selectingPC" toSet="selectingPC:">selectingPC</field>
	<field instanceVariable="true" toGet="sourceMap" toSet="sourceMap:">sourceMap</field>
	<field instanceVariable="true" toGet="tempNames" toSet="tempNames:">tempNames</field>
	<field instanceVariable="true" toGet="savedCursor" toSet="savedCursor:">savedCursor</field>
	<field instanceVariable="true" toGet="labelString" toSet="labelString:">labelString</field>
	<field instanceVariable="true" toGet="theMethodNode" toSet="theMethodNode:">theMethodNode</field>
	<field toGet="stackListPane" toSet="stackListPane:" changeEvent="stackListPaneChanged">stackListPane</field>
	<field toGet="scriptList" toSet="scriptList:" changeEvent="scriptListChanged">scriptList</field>
	<field toGet="messagePane" toSet="messagePane:" changeEvent="messagePaneChanged">messagePane</field>
	<field toGet="receiverInspectorList" toSet="receiverInspectorList:" changeEvent="receiverInspectorListChanged">receiverInspectorList</field>
	<field toGet="receiverInspectorContents" toSet="receiverInspectorContents:" changeEvent="receiverInspectorContentsChanged">receiverInspectorContents</field>
	<field toGet="contextVariableInspectorList" toSet="contextVariableInspectorList:" changeEvent="contextVariableInspectorListChanged">contextVariableInspectorList</field>
	<field toGet="contextVariableInspectorContents" toSet="contextVariableInspectorContents:" changeEvent="contextVariableInspectorContentsChanged">contextVariableInspectorContents</field>
	<field toGet="proceedButton" toSet="proceedButton:" changeEvent="proceedButtonChanged">proceedButton</field>
	<field toGet="restartButton" toSet="restartButton:" changeEvent="restartButtonChanged">restartButton</field>
	<field toGet="sendButton" toSet="sendButton:" changeEvent="sendButtonChanged">sendButton</field>
	<field toGet="doStepButton" toSet="doStepButton:" changeEvent="doStepButtonChanged">doStepButton</field>
	<field toGet="stepIntoBlockButton" toSet="stepIntoBlockButton:" changeEvent="stepIntoBlockButtonChanged">stepIntoBlockButton</field>
	<field toGet="fullStackButton" toSet="fullStackButton:" changeEvent="fullStackButtonChanged">fullStackButton</field>
	<field toGet="whereButton" toSet="whereButton:" changeEvent="whereButtonChanged">whereButton</field>
	<field toGet="completeButton" toSet="completeButton:" changeEvent="completeButtonChanged">completeButton</field>
</fields>'!

!CDebugger class methodsFor: 'instance creation' stamp: 'ar 11/12/2004 13:15'!
context: aContext 
	"Answer an instance of me for debugging the active process starting with the given context."

	^ self new process: Processor activeProcess context: aContext! !

!CDebugger class methodsFor: 'instance creation' stamp: 'ar 5/18/2005 11:56'!
informExistingDebugger: aContext label: aString
	"Walking the context chain, we try to find out if we're in a debugger stepping situation.
	If we find the relevant contexts, we must rearrange them so they look just like they would
	if the methods were excuted outside of the debugger."
	| ctx quickStepMethod oldSender baseContext |
	ctx := thisContext.
	quickStepMethod := ContextPart compiledMethodAt: #quickSend:to:with:super:.
	[ctx sender == nil or: [ctx sender method == quickStepMethod]] whileFalse: [ctx := ctx sender].
	ctx sender == nil ifTrue: [^self].
	baseContext := ctx.
	"baseContext is now the context created by the #quickSend... method."
	oldSender := ctx := ctx sender home sender.
	"oldSender is the context which originally sent the #quickSend... method"
	[ctx == nil or: [ctx receiver isKindOf: self]] whileFalse: [ctx := ctx sender].
	ctx == nil ifTrue: [^self].
	Processor activeProcess errorRecursion: false.

	"ctx is the context of the Debugger method #doStep"
	ctx receiver labelString: aString.
	ctx receiver externalInterrupt: false; proceedValue: aContext receiver.
	baseContext swapSender: baseContext sender sender sender.	"remove intervening contexts"
	thisContext swapSender: oldSender.	"make myself return to debugger"
	^aContext! !

!CDebugger class methodsFor: 'class initialization' stamp: 'ar 5/18/2005 11:46'!
initialize

	ContextStackKeystrokes := Dictionary new
		at: $e put: #send;
		at: $t put: #doStep;
		at: $T put: #stepIntoBlock;
		at: $p put: #proceed;
		at: $r put: #restart;
		at: $f put: #fullStack;
		at: $w put: #where;
		yourself.

	"CDebugger initialize"! !

!CDebugger class methodsFor: 'opening' stamp: 'jrd 5/23/2009 00:51'!
openContext: aContext label: aString contents: contentsStringOrNil
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."
	| aProcess debugger |
	<primitive: 19> "Simulation guard"
	aProcess := Processor activeProcess.
	aProcess errorRecursion ifTrue:[
		aProcess errorRecursion: false.
		^self primitiveError: aString].
	"Indicate error recursion"
	aProcess errorRecursion: true.

	Preferences logDebuggerStackToFile
		ifTrue:[Smalltalk logSqueakError: aString inContext: aContext].
	self informExistingDebugger: aContext label: aString.

	debugger := CDebugger context: aContext.
	debugger
		openNotifierContents: contentsStringOrNil
		label: aString.
	debugger startScript:[
		self waitTick. "or else error recursion might happen"
		aProcess errorRecursion: false.
	].
	aProcess suspend.
! !

!CDebugger class methodsFor: 'testing' stamp: 'md 4/28/2004 17:52'!
openForTest
	"Just for testing...   "

	CDebugger
		openContext: thisContext
		label: 'a Test'
		contents: 'a Test'! !

!CDebugger class methodsFor: 'opening' stamp: 'mtf 2/5/2011 19:38'!
openInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."
	| debugger |
	<primitive: 19> "Simulation guard"
	debugger := self new.
	debugger
		process: interruptedProcess
		context: interruptedProcess suspendedContext.
	debugger externalInterrupt: true.

Preferences logDebuggerStackToFile ifTrue:
	[(aString includesSubString: 'Space') & 
		(aString includesSubString: 'low') ifTrue: [
			Smalltalk logError: aString inContext: debugger interruptedContext to:'LowSpaceDebug.log']
		ifFalse: [Smalltalk logSqueakError: aString inContext: debugger interruptedContext]].

	^ debugger
		openNotifierContents: nil
		label: aString
! !

!CDebugger class methodsFor: 'opening' stamp: 'jrd 5/23/2009 00:52'!
openOn: aProcess context: aContext label: aString contents: contentsStringOrNil fullView: bool
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."
	| debugger dbgScript |
	debugger := self new process: aProcess context: aContext.
	debugger startScript:[
		aProcess errorRecursion ifTrue:[
			aProcess errorRecursion: false.
			^self primitiveError: aString].
		"Indicate error recursion"
		dbgScript := Processor activeProcess.
		dbgScript errorRecursion: true.

		Preferences logDebuggerStackToFile
			ifTrue:[Smalltalk logSqueakError: aString inContext: aContext].
		aProcess eventTracer: debugger.
		aProcess traceEvents: true.
		bool ifTrue: [debugger openLabel: aString]
			ifFalse: [debugger openNotifierContents: contentsStringOrNil label: aString].
		debugger startScript:[
			self waitTick. "or else error recursion might happen"
			dbgScript errorRecursion: false.
		].
	].
	aProcess suspend.! !

!CDebugger class methodsFor: 'opening' stamp: 'ar 12/21/2004 11:55'!
primitiveError: aString
	Smalltalk isMorphic 
		ifTrue:[^Debugger openContext: thisContext label: aString contents: nil].
	^CWorldPlayer primitiveError: aString! !

!CDebugger class methodsFor: 'instance creation' stamp: 'ar 11/12/2004 13:14'!
process: aProcess context: aContext
	^self new process: aProcess context: aContext! !

!CDebugger class methodsFor: 'window color' stamp: 'md 4/6/2004 16:26'!
windowColorSpecification
	"Answer a WindowColorSpec object that declares my preference"

	^ WindowColorSpec classSymbol: self name  wording: 'Debugger' brightColor: #lightRed pastelColor: #veryPaleRed helpMessage: 'The system debugger.'! !

!CDebugger methodsFor: 'initialize' stamp: 'ar 11/13/2004 18:29'!
abort
	interruptedProcess == nil ifTrue: [^ self].
	[interruptedProcess terminate] on: Error do: [].
	scriptList do:[:each| each terminate].
	interruptedProcess := nil.
	receiverInspector := nil.
	contextVariablesInspector := nil.
	Smalltalk installLowSpaceWatcher.  "restart low space handler"! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:36'!
browseMessages
	"Present a menu of all messages sent by the currently selected message.
	Open a message set browser of all implementors of the message chosen.
	Do nothing if no message is chosen."

	stackListPane cursor = 0 ifTrue: [^ self].
	super browseImplementors.! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:38'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all
	messages sent by it.  Open a message set browser of all implementors
	of the message chosen."

	stackListPane cursor = 0 ifTrue: [^ self].
	super browseSenders! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/6/2004 16:26'!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."

	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	VersionsBrowser
		browseVersionsOf: (class compiledMethodAt: selector)
		class: self selectedClass theNonMetaClass
		meta: class isMeta
		category: self selectedMessageCategoryName
		selector: selector! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:36'!
buildMessageBrowser
	"Create and schedule a message browser on the current method."

	stackListPane cursor = 0 ifTrue: [^ self].
	^ Browser
		openMessageBrowserForClass: self selectedClassOrMetaClass
		selector: self selectedMessageName
		editString: nil! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 2/7/2005 01:06'!
buildNotifierLabelled: label message: messageString
	| w |
	self expandStack.
	
	w := CPreDebugger new.
	w label: label.
	w player: self.
	w open.
	stackListPane size > 0 ifTrue:[
		w notifyPane contents: stackListPane contents.
	] ifFalse:[
		w notifyPane contents: messagePane contents.
	].
	^w
! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/12/2004 18:38'!
checkContextSelection

	stackListPane cursor = 0 ifTrue: [stackListPane cursor := 1]! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/14/2004 14:25'!
complete
	"Complete current message"
	interruptedProcess ifNil:[^self].
	interruptedProcess isDone ifTrue:[^self].
	interruptedProcess topContext: nil.
	interruptedProcess resume.
	self startScript:[
		self newStack: #().
		stackListPane cursor := 0.
	].! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 18:21'!
completeButton
	"Answer the completeButton of the receiver"
	<bewareOf: #completeButtonChanged>
	^self propertyValueAt: #completeButton! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 18:21'!
completeButton: aValue
	"Modify the receiver's completeButton"
	^self propertyValueAt: #completeButton put: aValue with: #completeButtonChanged! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 23:31'!
completeIfBottom.
	"Complete current message"
	(self isBottomContext: interruptedProcess topContext) ifTrue:[self complete].! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 18:38'!
contents: aText notifying: aController 
	"The retrieved information has changed and its source must now be 
	updated. In this case, the retrieved information is the method of the 
	selected context."

	| selector classOfMethod category h ctxt |
	stackListPane cursor = 0 ifTrue: [self halt.^ false].
	self selectedContext isExecutingBlock ifTrue: [
		h := self selectedContext finalBlockHome.
		h ifNil: [self inform: 'Method not found for block, can''t edit'. ^ false].
		(self confirm: 'I will have to revert to the method from
which this block originated.  Is that OK?')
			ifTrue: [self resetContext: h]
			ifFalse: [^ false]
	].
	classOfMethod := self selectedClass.
	category := self selectedMessageCategoryName.
	selector := Compiler parserClass new parseSelector: aText.
	selector == self selectedMessageName
		ifFalse: [self inform: 'can''t change selector'. ^ false].

	selector := classOfMethod
		compile: aText
		classified: category
		notifying: aController.
	selector ifNil: [^ false].  "compile cancelled"

	ctxt := self selectedContext.
	interruptedProcess
		popTo: ctxt;
		restartTopWith: (classOfMethod compiledMethodAt: selector);
		stepToSendOrReturn.

	contextVariablesInspector object: nil.
	theMethodNode := ctxt methodNode.
	sourceMap := theMethodNode sourceMap.
	tempNames := theMethodNode tempNames.
	self resetContext: ctxt.

	^true! !

!CDebugger methodsFor: 'code pane' stamp: 'md 4/6/2004 16:26'!
contentsSelection

	^ self pcRange! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/12/2004 18:54'!
contextStackIndex: anInteger oldContextWas: oldContext

	| |
	anInteger = 0 ifTrue:[
		theMethodNode := tempNames := sourceMap := nil.
		self contents: ''.
		contextVariablesInspector object: nil.
		receiverInspector object: self receiver.
		^self].
	tempNames := sourceMap := nil.
	theMethodNode := self selectedContext methodNode.
	self contents: self selectedMessageText.
	self pcRange "will compute tempNamesunless noFrills".
	tempNames == nil
		ifTrue: [tempNames := 
					self selectedClassOrMetaClass parserClass new parseArgsAndTemps: self contents notifying: nil].

	contextVariablesInspector object: self selectedContext.
	receiverInspector object: self receiver.

	self signal: #updateCodePane! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/6/2004 16:26'!
contextStackKey: aChar from: view
	"Respond to a keystroke in the context list"

 	| selector |
	selector := ContextStackKeystrokes at: aChar ifAbsent: [nil].
	selector ifNil: [self messageListKey: aChar from: view]
		ifNotNil: [self perform: selector]! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 23:02'!
contextStackTop
	^interruptedProcess ifNotNil:[interruptedProcess topContext].! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:50'!
contextVariableInspectorContents
	"Answer the contextVariableInspectorContents of the receiver"
	<bewareOf: #contextVariableInspectorContentsChanged>
	^self propertyValueAt: #contextVariableInspectorContents! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:50'!
contextVariableInspectorContents: aValue
	"Modify the receiver's contextVariableInspectorContents"
	^self propertyValueAt: #contextVariableInspectorContents put: aValue with: #contextVariableInspectorContentsChanged! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:50'!
contextVariableInspectorList
	"Answer the contextVariableInspectorList of the receiver"
	<bewareOf: #contextVariableInspectorListChanged>
	^self propertyValueAt: #contextVariableInspectorList! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:50'!
contextVariableInspectorList: aValue
	"Modify the receiver's contextVariableInspectorList"
	^self propertyValueAt: #contextVariableInspectorList put: aValue with: #contextVariableInspectorListChanged! !

!CDebugger methodsFor: 'inspectors' stamp: 'md 4/28/2004 11:01'!
contextVariableInspectorUpdate
	contextVariablesInspector selectionIndex: (contextVariableInspectorList contents indexOf: contextVariableInspectorList selection).
	contextVariableInspectorContents contents: contextVariablesInspector selection.! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/6/2004 16:26'!
contextVariablesInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context."

	^contextVariablesInspector! !

!CDebugger methodsFor: 'menu notifier' stamp: 'ar 11/12/2004 18:42'!
debug
	"Open a full DebuggerView."
	self open.
! !

!CDebugger methodsFor: 'initialize' stamp: 'md 4/6/2004 16:26'!
debugAt: anInteger
	self toggleContextStackIndex: anInteger. 
	 ^ self debug.! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 11/13/2004 18:27'!
debugScript: script
	| rcvr selector method item |
	scriptList add: script.
	item := messagePane treeNodes at: messagePane cursor.
	item addChild: (CDebuggerTreeItem on: script).
	rcvr := script message receiver.
	selector := script message selector.
	method := rcvr class lookupSelector: selector.
	method ifNil:[method := rcvr class lookupSelector: #doesNotUnderstand:].
	method ifNil:[^self].
	[script suspendedContext method == method] whileFalse:[script stepToCallee].
	script topContext: script suspendedContext.! !

!CDebugger methodsFor: 'initialize' stamp: 'ar 11/12/2004 16:56'!
debuggerButtons
	^{completeButton. restartButton. sendButton. doStepButton. stepIntoBlockButton. fullStackButton. whereButton}! !

!CDebugger methodsFor: 'code pane' stamp: 'ar 11/12/2004 18:36'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	stackListPane cursor = 0
		ifTrue: [^super doItContext]
		ifFalse: [^self selectedContext]! !

!CDebugger methodsFor: 'code pane' stamp: 'md 4/6/2004 16:26'!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^self receiver! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 22:47'!
doStep
	"Send the selected message in the accessed method, and regain control 
	after the invoked method returns."
	
	| currentContext newContext |
	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	currentContext := self selectedContext ifNil:[^self].
	self traceEventsIn:[
		newContext := interruptedProcess completeStep: currentContext.
		newContext == currentContext 
			ifTrue: [newContext := interruptedProcess stepToSendOrReturn].
	].
	stackListPane cursor > 1
		ifTrue: [self resetContext: newContext]
		ifFalse: [newContext == currentContext
				ifTrue: [self signal: #updateCodePane.
						self updateInspectors]
				ifFalse: [self resetContext: newContext]].
	self completeIfBottom.! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
doStepButton
	"Answer the doStepButton of the receiver"
	<bewareOf: #doStepButtonChanged>
	^self propertyValueAt: #doStepButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
doStepButton: aValue
	"Modify the receiver's doStepButton"
	^self propertyValueAt: #doStepButton put: aValue with: #doStepButtonChanged! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:38'!
down
	"move down the context stack to the previous (enclosing) context"

	self toggleContextStackIndex: stackListPane cursor+1! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'ar 11/13/2004 16:49'!
expandStack
	"A Notifier is being turned into a full debugger.  Show a substantial amount of stack in the context pane."

	stackListPane cursor := 0.
	proceedValue := nil.
	self newStack: ((self contextStackTop ifNil:[^self]) stackOfSize: 20).
! !

!CDebugger methodsFor: 'private' stamp: 'md 4/6/2004 16:26'!
externalInterrupt: aBoolean

	externalInterrupt := aBoolean ! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 22:37'!
fullStack
	"Change from displaying the minimal stack to a full one."
	self okToChange ifFalse: [^ self].
	self newStack: self contextStackTop contextStack.! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
fullStackButton
	"Answer the fullStackButton of the receiver"
	<bewareOf: #fullStackButtonChanged>
	^self propertyValueAt: #fullStackButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
fullStackButton: aValue
	"Modify the receiver's fullStackButton"
	^self propertyValueAt: #fullStackButton put: aValue with: #fullStackButtonChanged! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'ar 11/12/2004 18:42'!
fullyExpandStack
	"Expand the stack to include all of it, rather than the first four or five
	contexts."

	self okToChange ifFalse: [^ self].
	self newStack: self contextStackTop contextStack.! !

!CDebugger methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:57'!
initialize
	super initialize.
	"Define our components"
	
	self define: #stackListPane as: CStringList new.
	stackListPane menu: self stackListMenu.
	"listPane userDataAt: #keyMap put: self fieldListKeyMap."
	stackListPane allowDeselect: true.

	self define: #scriptList as: CStringList new.
	scriptList allowDeselect: true.

	self define: #messagePane as: CTreeWidget new.
	messagePane allowDeselect: true.

	self define: #receiverInspectorList as:  CStringList new.
	receiverInspectorList allowDeselect: true.

	self define: #receiverInspectorContents as: CTextEditor new.

	self define: #contextVariableInspectorList as:  CStringList new.
	contextVariableInspectorList allowDeselect: true.

	self define: #contextVariableInspectorContents as: CTextEditor new.
	
	    
	"the buttons"

	self define: #completeButton as: CButton new.
	self define: #proceedButton as: CButton new.
	self define: #restartButton as: CButton new.
	self define: #sendButton as: CButton new.
	self define: #doStepButton as: CButton new.
	self define: #stepIntoBlockButton as: CButton new.
	self define: #fullStackButton as: CButton new.
	self define: #whereButton as: CButton new.

	receiverInspector := CInspector inspect: nil.
	receiverInspectorList menu: receiverInspector fieldListMenu.

	contextVariablesInspector := CContextVariablesInspector inspect: nil.
	contextVariableInspectorList menu: contextVariablesInspector fieldListMenu.! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 18:41'!
interruptedContext
	"Answer the suspended context of the interrupted process."

	^self contextStackTop! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
interruptedProcess
	"Answer the interrupted process."

	^interruptedProcess! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 11/13/2004 18:46'!
interruptedProcess: aProcess
	interruptedProcess == aProcess ifTrue:[^self].
	selectingPC := false.
	self signal: #updateCodePane.
	interruptedProcess ifNotNil:[
		interruptedProcess traceEvents: false.
		interruptedProcess eventTracer: nil.
		interruptedProcess := nil.
	].
	interruptedProcess := aProcess.
	interruptedProcess ifNotNil:[
		messagePane cursor := messagePane findFirstIndex:[:each| each value == interruptedProcess].
		scriptList selection: interruptedProcess.
		interruptedProcess topContext ifNil:[interruptedProcess := nil].
	].
	interruptedProcess ifNil:[
		self newStack: #().
		stackListPane cursor := 0.
		^self].
	interruptedProcess eventTracer: self.
	interruptedProcess traceEvents: true.
	self newStack: (self contextStackTop stackOfSize: 100).
	stackListPane cursor := 1.
	selectingPC := true.! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/12/2004 23:38'!
isBottomContext: aContext
	| method |
	method := CCompiler compiledMethodAt: #evaluate:in:to:notifying:ifFail:.
	method == aContext method ifTrue:[^true].
	method := ScriptMessageSend compiledMethodAt: #synchronousValueWithArguments:event:.
	method == aContext method ifTrue:[^true].
	^false! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
isNotifier
	"Return true if this debugger has not been expanded into a full sized window"

	^ receiverInspector == nil! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
labelString
	^labelString! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 11:09'!
labelString: aString
	labelString := aString.
	self signal: #updateWindowLabel! !

!CDebugger methodsFor: 'private' stamp: 'md 4/6/2004 16:26'!
lowSpaceChoices
	"Return a notifier message string to be presented when space is running low."

	^ 'Warning!! Squeak is almost out of memory!!

Low space detection is now disabled. It will be restored when you close or proceed from this error notifier. Don''t panic, but do proceed with caution.

Here are some suggestions:

Ä If you suspect an infinite recursion (the same methods calling each other again and again), then close this debugger, and fix the problem.

Ä If you want this computation to finish, then make more space available (read on) and choose "proceed" in this debugger. Here are some ways to make more space available...
   > Close any windows that are not needed.
   > Get rid of some large objects (e.g., images).
   > Leave this window on the screen, choose "save as..." from the screen menu, quit, restart the Squeak VM with a larger memory allocation, then restart the image you just saved, and choose "proceed" in this window.

Ä If you want to investigate further, choose "debug" in this window.  Do not use the debugger "fullStack" command unless you are certain that the stack is not very deep. (Trying to show the full stack will definitely use up all remaining memory if the low-space problem is caused by an infinite recursion!!).

'
! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/23/2004 18:54'!
mailOutBugReport
	"Compose a useful bug report showing the state of the process as well as vital image statistics as suggested by Chris Norton - 
'Squeak could pre-fill the bug form with lots of vital, but
oft-repeated, information like what is the image version, last update
number, VM version, platform, available RAM, author...'

and address it to the list with the appropriate subject prefix."

	self cNotYet.

"	| messageStrm |
	MailSender default ifNil: [^self].

	Cursor write
		showWhile: 
			[""Prepare the message""
			messageStrm := WriteStream on: (String new: 1500).
			messageStrm nextPutAll: 'From: ';
			 nextPutAll: MailSender userName;
			 cr;
			 nextPutAll: 'To: squeak-dev@lists.squeakfoundation.org';
			 cr;
			 nextPutAll: 'Subject: ';
			 nextPutAll: '[BUG]'; nextPutAll: self interruptedContext printString;
			 cr;cr;
			 nextPutAll: 'here insert explanation of what you were doing, suspect changes you''ve made and so forth.';cr;cr.
			self interruptedContext errorReportOn: messageStrm.

			MailSender sendMessage: (MailMessage from: messageStrm contents)].
"! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'ar 11/12/2004 18:38'!
messageListIndex
	"Answer the index of the currently selected context."

	^stackListPane cursor! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 15:47'!
messagePane
	"Answer the messagePane of the receiver"
	<bewareOf: #messagePaneChanged>
	^self propertyValueAt: #messagePane! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 15:47'!
messagePane: aValue
	"Modify the receiver's messagePane"
	^self propertyValueAt: #messagePane put: aValue with: #messagePaneChanged! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/12/2004 23:31'!
newStack: stack
	| idx lowest |
	idx := stackListPane cursor.
	lowest := stack findFirst:[:ctx| self isBottomContext: ctx].
	lowest = 0 
		ifTrue:[stackListPane contents: stack]
		ifFalse:[stackListPane contents: (stack copyFrom: 1 to: lowest-1)].
	stackListPane cursor: idx.! !

!CDebugger methodsFor: 'events' stamp: 'md 4/28/2004 17:52'!
onAcceptCodePane
	"Update the class list when the system category list's cursor changes"
	<on: save in: codePane> 
	self contents: codePane contents notifying: nil 
	! !

!CDebugger methodsFor: 'events - buttons' stamp: 'ar 11/13/2004 19:01'!
onCompleteButton
	| next |
	<on: fire in: completeButton>
	hand shiftPressed ifFalse:[^self complete].
	"complete recursively"
	next := scriptList size.
	[self complete.
	self waitTick.
	(next := next + 1) > scriptList size] whileFalse:[
		scriptList cursor: next.
		self waitTick.
	].! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 6/19/2005 13:45'!
onContextPaneKeyDown: evt
	<on: keyDown in: contextVariableInspectorContents>
	self onCodePaneKeyDown: evt in: contextVariableInspectorContents! !

!CDebugger methodsFor: 'events - lists' stamp: 'md 4/28/2004 11:01'!
onContextVariableInspectorListChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: contextVariableInspectorList> 

	self contextVariableInspectorUpdate.! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:14'!
onDoStepButton
	<on: fire in: doStepButton>
	self doStep.! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:19'!
onFullStackButton
	<on: fire in: fullStackButton>
	self fullStack.! !

!CDebugger methodsFor: 'events - lists' stamp: 'ar 11/14/2004 14:30'!
onMessagePaneCursorChanged
	<on: cursorChanged in: messagePane>
	self interruptedProcess: messagePane selection value.
	self signal: #updateCodePane.
	self signal: #updateButtons.
! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:13'!
onProceedButton
	<on: fire in: proceedButton>
	self proceed.! !

!CDebugger methodsFor: 'events - lists' stamp: 'md 4/28/2004 11:02'!
onReceiverInspectorListChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: receiverInspectorList> 
	self receiverInspectorUpdate.! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 6/19/2005 13:45'!
onReceiverPaneKeyDown: evt
	<on: keyDown in: receiverInspectorContents>
	self onCodePaneKeyDown: evt in: receiverInspectorContents! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:13'!
onRestartButton
	<on: fire in: restartButton>
	self restart.! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 11/13/2004 18:39'!
onScriptListCursorChanged
	<on: cursorChanged in: scriptList>
	self interruptedProcess: scriptList selection.! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:14'!
onSendButton
	<on: fire in: sendButton>
	self send.! !

!CDebugger methodsFor: 'events - lists' stamp: 'ar 11/14/2004 14:29'!
onStackFrameChanged
	<on: cursorChanged in: stackListPane>
	self signal: #updateCodePane.
	self signal: #updateButtons.
	stackListPane selection ifNil:[
		theMethodNode := tempNames := sourceMap := nil.
		contextVariablesInspector object: nil.
		receiverInspector object: self receiver.
	] ifNotNil:[
		tempNames := sourceMap := nil.
		theMethodNode := self selectedContext methodNode.
		self pcRange "will compute tempNamesunless noFrills".

		tempNames ifNil:[
			tempNames := self selectedClassOrMetaClass parserClass new 
				parseArgsAndTemps: self selectedMessageText notifying: nil].

		contextVariablesInspector object: self selectedContext.
		receiverInspector object: self receiver.
	].

	receiverInspectorList contents: receiverInspector fieldList.
	contextVariableInspectorList contents: contextVariablesInspector fieldList.! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:16'!
onStepIntoBlockButton
	<on: fire in: stepIntoBlockButton>
	self stepIntoBlock! !

!CDebugger methodsFor: 'events - buttons' stamp: 'ar 11/14/2004 14:31'!
onUpdateButtons
	"Enable/disable buttons as appropriate"
	| enable |
	<on: updateButtons>
	enable := self selectedContext notNil.
	{sendButton. doStepButton. stepIntoBlockButton. fullStackButton. whereButton} do:[:each|
		enable ifTrue:[each enable] ifFalse:[each disable].
	].
	enable := stackListPane size > 0. "e.g., we have a stack"
	{completeButton. restartButton} do:[:each|
		enable ifTrue:[each enable] ifFalse:[each disable].
	].! !

!CDebugger methodsFor: 'events' stamp: 'bf 5/30/2005 22:24'!
onUpdateCodePane
	"Update the contents of the code pane"
	| ctx script msg code range cm |
	(ctx := stackListPane selection) ifNil:[
		(script := messagePane selection) ifNil:[^codePane contents: ''].
		msg := script message.
		cm := msg receiver class lookupSelector: msg selector.
		cm ifNil: [^codePane contents: ''].
		code := cm getSourceFor: msg selector in: msg receiver class.
		range := 1 to: 0.
	] ifNotNil:[
		code := ctx sourceCode.
		range := self contentsSelection.
	].
	codePane contents: code.
	codePane selectFrom: range first to: (range last + 1).! !

!CDebugger methodsFor: 'events' stamp: 'ar 2/7/2005 01:05'!
onUpdateWindowLabel
	<on: updateWindowLabel>
	self label: self labelString.! !

!CDebugger methodsFor: 'events - buttons' stamp: 'md 4/28/2004 16:17'!
onWhereButton
	<on: fire in: whereButton>
	self where.! !

!CDebugger methodsFor: 'events' stamp: 'ar 2/7/2005 01:06'!
onWindowCloseRequest
	<on: closeRequest>
	self abort.
	self destroy. "bye, bye"! !

!CDebugger methodsFor: 'events' stamp: 'ar 7/29/2004 22:00'!
onWindowColorChanged: newColor
	super onWindowColorChanged: newColor.
	self debuggerButtons do:[:each| each windowColor: newColor].! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/13/2004 15:28'!
openLabel: aString
	self open.
! !

!CDebugger methodsFor: 'other' stamp: 'ar 11/12/2004 13:17'!
openNotifierContents: msgString label: label
	"Create and schedule a notifier view with the given label and message. A notifier view shows just the message or the first several lines of the stack, with a menu that allows the user to open a full debugger if so desired."
	"NOTE: When this method returns, a new process has been scheduled to run the windows, and thus this notifier, but the previous active porcess has not been suspended.  The sender will do this."
	| msg |
	interruptedProcess ifNotNil:[
		interruptedProcess isScriptingProcess ifTrue:[
			(self openTweakNotifier: label) ifTrue:[^self].
		].
	].
	Sensor flushKeyboard.
	savedCursor := Sensor currentCursor.
	Sensor currentCursor: Cursor normal.
	(label beginsWith: 'Space is low')
		ifTrue: [msg := self lowSpaceChoices, (msgString ifNil: [''])]
		ifFalse: [msg := msgString].

	self buildNotifierLabelled: label message: msg.! !

!CDebugger methodsFor: 'other' stamp: 'ar 11/12/2004 18:41'!
openTweakNotifier: label
	"See if there's a player in the receiver chain"
	| ctx aPlayer |
	ctx := self contextStackTop.
	[[ctx == nil] whileFalse:[
		(ctx receiver isPlayer and:[ctx receiver world notNil]) ifTrue:[
			aPlayer := ctx receiver.
			ctx := nil.
		] ifFalse:[ctx := ctx sender].
	]] on: Error do:[:ex| ex return].
	aPlayer ifNil:[^false].
	^false
! !

!CDebugger methodsFor: 'code pane' stamp: 'md 4/6/2004 16:26'!
pc

	^ self pcRange! !

!CDebugger methodsFor: 'code pane' stamp: 'ar 5/30/2005 17:17'!
pcRange
	"Answer the indices in the source code for the method corresponding to 
	the selected context's program counter value."

	| i pc end |
	(selectingPC and: [stackListPane cursor ~= 0])
		ifFalse: [^1 to: 0].
	theMethodNode ifNil:[^1 to: 0].
	sourceMap == nil ifTrue:
		[sourceMap := theMethodNode sourceMap.
		tempNames := theMethodNode tempNames].
	(sourceMap size = 0 or: [ self selectedContext isDead ]) ifTrue: [^1 to: 0].
	Smalltalk at: #RBProgramNode ifPresent:[:nodeClass|
		(theMethodNode isKindOf: nodeClass) ifTrue: [
			pc := stackListPane cursor = 1
				ifTrue: [self selectedContext pc]
				ifFalse: [self selectedContext previousPc].
			i := sourceMap findLast:[:pcRange | pcRange key <= pc].
			i = 0 ifTrue:[^ 1 to: 0].
			^ (sourceMap at: i) value
		].
	].
	pc:= self selectedContext pc - 2.
	i := sourceMap indexForInserting: (Association key: pc value: nil).
	i < 1 ifTrue: [^1 to: 0].
	i > sourceMap size
		ifTrue:
			[end := sourceMap inject: 0 into:
				[:prev :this | prev max: this value last].
			^ end+1 to: end].
	^(sourceMap at: i) value! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:35'!
peelToFirst
	"Peel the stack back to the second occurance of the currently selected message.  Very useful for an infinite recursion.  Gets back to the second call so you can see one complete recursion cycle, and how it was called at the beginning.  Also frees a lot of space!!"

	| upperGuy meth second |
	upperGuy := stackListPane selection ifNil:[^self beep].
	meth := upperGuy method.
	stackListPane cursor+1 to: stackListPane size do: [:ind |
		(stackListPane at: ind) method == meth ifTrue: [
			second := upperGuy.
			upperGuy := stackListPane at: ind]].
	second ifNil: [second := upperGuy].
	self resetContext: second.
	interruptedProcess popTo: self selectedContext.! !

!CDebugger methodsFor: 'code pane menu' stamp: 'md 4/6/2004 16:26'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then perform it on myself.  If not, send it to otherTarget, presumably the editPane from which the menu was invoked." 

	| result |
	(#(debug proceed) includes: selector)		"When I am a notifier window"
		ifTrue: [^ self perform: selector]
		ifFalse: [result := super perform: selector orSendTo: otherTarget.
				selector == #doIt ifTrue: [
					result ~~ #failedDoit ifTrue: [self proceedValue: result]].
				^ result]! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 2/7/2005 01:06'!
proceed
	"Proceed execution of the receiver's model, starting after the expression at 
	which an interruption occurred."


	Smalltalk okayToProceedEvenIfSpaceIsLow ifTrue: [self proceed: costume].
! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/6/2004 16:26'!
proceed: aTopView 
	"Proceed from the interrupted state of the currently selected context. The 
	argument is the topView of the receiver. That view is closed."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	self resumeProcess: aTopView! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
proceedButton
	"Answer the proceedButton of the receiver"
	<bewareOf: #proceedButtonChanged>
	^self propertyValueAt: #proceedButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
proceedButton: aValue
	"Modify the receiver's proceedButton"
	^self propertyValueAt: #proceedButton put: aValue with: #proceedButtonChanged! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
proceedValue
	"Answer the value to return to the selected context when the interrupted 
	process proceeds."

	^proceedValue! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
proceedValue: anObject 
	"Set the value to be returned to the selected context when the interrupted 
	process proceeds."

	proceedValue := anObject! !

!CDebugger methodsFor: 'private' stamp: 'ar 5/13/2005 10:41'!
process: aProcess context: aContext

	Smalltalk at: #MessageTally ifPresentAndInMemory: [:c | c new close].
	interruptedProcess := aProcess.
	interruptedProcess topContext: aContext.
	scriptList addLast: interruptedProcess.
	messagePane rootNode: (CDebuggerTreeItem on: interruptedProcess).
	messagePane cursor: 1.
	aContext ifNotNil:[
		self newStack: (aContext stackOfSize: 1).
		stackListPane cursor: 1.
	].
	externalInterrupt := false.
	selectingPC := true.
! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/12/2004 18:38'!
receiver
	"Answer the receiver of the selected context, if any. Answer nil 
	otherwise."

	stackListPane cursor = 0
		ifTrue: [^nil]
		ifFalse: [^self selectedContext receiver]! !

!CDebugger methodsFor: 'accessing' stamp: 'md 4/6/2004 16:26'!
receiverInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context's receiver."

	^receiverInspector! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:49'!
receiverInspectorContents
	"Answer the receiverInspectorContents of the receiver"
	<bewareOf: #receiverInspectorContentsChanged>
	^self propertyValueAt: #receiverInspectorContents! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:49'!
receiverInspectorContents: aValue
	"Modify the receiver's receiverInspectorContents"
	^self propertyValueAt: #receiverInspectorContents put: aValue with: #receiverInspectorContentsChanged! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:49'!
receiverInspectorList
	"Answer the receiverInspectorList of the receiver"
	<bewareOf: #receiverInspectorListChanged>
	^self propertyValueAt: #receiverInspectorList! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/9/2004 14:49'!
receiverInspectorList: aValue
	"Modify the receiver's receiverInspectorList"
	^self propertyValueAt: #receiverInspectorList put: aValue with: #receiverInspectorListChanged! !

!CDebugger methodsFor: 'inspectors' stamp: 'md 4/28/2004 11:01'!
receiverInspectorUpdate
	receiverInspector selectionIndex: (receiverInspectorList contents indexOf: receiverInspectorList selection).
	receiverInspectorContents contents: receiverInspector selection.! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/14/2004 14:18'!
resetContext: aContext 
	"Used when a new context becomes top-of-stack, for instance when the
	method of the selected context is re-compiled, or the simulator steps or
	returns to a new method. There is room for much optimization here, first
	to save recomputing the whole stack list (and text), and secondly to avoid
	recomposing all that text (by editing the paragraph instead of recreating it)."
	interruptedProcess topContext: aContext.
	self newStack: self contextStackTop contextStack.
	stackListPane cursor := 1.
	self signal: #updateCodePane.! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 22:46'!
restart
	"Proceed from the initial state of the currently selected context. The 
	argument is a controller on a view of the receiver. That view is closed."
	"Closing now depends on a preference #restartAlsoProceeds - hmm 9/7/2001 16:46"

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	self selectedContext ifNil:[^self].
	interruptedProcess
		popTo: self selectedContext;
		restartTop;
		stepToSendOrReturn.
	self resetContext: self selectedContext.
	Preferences restartAlsoProceeds ifTrue: [self proceed]! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
restartButton
	"Answer the restartButton of the receiver"
	<bewareOf: #restartButtonChanged>
	^self propertyValueAt: #restartButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
restartButton: aValue
	"Modify the receiver's restartButton"
	^self propertyValueAt: #restartButton put: aValue with: #restartButtonChanged! !

!CDebugger methodsFor: 'private' stamp: 'ar 6/24/2005 11:05'!
resumeProcess: aTopView 
	savedCursor ifNotNil: [Sensor currentCursor: savedCursor].
	scriptList do:[:each|
		each eventTracer == self ifTrue:[
			each eventTracer: nil.
			each traceEvents: false.
		].
		each isDone ifFalse:[each resume]].
	interruptedProcess isTerminated ifFalse: [
		interruptedProcess eventTracer == self ifTrue:[
			interruptedProcess eventTracer: nil.
			interruptedProcess traceEvents: false.
		].
		interruptedProcess resume].
	"if old process was terminated, just terminate current one"
	interruptedProcess := nil.
	"Before delete, so release doesn't terminate it"
	aTopView close.
	Smalltalk installLowSpaceWatcher.
! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/13/2004 18:25'!
scriptList
	"Answer the scriptList of the receiver"
	<bewareOf: #scriptListChanged>
	^self propertyValueAt: #scriptList! !

!CDebugger methodsFor: 'accessing' stamp: 'ar 11/13/2004 18:25'!
scriptList: aValue
	"Modify the receiver's scriptList"
	^self propertyValueAt: #scriptList put: aValue with: #scriptListChanged! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/6/2004 16:26'!
selectPC
	"Toggle the flag telling whether to automatically select the expression 
	currently being executed by the selected context."

	selectingPC := selectingPC not! !

!CDebugger methodsFor: 'class list' stamp: 'md 4/6/2004 16:26'!
selectedClass
	"Answer the class in which the currently selected context's method was 
	found."

	^self selectedContext mclass! !

!CDebugger methodsFor: 'class list' stamp: 'md 4/6/2004 16:26'!
selectedClassOrMetaClass
	"Answer the class in which the currently selected context's method was 
	found."

	^self selectedContext mclass! !

!CDebugger methodsFor: 'private' stamp: 'ar 11/12/2004 18:31'!
selectedContext
	^stackListPane selection! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'md 4/23/2004 19:11'!
selectedMessage
	"Answer the message selector of the currently selected context."

	^self selectedContext methodSelector! !

!CDebugger methodsFor: 'message category list' stamp: 'md 4/6/2004 16:26'!
selectedMessageCategoryName
	"Answer the name of the message category of the message of the 
	currently selected context."

	^self selectedClass organization categoryOfElement: self selectedMessageName! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'md 4/6/2004 16:26'!
selectedMessageName
	"Answer the message selector of the currently selected context."

	^self selectedContext methodSelector! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'md 4/23/2004 19:11'!
selectedMessageText
	"Answer the source code of the currently selected context."

	| contents |

	contents := theMethodNode sourceText.
	Preferences browseWithPrettyPrint ifTrue: [
		contents := self selectedClass compilerClass new
				format: contents
				in: self selectedClass
				notifying: nil
				decorated: Preferences colorWhenPrettyPrinting].
	^ contents asText makeSelectorBold! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 22:48'!
send
	"Send the selected message in the accessed method, and take control in 
	the method invoked to allow further step or send."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	self selectedContext ifNil:[^self].
	self traceEventsIn:[interruptedProcess step: self selectedContext].
	self resetContext: interruptedProcess stepToSendOrReturn.
	self completeIfBottom.! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
sendButton
	"Answer the sendButton of the receiver"
	<bewareOf: #sendButtonChanged>
	^self propertyValueAt: #sendButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:32'!
sendButton: aValue
	"Modify the receiver's sendButton"
	^self propertyValueAt: #sendButton put: aValue with: #sendButtonChanged! !

!CDebugger methodsFor: 'initialize' stamp: 'bf 6/23/2005 23:25'!
setupCostume
	"Create the default costume"
	| row |
	super setupCostume.
	self extent:  600@400.
	self layout: CProportionalLayout new.

	messagePane color: Color transparent.
	messagePane layoutFrame: (0@0 corner: 0.5@0.25).
	self add: messagePane.

	stackListPane color: Color transparent.
	stackListPane layoutFrame: (0.5@0 corner: 1@0.25).
	self add: stackListPane.

	scriptList color: Color transparent.
	scriptList layoutFrame: (0.5@0 corner: 1@0.25).
"	self add: scriptList."


	completeButton label: 'Complete'.
	proceedButton label: 'Proceed'.
	restartButton label: 'Restart'.
	sendButton label: 'Send'.	
	doStepButton label: 'Step'.
	stepIntoBlockButton label: 'Through'.
	fullStackButton label: 'Full Stack'.
	whereButton label: 'Where'.

	self debuggerButtons do:[:button|
		button hResizing: #spaceFill. "vResizing: #shrinkWrap."
		button roundCorners: true.
		button height: 20.
	].
	row := CPlayer new.
	row layout: CTableLayout new.
	row layoutFrame: (0@0.25 corner: 1@0.25).
	row layout listDirection: #leftToRight; layoutInset: 2; cellInset: 2.
	row hResizing: #spaceFill; vResizing: #spaceFill.
	self debuggerButtons do: [:button | row add: button].
	self add: row.

	codePane color: Color transparent.
	codePane layoutFrame: (CLayoutFrame fractions: (0@0.25 corner: 1@0.8) offsets: (0@26 extent: 0@-26)).

	codePane hResizing: #spaceFill; vResizing: #spaceFill.
	codePane textWrap: true.
	codePane hScrollable: false; vScrollable: true.
	codePane showCaret: true.
	self add: codePane.


	receiverInspectorList color: Color transparent.
	receiverInspectorList layoutFrame: (0.0@0.8 corner: 0.25@1).
	self add: receiverInspectorList.
	
	receiverInspectorContents color: Color transparent.
	receiverInspectorContents layoutFrame: (0.25@0.8 corner: 0.5@1).
	receiverInspectorContents hResizing: #spaceFill; vResizing: #spaceFill.
	receiverInspectorContents textWrap: true.
	receiverInspectorContents hScrollable: false; vScrollable: true.
	receiverInspectorContents menu: (self newCodePaneMenu: receiverInspectorContents).
	self add: receiverInspectorContents.

	contextVariableInspectorList color: Color transparent.
	contextVariableInspectorList layoutFrame: (0.5@0.8 corner: 0.75@1).
	self add: contextVariableInspectorList.

	contextVariableInspectorContents color: Color transparent.
	contextVariableInspectorContents layoutFrame: (0.75@0.8 corner: 1@1).
	contextVariableInspectorContents hResizing: #spaceFill; vResizing: #spaceFill.
	contextVariableInspectorContents textWrap: true.
	contextVariableInspectorContents hScrollable: false; vScrollable: true.
	contextVariableInspectorContents menu: (self newCodePaneMenu: contextVariableInspectorContents).
	self add: contextVariableInspectorContents.

	CPaneResizeBehavior attachTo: messagePane.		"let user move pane boundaries"
	CPaneResizeBehavior attachTo: stackListPane.
	CPaneResizeBehavior attachTo: codePane.
	CPaneResizeBehavior attachTo: receiverInspectorList.
	CPaneResizeBehavior attachTo: receiverInspectorContents.
	CPaneResizeBehavior attachTo: contextVariableInspectorList.
	"CPaneResizeBehavior attachTo: contextVariableInspectorContents.	at bottom right corner, so not needed"



	"Give all of my contents the default look"
	costume decorate: elements color: (Color r: 1.0 g: 0.5 b: 0.5).! !

!CDebugger methodsFor: 'context stack menu' stamp: 'md 4/20/2004 14:14'!
stackListMenu
	"Set up the menu appropriately for the context-stack-list"
	| menu subMenu |

	menu := CMenu new.

	menu add: 'fullStack (f)' target: self action: #fullStack.
	menu add: 'restart (r)' target: self action: #restart.
	menu add: 'proceed (p)' target: self action: #proceed.	
	menu add: 'step (t)' target: self action: #doStep.	
	menu add: 'step through (T)' target: self action: #stepIntoBlock.	
	menu add: 'send (e)' target: self action: #send.	
	menu add: 'where (w)' target: self action: #where.	
	menu add: 'peel to first like this' target: self action: #peelToFirst.	
	menu addSeparator.
	menu add: 'senders of... (n)' target: self action: #browseSendersOfMessages.
	menu add: 'implementors of... (m)' target: self action: #browseMessages.
	menu add: 'inheritance (i)' target: self action: #methodHierarchy.
	menu add: 'versions (v)' target: self action: #browseVersions.
	menu addSeparator.
	menu add: 'inst var refs...' target: self action: #browseInstVarRefs.
	menu add: 'class var refs...' target: self action: #browseClassVarRefs.
	menu add: 'class variables' target: self action:  #browseClassVariables.
	menu add: 'class refs (N)' target: self action: #browseClassRefs.
	menu addSeparator.
	menu add: 'browse full (b)' target: self action: #browseMethodFull.
	menu add: 'file out' target: self action: #fileOutMessage.	
	menu add: 'mail out bug report' target: self action: #mailOutBugReport.
	menu addSeparator.
	menu add: 'more...' subMenu: (subMenu := CMenu new).	

	subMenu add: 'browse class hierarchy' target: self action: #classHierarchy.
	subMenu add: 'browse class' target: self action: #browseClass.
	subMenu add: 'browse method (O)' target: self action: #openSingleMessageBrowser.
	subMenu add: 'implementors of sent messages' target: self action: #browseAllMessages.
	subMenu add: 'change sets with this method' target: self action: #findMethodInChangeSets.
	subMenu addSeparator.
	subMenu add: 'inspect instances' target: self action: #inspectInstances.
	subMenu add: 'inspect subinstances' target: self action: #inspectSubInstances.
	subMenu addSeparator.
	subMenu add: 'revert to previous version' target: self action: #revertToPreviousVersion.
	subMenu add: 'remove from current change set' target: self action: #removeFromCurrentChanges.
	subMenu add: 'revert & remove from changes' target: self action: #revertAndForget.
	
	^menu 
! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/8/2004 14:49'!
stackListPane
	"Answer the stackListPane of the receiver"
	<bewareOf: #stackListPaneChanged>
	^self propertyValueAt: #stackListPane! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/8/2004 14:49'!
stackListPane: aValue
	"Modify the receiver's stackListPane"
	^self propertyValueAt: #stackListPane put: aValue with: #stackListPaneChanged! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 22:47'!
stepIntoBlock
	"Send messages until you return to the present method context.
	 Used to step into a block in the method."
	self selectedContext ifNil:[^self].
	self traceEventsIn:[interruptedProcess stepToHome: self selectedContext].
	self resetContext: interruptedProcess stepToSendOrReturn.
	self completeIfBottom.! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
stepIntoBlockButton
	"Answer the stepIntoBlockButton of the receiver"
	<bewareOf: #stepIntoBlockButtonChanged>
	^self propertyValueAt: #stepIntoBlockButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
stepIntoBlockButton: aValue
	"Modify the receiver's stepIntoBlockButton"
	^self propertyValueAt: #stepIntoBlockButton put: aValue with: #stepIntoBlockButtonChanged! !

!CDebugger methodsFor: 'context stack (message list)' stamp: 'ar 11/12/2004 18:33'!
toggleContextStackIndex: anInteger 
	"If anInteger is the same as the index of the selected context, deselect it. 
	Otherwise, the context whose index is anInteger becomes the selected 
	context."
	stackListPane selectionIndex = anInteger 
		ifTrue:[stackListPane selectionIndex: 0]
		ifFalse:[stackListPane selectionIndex: anInteger].! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 2/28/2005 15:43'!
traceEvent: anEvent message: aMessage script: aScript from: parent
	| script |
	aScript ifNotNil:[^self].
	((aMessage selector == #onEventSourceChanged) 
		and:[aMessage receiver isKindOf: AsyncScriptMessageSend]) ifTrue:[^self].
	((aMessage selector == #onUpdateTrigger:from:) 
		and:[aMessage receiver isKindOf: AsyncScriptMessageSend]) ifTrue:[^self].
	(aMessage selector == #privateChangeValueTo:)
		ifTrue:[^self].
	script := Processor activeProcess.
	script traceEvents: false. "or else..."
	self startScript:#debugScript: withArguments: {script}.
	script traceEvents: true.
	script suspend.
! !

!CDebugger methodsFor: 'events-debugging' stamp: 'ar 11/12/2004 20:07'!
traceEventsIn: aBlock
	Processor activeProcess eventTracer: self.
	Processor activeProcess traceEvents: true.
	[aBlock value] ensure:[
		Processor activeProcess traceEvents: false.
		Processor activeProcess eventTracer: nil.
	].! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:36'!
up
	"move up the context stack to the next (enclosed) context"

	stackListPane cursor > 1 ifTrue: [self toggleContextStackIndex: stackListPane cursor-1]! !

!CDebugger methodsFor: 'inspectors' stamp: 'md 4/28/2004 15:59'!
updateInspectors
	self contextVariableInspectorUpdate.
	self receiverInspectorUpdate.! !

!CDebugger methodsFor: 'context stack menu' stamp: 'ar 11/12/2004 18:37'!
where
	"Select the expression whose evaluation was interrupted."

	selectingPC := true.
! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
whereButton
	"Answer the whereButton of the receiver"
	<bewareOf: #whereButtonChanged>
	^self propertyValueAt: #whereButton! !

!CDebugger methodsFor: 'field-access' stamp: 'md 4/27/2004 16:33'!
whereButton: aValue
	"Modify the receiver's whereButton"
	^self propertyValueAt: #whereButton put: aValue with: #whereButtonChanged! !

CCodeHolder subclass: #CInspector
	instanceVariableNames: 'object selectionUpdateTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CInspector subclass: #CContextVariablesInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CContextVariablesInspector defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CContextVariablesInspector methodsFor: 'code' stamp: 'md 4/7/2004 16:46'!
doItContext

	^object! !

!CContextVariablesInspector methodsFor: 'code' stamp: 'md 4/7/2004 16:46'!
doItReceiver

	^object receiver! !

!CContextVariablesInspector methodsFor: 'accessing' stamp: 'md 4/7/2004 16:46'!
fieldList 
	"Refer to the comment in Inspector|fieldList."

	object == nil ifTrue: [^Array with: 'thisContext'].
	^(Array with: 'thisContext' with: 'all temp vars') , object tempNames! !

!CContextVariablesInspector methodsFor: 'selecting' stamp: 'ar 1/22/2005 14:32'!
replaceSelectionValue: anObject 
	"Refer to the comment in Inspector|replaceSelectionValue:."

	listPane selectionIndex = 1
		ifTrue: [^object]
		ifFalse: [^object tempAt: listPane selectionIndex - 2 put: anObject]! !

!CContextVariablesInspector methodsFor: 'selecting' stamp: 'ar 1/22/2005 14:32'!
selection 
	"Refer to the comment in Inspector|selection."
	listPane selectionIndex = 0 ifTrue:[^''].
	listPane selectionIndex = 1 ifTrue: [^object].
	listPane selectionIndex = 2
		ifTrue: [^object tempsAndValues]
		ifFalse: [^object tempAt: listPane selectionIndex - 2]! !

CInspector defineFields: '<?xml  version="1.0" ?>
<fields>
	<field instanceVariable="true" toGet="object" toSet="object:">object</field>
	<field instanceVariable="true" toGet="selectionUpdateTime" toSet="selectionUpdateTime:">selectionUpdateTime</field>
	<field toGet="listPane" toSet="listPane:" changeEvent="listPaneChanged">listPane</field>
	<field toGet="contentsPane" toSet="contentsPane:" changeEvent="contentsPaneChanged">contentsPane</field>
</fields>'!

!CInspector class methodsFor: 'as yet unclassified' stamp: 'md 4/8/2004 12:13'!
inspect: anObject 
	"Answer an instance of me to provide an inspector for anObject."

	^self new inspect: anObject! !

!CInspector class methodsFor: 'as yet unclassified' stamp: 'md 4/8/2004 12:10'!
openForTest
	^self openOn: self! !

!CInspector class methodsFor: 'as yet unclassified' stamp: 'md 4/22/2004 13:13'!
openForTest2
	^self openOn: 1@2! !

!CInspector class methodsFor: 'as yet unclassified' stamp: 'md 4/8/2004 12:11'!
openOn: anObject
	^ self openOn: anObject withLabel: anObject defaultLabelForInspector! !

!CInspector class methodsFor: 'as yet unclassified' stamp: 'md 5/3/2004 13:55'!
openOn: anObject withEvalPane: aBoolean
	^self openOn: anObject! !

!CInspector class methodsFor: 'as yet unclassified' stamp: 'ar 2/7/2005 01:14'!
openOn: anObject withLabel: aLabel
	|  inspector |
	inspector := self inspect: anObject.
	inspector open.
	inspector label: aLabel.
	^ inspector
		! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 15:00'!
addCollectionItemsTo: aMenu
	"If the current selection is an appropriate collection, add items to aMenu that cater to that kind of selection"

	| sel |
	((((sel := self selection) isMemberOf: Array) or: [sel isMemberOf: OrderedCollection]) and: 
		[sel size > 0]) ifTrue: [
			aMenu addList: #(
				('inspect element...'					inspectElement))].

	(sel isKindOf: MorphExtension) ifTrue: [
			aMenu addList: #(
				('inspect property...'				inspectElement))].! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/8/2004 14:17'!
baseFieldList
	"Answer an Array consisting of 'self'
	and the instance variable names of the inspected object."

	^ (Array with: 'self' with: 'all inst vars')
			, object class allInstVarNames! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/23/2004 14:00'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.  If in mvc, an old-style protocol browser is opened instead."

	self cNotYet.

	"| objectToRepresent |
	Smalltalk isMorphic ifFalse: [^ self spawnProtocol].

	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	InstanceBrowser new openOnObject: objectToRepresent inWorld: ActiveWorld showingSelector: nil"! !

!CInspector methodsFor: 'menu commands' stamp: 'ar 11/24/2004 11:48'!
chasePointers
	| saved finder |
	self selectionIndex == 0 ifTrue: [^ self changed: #flash].
	saved := self object.
	[self object: nil.
		finder := CPointerFinder new.
		finder goal: saved.
		finder search.
		finder open.
	] ensure: [self object: saved]! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 14:42'!
classOfSelection
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/22/2004 16:53'!
classVarRefs
	"Request a browser of methods that store into a chosen instance variable"

	| aClass |
	(aClass := self classOfSelection) ifNotNil:
		[self  browseClassVarRefs: aClass].
! !

!CInspector methodsFor: 'selecting' stamp: 'md 4/21/2004 17:26'!
contentsIsString
	"Hacked so contents empty when deselected and = long printString when item 2"

	^ (listPane selectionIndex = 2) | (listPane selectionIndex = 0)! !

!CInspector methodsFor: 'field access' stamp: 'md 4/8/2004 12:02'!
contentsPane
	"Answer the contentsPane of the receiver"
	<bewareOf: #contentsPaneChanged>
	^self propertyValueAt: #contentsPane! !

!CInspector methodsFor: 'field access' stamp: 'md 4/8/2004 12:02'!
contentsPane: aValue
	"Modify the receiver's contentsPane"
	^self propertyValueAt: #contentsPane put: aValue with: #contentsPaneChanged! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 17:26'!
copyName
	"Copy the name of the current variable, so the user can paste it into the window below and work with is.  If collection, do (xxx at: 1). "

	| sel aClass |
	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: listPane selectionIndex - 2.
	(self selection isKindOf: Collection) ifTrue: [sel := '(',sel,' at: 1)'].
	Clipboard clipboardText: sel asText.	"no undo allowed"! !

!CInspector methodsFor: 'initialize' stamp: 'ar 5/26/2005 20:21'!
defaultWindowColor
	^Color r: 0.652 g: 0.8 b: 0.504! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 17:05'!
defsOfSelection
	"Open a browser on all defining references to the selected instance variable, if that's what currently selected. "
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation  browseAllStoresInto: sel from: aClass! !

!CInspector methodsFor: 'updating' stamp: 'ar 11/18/2004 19:20'!
displayContents

	| newText |
	contentsPane modified ifTrue:[^self].
	newText := self contentsIsString
		ifTrue: [newText := self selection]
		ifFalse: ["keep it short to reduce time to compute it"
			self selectionPrintString ].
	newText = contentsPane contents ifFalse: [
		self contentsPane contents: newText.
	]! !

!CInspector methodsFor: 'code' stamp: 'md 4/21/2004 14:50'!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^object! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/8/2004 14:17'!
fieldList
	"Answer the base field list plus an abbreviated list of indices."

	object class isVariable ifFalse: [^ self baseFieldList].
	^ self baseFieldList ,
		(object basicSize <= (self i1 + self i2)
			ifTrue: [(1 to: object basicSize)
						collect: [:i | i printString]]
			ifFalse: [(1 to: self i1) , (object basicSize-(self i2-1) to: object basicSize)
						collect: [:i | i printString]])! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/27/2004 13:49'!
fieldListKeyMap
	"Set up the keymap"
	| map |
	map := CKeyboardDispatchTable new.
	map add: #(cmd i) action: #inspectSelection.
	map add: #(cmd I) action: #exploreSelection.
	map add: #(cmd b) action: #browseMethodFull.
	map add: #(cmd c) action: #copyName.
	map add: #(cmd p) action: #browseFullProtocol.
	map add: #(cmd r) action: #browseClassRefs.
	map add: #(cmd v)  action: #viewerForValue.
	^map! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 19:21'!
fieldListMenu
	"Arm the supplied menu with items for the field-list of the receiver"

	| menu |
	menu := CMenu new.
	menu label: 'System Categories'.
	menu addList: #(
		('inspect (i)'						inspectSelection)
		-
		('method refs to this inst var'		referencesToSelection)
		('methods storing into this inst var'	defsOfSelection)
		('objects pointing to this value'		objectReferencesToSelection)
		('chase pointers'					chasePointers)
		-
		('browse full (b)'					browseMethodFull)
		('browse class'						browseMethodFull)
		('browse hierarchy'					classHierarchy)
		('browse protocol (p)'				browseFullProtocol)
		-
		('inst var refs...'					browseInstVarRefs)
		('inst var defs...'					browseInstVarDefs)
		('class var refs...'					classVarRefs)
		('class variables'					browseClassVariables)
		('class refs (N)'						browseClassRefs)
		-
		('copy name (c)'					copyName)		
		('basic inspect'						inspectBasic)
		) target: self.
	^menu			

	! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 11:10'!
i1
	"This is the max index shown before skipping to the 
	last i2 elements of very long arrays"
	^ 100! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 11:10'!
i2
	"This is the number of elements to show at the end
	of very long arrays"
	^ 10! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 14:53'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 250 @ 200! !

!CInspector methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:57'!
initialize
	super initialize.
	"Define our components"
	
	self define: #listPane as: CStringList new.
	listPane menu: self fieldListMenu.
	"listPane userDataAt: #keyMap put: self fieldListKeyMap."
	listPane allowDeselect: true.

	self define: #contentsPane as: CTextEditor new.
! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 17:24'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no 
	current selection."

	object := anObject.
	listPane contents: self fieldList.! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 17:25'!
inspectBasic
	"Bring up a non-special inspector"

	listPane selectionIndex = 0 ifTrue: [^ object basicInspect].
	self selection basicInspect! !

!CInspector methodsFor: 'menu commands' stamp: 'ar 12/27/2004 10:19'!
inspectElement
	| sel selSize countString count nameStrs |
	"Create and schedule an Inspector on an element of the receiver's model's currently selected collection."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	((sel := self selection) isKindOf: SequenceableCollection) ifFalse:
		[(sel isKindOf: MorphExtension) ifTrue: [^ sel inspectElement].
		^ sel inspect].
	(selSize := sel size) == 1 ifTrue: [^ sel first inspect].
	selSize <= 20 ifTrue:
		[nameStrs := (1 to: selSize) asArray collect: [:ii | 
			ii printString, '   ', ((sel at: ii) printStringLimitedTo: 25)].
		count := UIManager default chooseFrom: nameStrs title: 'which element?'.
		count = 0 ifTrue: [^ self].
		^ (sel at: count) inspect].

	countString := self request: 'Which element? (1 to ', selSize printString, ')' initialAnswer: '1'.
	countString isEmptyOrNil ifTrue: [^ self].
	count := Integer readFrom: (ReadStream on: countString).
	(count > 0 and: [count <= selSize])
		ifTrue: [(sel at: count) inspect]
		ifFalse: [self beep]! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 11:11'!
inspectSelection
	"Create and schedule an Inspector on the receiver's model's currently selected object."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	^CInspector openOn: self selection.! !

!CInspector methodsFor: 'field access' stamp: 'md 4/8/2004 12:02'!
listPane
	"Answer the listPane of the receiver"
	<bewareOf: #listPaneChanged>
	^self propertyValueAt: #listPane! !

!CInspector methodsFor: 'field access' stamp: 'md 4/8/2004 12:02'!
listPane: aValue
	"Modify the receiver's listPane"
	^self propertyValueAt: #listPane put: aValue with: #listPaneChanged! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/9/2004 15:49'!
object
	^object! !

!CInspector methodsFor: 'accessing' stamp: 'ar 11/12/2004 19:54'!
object: anObject 
	"Set anObject to be the object being inspected by the receiver."

	| oldIndex |
	anObject == object
		ifFalse:
			[oldIndex := listPane selectionIndex <= 2 ifTrue: [listPane selectionIndex] ifFalse: [0].
			self inspect: anObject.
			oldIndex := oldIndex min: self fieldList size].! !

!CInspector methodsFor: 'menu commands' stamp: 'ar 11/24/2004 10:07'!
objectReferencesToSelection
	"Open a list inspector on all the objects that point to the value of the selected instance variable, if any.  "
	| obj aList shortName |
	self selectionIndex == 0 ifTrue: [^self flash].
	obj := self selection.
	aList := Smalltalk pointersTo: obj except: (Array with: self object).
	aList size > 0 ifFalse: [^self flash].
	shortName := (obj name ifNil: [obj printString]) contractTo: 20.
	CInspector openOn: aList withEvalPane: false.! !

!CInspector methodsFor: 'events' stamp: 'md 4/22/2004 13:18'!
onAcceptContentsPane
	"Update the class list when the system category list's cursor changes"
	| result |

	<on: save in: contentsPane> 
	
	result := self doItReceiver class evaluatorClass new
				evaluate: (ReadStream on: contentsPane contents)
				in: self doItContext
				to: self doItReceiver
				notifying: nil	"fix this"
				ifFail:  [^ false].
	result == #failedDoit ifFalse: 
			[contentsPane contents: result printString.
			self replaceSelectionValue: result].
	self onListCursorChanged.
	! !

!CInspector methodsFor: 'events' stamp: 'ar 6/19/2005 13:44'!
onContentsPaneKeyDown: evt
	<on: keyDown in: contentsPane>
	self onCodePaneKeyDown: evt in: contentsPane! !

!CInspector methodsFor: 'events' stamp: 'md 4/27/2004 12:58'!
onListCursorChanged
	"Update the class list when the system category list's cursor changes"
	<on: cursorChanged in: listPane> 
	self displayContents.
	! !

!CInspector methodsFor: 'initialize' stamp: 'ar 12/18/2004 23:39'!
postOpen
	self preLoad: listPane menu.
	self preLoad: contentsPane menu.
	super postOpen.! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/21/2004 17:06'!
referencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what currently selected.  1/25/96 sw"
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation   browseAllAccessesTo: sel from: aClass! !

!CInspector methodsFor: 'selecting' stamp: 'ar 11/12/2004 19:54'!
replaceSelectionValue: anObject 
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. The value of the selected variable is set to the value, 
	anObject."
	| basicIndex |
	listPane selectionIndex <= 2 ifTrue: [^object].
	object class isVariable
		ifFalse: [^ object instVarAt: listPane selectionIndex - 2 put: anObject].
	basicIndex := listPane selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^object basicAt: basicIndex put: anObject]
		ifFalse: [^object basicAt: object basicSize - (self i1 + self i2) + basicIndex
					put: anObject]! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 10:45'!
selectedClass
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 10:49'!
selectedMessage
	"This method was automatically generated"
	^listPane selection.! !

!CInspector methodsFor: 'selecting' stamp: 'md 4/21/2004 17:12'!
selectedSlotName

	^ self fieldList at: self selectionIndex! !

!CInspector methodsFor: 'selecting' stamp: 'md 4/21/2004 17:25'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."
	| basicIndex |
	listPane selectionIndex = 0 ifTrue: [^ ''].
	listPane selectionIndex = 1 ifTrue: [^ object].
	listPane selectionIndex = 2 ifTrue: [^ object longPrintString].
	(listPane selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: listPane selectionIndex - 2].
	basicIndex := listPane selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^ object basicAt: basicIndex]
		ifFalse: [^ object basicAt: object basicSize - (self i1 + self i2) + basicIndex]! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 17:27'!
selectionIndex
	^listPane selectionIndex.! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/21/2004 17:28'!
selectionIndex: anIndex
	listPane selectionIndex: anIndex.! !

!CInspector methodsFor: 'accessing' stamp: 'md 4/27/2004 13:12'!
selectionPrintString
	| mtext nm |
	selectionUpdateTime := [mtext := [self selection printStringLimitedTo: 5000]
		on: Error do: 
		[nm := self selectionIndex < 3
					ifTrue: ['self']
					ifFalse: [self selectedSlotName].
		mtext := ('<error in printString: evaluate "' , nm , ' printString" to debug>') asText.
		mtext
			addAttribute: TextColor red
			from: 1
			to: mtext size.
		mtext]] timeToRun.
	^ mtext! !

!CInspector methodsFor: 'selecting' stamp: 'md 4/21/2004 17:26'!
selectionUnmodifiable
	"Answer if the current selected variable is modifiable via acceptance in the code pane.  For most inspectors, no selection and a selection of self (selectionIndex = 1) are unmodifiable"

	^ listPane selectionIndex <= 2! !

!CInspector methodsFor: 'accessing' stamp: 'tk 2/28/2005 11:49'!
selectionUpdateTime
	^ selectionUpdateTime! !

!CInspector methodsFor: 'accessing' stamp: 'tk 2/28/2005 11:50'!
selectionUpdateTime: aValue

	selectionUpdateTime := aValue! !

!CInspector methodsFor: 'initialize' stamp: 'ar 6/19/2005 13:44'!
setupCostume
	"Create the default costume"
	super setupCostume.
	self extent: self initialExtent.
	self layout: CProportionalLayout new.

	listPane color: Color transparent.
	listPane layoutFrame: (0@0 extent: 0.25@0.7).
	self add: listPane.

	contentsPane color: Color transparent.
	contentsPane layoutFrame: (0.25@0 extent: 0.75@0.7).
	contentsPane hResizing: #spaceFill; vResizing: #spaceFill.
	contentsPane textWrap: true.
	contentsPane hScrollable: false; vScrollable: true.
	contentsPane menu: (self newCodePaneMenu: contentsPane).
	self add: contentsPane.

	codePane color: Color transparent.
	codePane layoutFrame: (0@0.7 corner: 1@1).
	codePane hResizing: #spaceFill; vResizing: #spaceFill.
	codePane textWrap: true.
	codePane hScrollable: false; vScrollable: true.
	self add: codePane.

	CPaneResizeBehavior attachTo: listPane.		"let user move pane boundaries"
	CPaneResizeBehavior attachTo: contentsPane.
	"CPaneResizeBehavior attachTo: codePane.	at bottom right corner, so not needed"



	"Give all of my contents the default look"
	self decorate: elements color: self defaultWindowColor.
! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/23/2004 13:59'!
spawnFullProtocol
	"Spawn a window showing full protocol for the receiver's selection"

	self cNotYet.

	"| objectToRepresent |
	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	ProtocolBrowser openFullProtocolForClass: objectToRepresent class"! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/23/2004 13:59'!
spawnProtocol
	"Spawn a protocol on browser on the receiver's selection"

	self cNotYet.

	"| objectToRepresent |
	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	ProtocolBrowser openSubProtocolForClass: objectToRepresent class"! !

!CInspector methodsFor: 'updating' stamp: 'md 4/27/2004 13:31'!
updateContentsPane
	<ticking: 2> 

	"todo: as fast as possible, update all selectionUpdateTime * 10 mseconds"

	self displayContents.! !

!CInspector methodsFor: 'updating' stamp: 'ar 10/18/2004 20:43'!
updateFull
	"Full update not more often than once every 5 seconds"
	| newContents |
	<ticking: 0.2> 
	newContents := self fieldList.
	newContents = listPane contents ifFalse:[listPane contents: self fieldList].! !

!CInspector methodsFor: 'menu commands' stamp: 'md 4/23/2004 13:59'!
viewerForValue
	"Open up a viewer on the value of the receiver's current selection"

	self cNotYet.

	"| objectToRepresent |
	objectToRepresent := self selectionIndex == 0 ifTrue: [object] ifFalse: [self selection].
	objectToRepresent beViewed"
	! !

CInspector subclass: #CObjectInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CObjectInspector commentStamp: 'tk 2/28/2005 12:03' prior: 0!
An Object Inspector shows the insides of an object.  Both fields and actual instance variables are visible.  Virtual fields are not shown.  To see an inspector on an object, get the halo of the object, and click the blue handle at the bottome with the wrench in it.  The left pane has a list of variables inside the object.  Click one and see the value on the right.  The bottom pane is for typing Tweak code and then choosing 'do it' or print it' from the right button menu.[general]


### Allow Users to see the above explanation ###!

CObjectInspector defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CObjectInspector methodsFor: 'accessing' stamp: 'ar 10/18/2004 20:45'!
fieldList
	^  (Array with: 'self: ', object defaultLabelForInspector with: 'all inst vars'), 
		(self objectFieldList collect: [ :field | field name])! !

!CObjectInspector methodsFor: 'accessing' stamp: 'ar 10/18/2004 20:48'!
objectFieldList
	| fields |
	fields := object class allFields.
	self showVirtualFields ifFalse:[
		fields := fields reject:[:f| f isVirtualField].
	].
	fields := fields reject:[:each| each toGet isNil].
	^fields! !

!CObjectInspector methodsFor: 'accessing' stamp: 'ar 10/18/2004 20:46'!
replaceSelectionValue: anObject 
	"Add colon to fieldname to get setter selector, and send it to object with the argument.
	 Refer to the comment in Inspector|replaceSelectionValue:."

	listPane selectionIndex = 1 ifTrue: [^object].
	^object perform: ((self fieldList at: listPane selectionIndex), ':') asSymbol with: anObject! !

!CObjectInspector methodsFor: 'accessing' stamp: 'ar 10/18/2004 20:46'!
selection 
	"Refer to the comment in Inspector|selection."
	| field |
	listPane selectionIndex = 0 ifTrue:[^object printString].
	listPane selectionIndex = 1 ifTrue: [^object].
	listPane selectionIndex = 2 ifTrue:[^object longPrintString].
	field := self objectFieldList at: listPane selectionIndex-2.
	^object getField: field.! !

!CObjectInspector methodsFor: 'accessing' stamp: 'ar 10/18/2004 20:39'!
showVirtualFields
	"Later: Allow us to modify this"
	^false! !

CCodeHolder subclass: #CMessageSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CMessageSet subclass: #CChangedMessageSet
	instanceVariableNames: 'changeSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CChangedMessageSet defineFields: '<?xml  version="1.0" ?>
<fields>
	<field instanceVariable="true" toGet="changeSet" toSet="changeSet:">changeSet</field>
</fields>'!

!CChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'md 4/23/2004 17:18'!
openFor: aChangeSet
	"Open up a ChangedMessageSet browser on the given change set; this is a conventional message-list browser whose message-list consists of all the methods in aChangeSet.  After any method submission, the message list is refigured, making it plausibly dynamic"

	| messageSet |

	messageSet := aChangeSet changedMessageListAugmented select: [ :each | each isValid].
	self 
		openMessageList: messageSet 
		name: 'Methods in Change Set ', aChangeSet name
		autoSelect: nil
		changeSet: aChangeSet! !

!CChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'md 4/23/2004 17:22'!
openMessageList: messageList name: labelString autoSelect: autoSelectString changeSet: aChangeSet

	self openMessageList: messageList name: labelString autoSelect: autoSelectString.! !

!CChangedMessageSet methodsFor: 'initialization' stamp: 'md 4/23/2004 17:18'!
changeSet: aChangeSet
	changeSet := aChangeSet! !

!CChangedMessageSet methodsFor: 'acceptance' stamp: 'md 4/23/2004 17:18'!
contents: aString notifying: aController
	"Accept the string as new source for the current method, and make certain the annotation pane gets invalidated"

	| existingSelector existingClass superResult newSelector |
	existingSelector := self selectedMessageName.
	existingClass := self selectedClassOrMetaClass.

	superResult := super contents: aString notifying: aController.
	superResult ifTrue:  "succeeded"
		[newSelector := Parser new parseSelector: aString.
		newSelector ~= existingSelector
			ifTrue:   "Selector changed -- maybe an addition"
				[self reformulateList.
				self changed: #messageList.
				self messageList doWithIndex:
					[:aMethodReference :anIndex |
						(aMethodReference actualClass == existingClass and:
									[aMethodReference methodSymbol == newSelector])
							ifTrue:
								[self messageListIndex: anIndex]]]].
	^ superResult! !

!CChangedMessageSet methodsFor: 'message list' stamp: 'md 4/23/2004 17:18'!
growable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!CChangedMessageSet methodsFor: 'reformulation' stamp: 'md 4/23/2004 17:18'!
reformulateList
	"Reformulate the message list of the receiver"

	self initializeMessageList: (changeSet changedMessageListAugmented select: 
		[:each | each isValid])
! !

CMessageSet defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="autoSelectString" toSet="autoSelectString:" changeEvent="autoSelectStringChanged">autoSelectString</field>
	<field toGet="selectedReference" toSet="selectedReference:" changeEvent="selectedReferenceChanged">selectedReference</field>
	<field toGet="messageList" toSet="messageList:" changeEvent="messageListChanged">messageList</field>
</fields>'!

!CMessageSet class methodsFor: 'instance creation' stamp: 'md 4/23/2004 14:42'!
openMessageList: messageList name: title 
	| msgSet |
	msgSet := self new.
	msgSet window label: title.
	msgSet messageList contents: messageList.
	msgSet open.
	^msgSet! !

!CMessageSet class methodsFor: 'instance creation' stamp: 'bf 5/27/2005 15:58'!
openMessageList: messageList name: title autoSelect: aString
	| msgSet |
	msgSet := self new.
	msgSet label: title.
	msgSet autoSelectString: aString.
	msgSet messageList contents: messageList.
	msgSet open.
	^msgSet! !

!CMessageSet methodsFor: 'field access' stamp: 'ar 3/17/2004 15:11'!
autoSelectString
	"This method was automatically generated"
	<bewareOf: #autoSelectStringChanged>
	^self propertyValueAt: #autoSelectString! !

!CMessageSet methodsFor: 'field access' stamp: 'ar 3/11/2004 14:23'!
autoSelectString: aValue
	"This method was automatically generated"
	^self propertyValueAt: #autoSelectString put: aValue with: #autoSelectStringChanged! !

!CMessageSet methodsFor: 'initialize' stamp: 'ar 5/26/2005 20:22'!
defaultWindowColor
	^Color r: 0.664 g: 0.8 b: 0.8! !

!CMessageSet methodsFor: 'actions-codepane' stamp: 'tk 9/16/2004 16:48'!
defineMessageFrom: aString
	"Compile the expressions in aString. Install the compiled method in the selected class classified under the currently selected message category name. Answer the selector obtained if compilation succeeds, nil otherwise."

	| selectedMessageName selector oldMessageList methodRef |
	methodRef := messageList selection.
	selectedMessageName := methodRef methodSymbol.
	oldMessageList := self messageList.

	selector := (Parser new parseSelector: aString).
	selector := methodRef actualClass
				compile: aString
				classified: methodRef category
				notifying: self.
	selector == nil ifTrue: [^ nil].
	codePane contents: aString copy.
	selector ~~ selectedMessageName 
		ifTrue: 
			[(oldMessageList includes: selector)
				ifFalse: ["messageList add: (MessageReference new ..)."].
			messageList selection: selector].
	^ selector! !

!CMessageSet methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:57'!
initialize
	"Initialize the receiver"
	super initialize.

	self define: #messageList as: CStringList new.
	messageList allowDeselect: true.
	messageList menu: self messageListMenu.
! !

!CMessageSet methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:28'!
messageList
	"Answer the messageList of the receiver"
	<bewareOf: #messageListChanged>
	^self propertyValueAt: #messageList! !

!CMessageSet methodsFor: 'accessing' stamp: 'ar 3/14/2004 15:28'!
messageList: aValue
	"Modify the receiver's messageList"
	^self propertyValueAt: #messageList put: aValue with: #messageListChanged! !

!CMessageSet methodsFor: 'menu' stamp: 'ar 3/6/2004 23:58'!
messageListMenu
	^nil! !

!CMessageSet methodsFor: 'events' stamp: 'tk 9/16/2004 16:31'!
onAcceptCodePane
	"Compile the method when the user saves in the pane."
	| str |
	<on: save in: codePane> 
	str := codePane contents asString.
	self defineMessageFrom: str.
	! !

!CMessageSet methodsFor: 'events' stamp: 'ar 3/7/2004 00:08'!
onMessageListCursorChanged
	"Update the code pane when the message list's cursor changes"
	<on: cursorChanged in: messageList>
	(self okToChangeList: messageList from: self selectedReference)
		ifTrue:[self selectedReference: messageList selection]! !

!CMessageSet methodsFor: 'events' stamp: 'ar 12/15/2004 00:01'!
onSelectedReferenceChanged
	<on: selectedReferenceChanged>
	self onUpdateCodePane.! !

!CMessageSet methodsFor: 'events' stamp: 'ar 12/15/2004 00:01'!
onUpdateCodePane
	| mref code |
	mref := self selectedReference ifNil:[^codePane contents: ''].
	code := mref sourceCode asText makeSelectorBoldIn: (mref actualClass ifNil:[self]).
	codePane contents: code.
	self autoSelectString ifNil:[^self].
	codePane findText: self autoSelectString wrapping: false caseSensitive: true.
	codePane showCaret: true.! !

!CMessageSet methodsFor: 'initialize' stamp: 'ar 12/18/2004 23:40'!
postOpen
	self preLoad: messageList menu.
	super postOpen.! !

!CMessageSet methodsFor: 'field access' stamp: 'tk 9/23/2004 21:15'!
selectedMessage

	selectedReference ifNil: [^ nil].
	^ selectedReference methodSymbol! !

!CMessageSet methodsFor: 'field access' stamp: 'ar 3/17/2004 15:11'!
selectedReference
	"This method was automatically generated"
	<bewareOf: #selectedReferenceChanged>
	^self propertyValueAt: #selectedReference! !

!CMessageSet methodsFor: 'field access' stamp: 'ar 3/11/2004 14:23'!
selectedReference: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selectedReference put: aValue with: #selectedReferenceChanged! !

!CMessageSet methodsFor: 'initialize' stamp: 'ar 5/26/2005 20:22'!
setupCostume
	"Create the default costume"
	super setupCostume.
	self extent: 400@265.

	messageList color: Color transparent.
	messageList layoutFrame: (0@0 corner: 1@0.3).
	self add: messageList.

	CPaneResizeBehavior attachTo: messageList.		"let user move pane boundaries"

	codePane layoutFrame: (0@0.3 corner: 1@1).

	"Give all of my contents the default look"
	self decorate: elements color: self defaultWindowColor.! !

CCodeHolder subclass: #CPreDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CPreDebugger defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="notifyPane" toSet="notifyPane:" changeEvent="notifyPaneChanged">notifyPane</field>
	<field toGet="proceedButton" toSet="proceedButton:" changeEvent="proceedButtonChanged">proceedButton</field>
	<field toGet="abortButton" toSet="abortButton:" changeEvent="abortButtonChanged">abortButton</field>
	<field toGet="debugButton" toSet="debugButton:" changeEvent="debugButtonChanged">debugButton</field>
</fields>'!

!CPreDebugger methodsFor: 'events' stamp: 'ar 2/7/2005 01:05'!
abort
	<on: fire in: abortButton>
	player abort.
	self close.! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:47'!
abortButton
	"Answer the abortButton of the receiver"
	<bewareOf: #abortButtonChanged>
	^self propertyValueAt: #abortButton! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:47'!
abortButton: aValue
	"Modify the receiver's abortButton"
	^self propertyValueAt: #abortButton put: aValue with: #abortButtonChanged! !

!CPreDebugger methodsFor: 'events' stamp: 'ar 2/7/2005 01:07'!
debug
	<on: fire in: debugButton>
	player debug.
	player label: self label.
	self close.! !

!CPreDebugger methodsFor: 'events' stamp: 'ar 2/7/2005 01:05'!
debugAt
	<on: cursorChanged in: notifyPane> 
	player debugAt: notifyPane selectionIndex.
	self close.! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:47'!
debugButton
	"Answer the debugButton of the receiver"
	<bewareOf: #debugButtonChanged>
	^self propertyValueAt: #debugButton! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:47'!
debugButton: aValue
	"Modify the receiver's debugButton"
	^self propertyValueAt: #debugButton put: aValue with: #debugButtonChanged! !

!CPreDebugger methodsFor: 'initializing' stamp: 'ar 4/24/2005 17:57'!
initialize
	"Initialize the receiver"
	super initialize.

	self define: #notifyPane as: CStringList new.
	"messageList menu: self messageListMenu."

	self define: #proceedButton as: CButton new.
	self define: #abortButton as: CButton new.
	self define: #debugButton as: CButton new.

	! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:46'!
notifyPane
	"Answer the notifyPane of the receiver"
	<bewareOf: #notifyPaneChanged>
	^self propertyValueAt: #notifyPane! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:46'!
notifyPane: aValue
	"Modify the receiver's notifyPane"
	^self propertyValueAt: #notifyPane put: aValue with: #notifyPaneChanged! !

!CPreDebugger methodsFor: 'events' stamp: 'ar 7/19/2004 18:58'!
onCloseRequest
	<on: closeRequest in: window>
	self abort.! !

!CPreDebugger methodsFor: 'events' stamp: 'ar 6/24/2005 11:29'!
proceed
	<on: fire in: proceedButton>
	self close.
	self waitTick.
	player proceed.! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:46'!
proceedButton
	"Answer the proceedButton of the receiver"
	<bewareOf: #proceedButtonChanged>
	^self propertyValueAt: #proceedButton! !

!CPreDebugger methodsFor: 'accessing' stamp: 'md 4/28/2004 12:46'!
proceedButton: aValue
	"Modify the receiver's proceedButton"
	^self propertyValueAt: #proceedButton put: aValue with: #proceedButtonChanged! !

!CPreDebugger methodsFor: 'initializing' stamp: 'ar 5/26/2005 20:23'!
setupCostume
	"Create the default costume"
	super setupCostume.
	self extent:  450 @ 200.
	self layout: CProportionalLayout new.

	proceedButton label: 'Proceed'.
	proceedButton layoutFrame: (0@0 extent: 0.33@0.2).
	proceedButton hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	proceedButton roundCorners: true.
	self add: proceedButton.

	abortButton label: 'Abort'.
	abortButton layoutFrame: (0.33@0 extent: 0.33@0.2).
	abortButton hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	abortButton roundCorners: true.
	self add: abortButton.

	debugButton label: 'Debug'.
	debugButton layoutFrame: (0.66@0 extent: 0.34@0.2).
	debugButton hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	debugButton roundCorners: true.
	self add: debugButton.

	notifyPane color: Color transparent.
	notifyPane layoutFrame: (0@0.2 corner: 1@1).
	self add: notifyPane.

	costume decorate: elements color: (Color r: 1.0 g: 0.5 b: 0.5).! !

CCodeHolder subclass: #CTranscriptView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CTranscriptView commentStamp: 'tk 2/28/2005 14:16' prior: 0!
A Transcript is a window that reports on some things that happen in Tweak.  If you make objects do thing with scripts, you probably don't need the Transcript.  The user can type in the Transcript and select the text and choose 'do it'.  Unlike other systems, pressing return does not evaluate an expression.  Text in the Transcript is not saved.  It is tossed out when it is some number of characters from the end.  Get a Transcript from the Tools menu in the main menu bar.[general]


### Allow Users to see the above explanation ###
TranscriptView is an extremely simple view on a transcript stream.!

CTranscriptView defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="stream" toSet="stream:" changeEvent="streamChanged">stream</field>
</fields>'!

!CTranscriptView class methodsFor: 'initialize' stamp: 'ar 1/22/2005 22:46'!
initialize
	"CTranscriptView initialize"
	self registerOpenCommand.! !

!CTranscriptView class methodsFor: 'initialize' stamp: 'ar 6/1/2005 10:03'!
openTranscript
	"Open a new transcript"
	| view |
	(Transcript isKindOf: CTranscriptStream) ifFalse:[
		CTranscriptStream newTranscript: CTranscriptStream new.
	].
	view := self new.
	view stream: Transcript.
	^view open.
! !

!CTranscriptView class methodsFor: 'initialize' stamp: 'ar 1/22/2005 22:45'!
registerOpenCommand
	"CTranscriptView initialize"
	Smalltalk at: #CProjectBuilder ifPresent:[:pb|
		pb registerOpenCommand: { 'Transcript' . { self . #openTranscript }. 'A Transcript' }.
	].! !

!CTranscriptView methodsFor: 'initialize' stamp: 'ar 11/23/2004 16:55'!
initialize
	super initialize.
	stream := CTranscriptStream new.
! !

!CTranscriptView methodsFor: 'events' stamp: 'ar 12/19/2004 18:11'!
onAppendEntry: aString with: anEvent
	"This is dog stupid and cheap... but it works."
	<on: appendEntry in: stream>
	anEvent withDroppedEventsDo:[:evt|
		(codePane contents size > stream characterLimit) ifTrue:[
			codePane selectFrom: 1 to: codePane contents size // 2.
			codePane paste: '' select: false.
		].
		codePane selectAt: codePane contents size+1.
		codePane paste: evt arguments first select: false.
	].! !

!CTranscriptView methodsFor: 'initialize' stamp: 'ar 2/7/2005 01:06'!
setupCostume
	super setupCostume.
	self label: 'Transcript'.
	self color: (Color r: 1.0 g: 0.677 b: 0.032).
! !

!CTranscriptView methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
stream
	"Answer the stream of the receiver"
	<bewareOf: #streamChanged>
	^self propertyValueAt: #stream! !

!CTranscriptView methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
stream: aValue
	"Modify the receiver's stream"
	^self propertyValueAt: #stream put: aValue with: #streamChanged! !

CCodeHolder subclass: #CWorkspace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CWorkspace commentStamp: 'tk 3/10/2005 16:49' prior: 0!
A Workspace is a window that lets the user can type simple commands in Tweak.  Type an expression, and choose 'do it' or 'print it' from the right button (or Command) menu.  Text in the Workspace is not saved.  It is tossed out when the window is closed.  Get a Workspace from the Tools menu in the main menu bar.[general]


### Allow Users to see the above explanation ###!

CWorkspace defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="bindings" toSet="bindings:" changeEvent="bindingsChanged">bindings</field>
</fields>'!

!CWorkspace class methodsFor: 'class initialization' stamp: 'ar 1/22/2005 22:46'!
initialize
	"CWorkspace initialize"
	self registerOpenCommand.! !

!CWorkspace class methodsFor: 'class initialization' stamp: 'ar 1/22/2005 22:45'!
registerOpenCommand
	"CWorkspace initialize"
	Smalltalk at: #CProjectBuilder ifPresent:[:pb|
		pb registerOpenCommand: { 'Workspace' . { self . #open }. 'A Programmer Workspace' }.
	].! !

!CWorkspace methodsFor: 'bindings' stamp: 'ar 3/31/2004 15:18'!
bindingOf: aString
	bindings isNil
		ifTrue: [bindings := Dictionary new].
	(bindings includesKey: aString)
		ifFalse: [bindings at: aString put: nil].
	^bindings associationAt: aString! !

!CWorkspace methodsFor: 'accessing' stamp: 'ar 3/31/2004 15:18'!
bindings
	"Answer the bindings of the receiver"
	<bewareOf: #bindingsChanged>
	^self propertyValueAt: #bindings! !

!CWorkspace methodsFor: 'accessing' stamp: 'ar 3/31/2004 15:18'!
bindings: aValue
	"Modify the receiver's bindings"
	^self propertyValueAt: #bindings put: aValue with: #bindingsChanged! !

CObject subclass: #CDebugEventTracer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CDebugEventTracer commentStamp: 'ar 11/13/2004 18:08' prior: 0!
Instances of CDebugEventTracer are little helper objects raising a debugger when an event of the specified kind is signaled in the sender. It uses the generic event tracing mechanism to achieve this goal.!

CDebugEventTracer defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="signaler" toSet="signaler:" changeEvent="signalerChanged">signaler</field>
	<field toGet="eventName" toSet="eventName:" changeEvent="eventNameChanged">eventName</field>
</fields>'!

!CDebugEventTracer methodsFor: 'accessing' stamp: 'ar 11/13/2004 17:33'!
eventName
	"Answer the eventName of the receiver"
	<bewareOf: #eventNameChanged>
	^self propertyValueAt: #eventName! !

!CDebugEventTracer methodsFor: 'accessing' stamp: 'ar 11/13/2004 17:33'!
eventName: aValue
	"Modify the receiver's eventName"
	^self propertyValueAt: #eventName put: aValue with: #eventNameChanged! !

!CDebugEventTracer methodsFor: 'tracing' stamp: 'ar 11/14/2004 14:37'!
findFirstContextFrom: initialContext
	| spl ctx |
	spl := IdentitySet withAll: self specialEventMethods.
	ctx := initialContext.
	[spl includes: ctx method] whileTrue:[ctx := ctx sender].
	^ctx! !

!CDebugEventTracer methodsFor: 'testing' stamp: 'ar 11/13/2004 17:46'!
isEventTracer
	^true! !

!CDebugEventTracer methodsFor: 'initialize' stamp: 'ar 11/13/2004 18:09'!
register
	"Note: This needs to prevent the receiver from being GCed thus the use of #debugEventTracers"
	signaler on: eventName traceWith: self.
	(signaler userDataAt: #debugEventTracers ifAbsentPut:[IdentitySet new]) 
		add: self.! !

!CDebugEventTracer methodsFor: 'accessing' stamp: 'ar 11/13/2004 17:33'!
signaler
	"Answer the signaler of the receiver"
	<bewareOf: #signalerChanged>
	^self propertyValueAt: #signaler! !

!CDebugEventTracer methodsFor: 'accessing' stamp: 'ar 11/13/2004 17:33'!
signaler: aValue
	"Modify the receiver's signaler"
	^self propertyValueAt: #signaler put: aValue with: #signalerChanged! !

!CDebugEventTracer methodsFor: 'private' stamp: 'ar 11/14/2004 14:40'!
specialEventMethods
	"Answer the set of special event methods which are known to be less than interesting for the user and should therefore be skipped."
	^{
		Object>>#privateSignalEvent:.
		Object>>#signalEvent:.
		Object>>#signal:.
		Object>>#signal:with:.
		Object>>#signal:with:with:.
		Object>>#signal:with:with:with:.
		Object>>#signal:with:with:with:with:.
		Object>>#signal:withArguments:.
		Object>>#signalChanged:from:to:.
		CObject>>#propertyValueAt:put:with:.
		CPlayer>>#signalEvent:.
	}! !

!CDebugEventTracer methodsFor: 'initialize' stamp: 'ar 11/13/2004 18:04'!
unregister
	signaler on: eventName traceWith: nil.
	(signaler userDataAt: #debugEventTracers ifAbsentPut:[IdentitySet new]) 
		remove: self ifAbsent:[].! !

!CDebugEventTracer methodsFor: 'tracing' stamp: 'ar 11/14/2004 14:44'!
valueWithEvent: anEvent
	"Pop up the debugger and proceed so that we see the result of the event"
	| debugger script context |
	context := self findFirstContextFrom: thisContext sender.
	self unregister.
	script := Processor activeProcess.
	debugger := CDebugger new.
	debugger startScript:[debugger openNotifierContents: '' label: anEvent selector].
	debugger process: script context: context.
	script eventTracer: debugger.
	script traceEvents: true.
	script suspend.! !

!CObject methodsFor: '*Tweak-DevTools' stamp: 'ar 1/22/2005 14:43'!
inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."

	(self ifTweak: [CObjectInspector] ifNot: [Inspector]) openOn: self withEvalPane: true.! !

!ScriptProcess methodsFor: '*Tweak-DevTools' stamp: 'ar 5/18/2005 11:55'!
errorRecursion
	"Answer whether this process is in a potential error recursion"
	^(self valueOfProperty: #errorRecursion) == true! !

!ScriptProcess methodsFor: '*Tweak-DevTools' stamp: 'ar 5/18/2005 11:55'!
errorRecursion: aBool
	"Mark the receiver as in a potential error recursion"
	^self setProperty: #errorRecursion toValue: aBool! !

!Browser class methodsFor: '*Tweak-DevTools' stamp: 'tak 5/16/2005 15:40'!
addRecentClasses: className 
	className
		ifNil: [^ self].
	(RecentClasses includes: className)
		ifTrue: [RecentClasses remove: className].
	RecentClasses addFirst: className.
	RecentClasses size > 16
		ifTrue: [RecentClasses removeLast]! !

!Browser class methodsFor: '*Tweak-DevTools' stamp: 'tak 5/16/2005 15:28'!
recentClasses
	"self recentClasses inspect"
	^ RecentClasses! !

CPlayer subclass: #CPointerFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

CPointerFinder defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="goal" toSet="goal:" changeEvent="goalChanged">goal</field>
	<field virtual="true" serializable="false" toGet="window" toSet="window:">window</field>
	<field toGet="pointerList" toSet="pointerList:" changeEvent="pointerListChanged">pointerList</field>
</fields>'!

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 13:25'!
buildList: parents
	| todo obj parent object key objectList |
	todo := OrderedCollection new.
	obj := goal.

	[todo addFirst: obj.
	obj := parents at: obj ifAbsent: [].
	obj == nil] whileFalse.
	todo removeFirst.
	parent := Smalltalk.
	objectList := OrderedCollection new.
	pointerList removeAll.
	[todo isEmpty] whileFalse:[
		object := todo removeFirst.
		key := nil.
		(parent isKindOf: Dictionary) ifTrue: [
			todo size >= 2 ifTrue:[
				key := parent keyAtValue: todo second ifAbsent: [].
				key == nil ifFalse:[
					object := todo removeFirst; removeFirst.
					pointerList add: key printString , ' -> ' , object class name]]
		].
		key == nil ifTrue:[
			parent class == object ifTrue: [key := 'CLASS'].
			key == nil ifTrue: [
				1 to: parent class instSize do: [:i | 
					key == nil ifTrue: [
						(parent instVarAt: i) == object ifTrue: [
							key := parent class allInstVarNames at: i]]]].
			key == nil ifTrue: [
				1 to: parent basicSize do: [:i | 
					key == nil ifTrue: [
						(parent basicAt: i) == object ifTrue: [
							key := i printString]]]].
			key == nil ifTrue: [
				(parent isMorph and: [object isKindOf: Array]) ifTrue: [
					key := 'submorphs?'
				]
			].
			key == nil ifTrue: [
				(parent class == CompiledMethod) ifTrue: [
					key := 'literal'
				]
			].
			key == nil ifTrue: [key := '???'].
			pointerList add: key , ': ' , object class name
		].
		objectList add: object.
		parent := object
	].
! !

!CPointerFinder methodsFor: 'initialize' stamp: 'ar 12/22/2004 19:43'!
defaultCostume
	^CWindow! !

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 13:23'!
follow: anObject from: parentObject parents: parents todo: todo
	anObject == goal ifTrue:[
		parents at: anObject put: parentObject.
		^true].
	anObject isLiteral ifTrue: [^ false].
	anObject class == CompiledMethod ifFalse:[
		anObject class isPointers ifFalse: [^ false].
		anObject class isWeak ifTrue: [^ false].
	].
	(parents includesKey: anObject)
		ifTrue: [^ false].
	parents at: anObject put: parentObject.
	todo add: anObject.
	^ false! !

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 13:24'!
followObject: anObject parents: parents todo: todo
	(self follow: anObject class from: anObject parents: parents todo: todo)
		ifTrue: [^ true].
	anObject class == CompiledMethod ifTrue:[
		1 to: anObject numLiterals do:[:i|
			(self follow: (anObject literalAt: i) from: anObject parents: parents todo: todo)
				ifTrue: [^ true]].
	].
	1 to: anObject class instSize do:
		[:i |
		(self follow: (anObject instVarAt: i) from: anObject parents: parents todo: todo)
			ifTrue: [^ true]].
	1 to: anObject basicSize do:
		[:i |
		(self follow: (anObject basicAt: i) from: anObject parents: parents todo: todo)
			ifTrue: [^ true]].
	^ false! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 11/24/2004 11:35'!
goal
	"Answer the goal of the receiver"
	<bewareOf: #goalChanged>
	^self propertyValueAt: #goal! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 11/24/2004 11:35'!
goal: aValue
	"Modify the receiver's goal"
	^self propertyValueAt: #goal put: aValue with: #goalChanged! !

!CPointerFinder methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:57'!
initialize
	super initialize.
	self define: #pointerList as: CStringList new.
! !

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 11:34'!
isLiteral
	"Horrible hack to omit other Pointer Finders from scanning."

	^ true! !

!CPointerFinder methodsFor: 'events' stamp: 'ar 11/24/2004 11:39'!
onWindowColorChanged: newColor
	<on: windowColorChanged in: window>
	window elements do:[:each| each windowColor: newColor].! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 11/24/2004 12:04'!
pointerList
	"Answer the pointerList of the receiver"
	<bewareOf: #pointerListChanged>
	^self propertyValueAt: #pointerList! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 11/24/2004 12:04'!
pointerList: aValue
	"Modify the receiver's pointerList"
	^self propertyValueAt: #pointerList put: aValue with: #pointerListChanged! !

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 12:03'!
search
	| parents todo todoNext |
	Smalltalk garbageCollect.
	parents := IdentityDictionary new: 20000.
	parents at: Smalltalk put: nil.
	parents at: Processor put: nil.
	parents at: self put: nil.

	todo := OrderedCollection new: 5000.
	todo add: Smalltalk.
	todoNext := OrderedCollection new: 5000.
	Cursor wait showWhile: [
		[[todo isEmpty or: [self followObject: todo removeFirst parents: parents todo: todoNext]] whileFalse.
		todo isEmpty and: [todoNext isEmpty not]]
			whileTrue: 
				[todo := todoNext.
				todoNext := OrderedCollection new: 5000]].

	self buildList: parents.! !

!CPointerFinder methodsFor: 'application' stamp: 'ar 11/24/2004 12:04'!
searchAgain
	pointerList cursor: 0.
	self search.! !

!CPointerFinder methodsFor: 'initialize' stamp: 'ar 4/24/2005 17:55'!
setupCostume
	super setupCostume.
	self extent: 200@200.
	self layout: CProportionalLayout new.

	pointerList color: Color transparent.
	pointerList layoutFrame: (0@0 corner: 1@1).
	pointerList hResizing: #spaceFill; vResizing: #spaceFill.
	pointerList hScrollable: false; vScrollable: true.

	window add: pointerList.

	"Give all of my contents the default look"
	window decorate: window elements color: Color magenta.
	window label: self class name.! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 12/22/2004 19:43'!
window
	"Answer the window of the receiver"
	^costume! !

!CPointerFinder methodsFor: 'accessing' stamp: 'ar 12/22/2004 19:44'!
window: aWindow
	"Modify the receiver's window"
	^costume := aWindow! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 4/28/2005 10:53'!
debuggerClass
	^self ifTweak:[CDebugger] ifNot:[Debugger]! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 5/18/2005 11:54'!
errorRecursion
	"Answer whether this process is in a potential error recursion"
	^false! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 5/18/2005 11:55'!
errorRecursion: aBool
	"Mark the receiver as in a potential error recursion"
	^nil! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 4/28/2005 10:53'!
interruptName: labelString
	"Open debugger on self"
	self debuggerClass openInterrupt: labelString onProcess: self! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 6/18/2005 15:56'!
topContext
	^nil! !

!Process methodsFor: '*Tweak-DevTools' stamp: 'ar 6/18/2005 15:58'!
topContext: ignored
	^nil
! !

!Stream methodsFor: '*Tweak-DevTools-ParseBackedText' stamp: 'tk 4/15/2005 12:50'!
attributeForNode: aNode toHereFrom: start
	"Install a text attribute pointing at the parse tree node from start to present position."
	"But, only if we are a CDialectStream"
! !

!Stream methodsFor: '*Tweak-DevTools-ParseBackedText' stamp: 'bf 7/19/2005 16:14'!
beginNode: aNode
	"Allow subclasses to put attributes on everything in a block or compound message call"! !

!Stream methodsFor: '*Tweak-DevTools-ParseBackedText' stamp: 'bf 7/19/2005 16:14'!
endNode: aNode
	"Allow subclasses to put attributes on everything in a block or compound message call"! !

Object subclass: #CAnnotationsProperty
	instanceVariableNames: 'property values methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CAnnotationsProperty class methodsFor: 'instance creation' stamp: 'rhi 1/10/2005 16:02'!
new

	^ super new initialize! !

!CAnnotationsProperty methodsFor: 'initialization' stamp: 'rhi 1/10/2005 16:04'!
initialize

	self
		values: Set new;
		methods: Set new.! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 15:52'!
methods
	"^ <Set of: MethodReference>"

	^ methods! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 15:52'!
methods: aSet

	methods := aSet.! !

!CAnnotationsProperty methodsFor: 'printing' stamp: 'rhi 1/14/2005 23:01'!
printOn: aStream

	self flag: #rhi. "none vs. asString vs. printString"
	aStream nextPutAll: self property printString,
		' (', self values size printString, ', ', self methods size printString, ')'.! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 15:50'!
property
	"^ <Symbol>"

	^ property! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 15:50'!
property: aSymbol

	property := aSymbol.! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 17:37'!
values
	"^ <Set of: Object>"

	^ values! !

!CAnnotationsProperty methodsFor: 'accessing' stamp: 'rhi 1/10/2005 15:52'!
values: aSet

	values := aSet.! !

!CTextEditor methodsFor: '*Tweak-DevTools-ParseBackedText' stamp: 'tk 4/19/2005 11:08'!
showDropZones: aTile
	"The user dragged in a tile.  If we are showing ParseTree Backed Text, highlight the drop zones for this kind of tile.  Know we are in #tree mode."

	aTile signal: #showDropZone with: self. 
	"Transcript show: aTile class printString."! !

WriteStream subclass: #CTranscriptStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-DevTools'!

!CTranscriptStream commentStamp: 'tk 2/28/2005 14:02' prior: 0!
The stream of a Transcript.!

!CTranscriptStream class methodsFor: 'instance creation' stamp: 'ar 11/23/2004 16:38'!
new
	^ self on: (String new: 1000)! !

!CTranscriptStream class methodsFor: 'instance creation' stamp: 'ar 11/23/2004 16:38'!
newTranscript: aTextCollector 
	"Store aTextCollector as the value of the system global Transcript."
	Smalltalk at: #Transcript put: aTextCollector! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:28'!
characterLimit
	"Tell the views how much to retain on screen"
	^ 20000! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
clear
	"Clear all characters and redisplay the view"
	self reset.
	self endEntry.! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:48'!
endEntry
	"Display all the characters since the last endEntry, and reset the stream"
	self signal: #appendEntry with: self contents.
	self reset.! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
flush
	self endEntry! !

!CTranscriptStream methodsFor: 'compatibility' stamp: 'ar 6/25/2005 13:41'!
openAsMorphLabel: label
	| prior |
	prior := Project current transcript.
	prior == self ifTrue:[prior := nil].
	TranscriptStream newTranscript: (prior ifNil:[TranscriptStream new]).
	^Transcript openAsMorphLabel: label! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
pastEndPut: anObject
	"If the stream reaches its limit, just output the contents and reset."
	self endEntry.
	^ self nextPut: anObject! !

!CTranscriptStream methodsFor: 'accessing' stamp: 'ar 11/23/2004 16:31'!
show: anObject  "TextCollector compatibility"
	self nextPutAll: anObject asString; endEntry! !

!TextAttribute methodsFor: '*Tweak-DevTools-ParseBackedText' stamp: 'tk 4/19/2005 11:30'!
dropAction
	"Answer the selector for the action to perform when the user drops an object on a piece of text.  This is a general way of allowing each piced of text to decide what happens when an object is dropped on it."
	^ nil! !
CAnnotationsBrowser initialize!
CChangeSorter initialize!
CClassBrowser initialize!
CDebugger initialize!
CTranscriptView initialize!
CWorkspace initialize!
